@@ -1,6 +1,6 @@
 <?php
-declare(strict_types = 1);
-namespace TYPO3\CMS\Backend\Authentication\Event;
+
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -15,32 +15,18 @@ namespace TYPO3\CMS\Backend\Authentication\Event;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Backend\Authentication\Event;
+
 /**
  * This event is triggered when a "SU" (switch user) action has been triggered
  */
-final class SwitchUserEvent
+final readonly class SwitchUserEvent
 {
-    /**
-     * @var string
-     */
-    private $sessionId;
-
-    /**
-     * @var array
-     */
-    private $targetUser;
-
-    /**
-     * @var array
-     */
-    private $currentUser;
-
-    public function __construct(string $sessionId, array $targetUser, array $currentUser)
-    {
-        $this->sessionId = $sessionId;
-        $this->targetUser = $targetUser;
-        $this->currentUser = $currentUser;
-    }
+    public function __construct(
+        private string $sessionId,
+        private array $targetUser,
+        private array $currentUser
+    ) {}
 
     public function getSessionId(): string
     {
