@@ -30,8 +30,6 @@ class FlashMessageController
 {
     /**
      * Renders the FlashMessages from queue and returns them as JSON.
-     *
-     * @return ResponseInterface
      */
     public function getQueuedFlashMessagesAction(): ResponseInterface
     {
@@ -44,10 +42,10 @@ class FlashMessageController
             $messages[] = [
                 'title' => $flashMessage->getTitle(),
                 'message' => $flashMessage->getMessage(),
-                'severity' => $flashMessage->getSeverity()
+                'severity' => $flashMessage->getSeverity(),
             ];
         }
 
-        return (new JsonResponse())->setPayload($messages);
+        return new JsonResponse($messages);
     }
 }
