@@ -19,15 +19,18 @@ namespace TYPO3\CMS\Backend\Controller;
 
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
+use TYPO3\CMS\Backend\Attribute\AsController;
 use TYPO3\CMS\Backend\Form\FormDataCompiler;
 use TYPO3\CMS\Backend\Form\FormDataGroup\TcaDatabaseRecord;
 use TYPO3\CMS\Backend\Form\InlineStackProcessor;
 use TYPO3\CMS\Backend\Form\NodeFactory;
 use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
+use TYPO3\CMS\Core\Crypto\HashService;
 use TYPO3\CMS\Core\DataHandling\DataHandler;
 use TYPO3\CMS\Core\Http\JsonResponse;
-use TYPO3\CMS\Core\Messaging\AbstractMessage;
 use TYPO3\CMS\Core\Messaging\FlashMessageService;
+use TYPO3\CMS\Core\Page\JavaScriptItems;
+use TYPO3\CMS\Core\Type\ContextualFeedbackSeverity;
 use TYPO3\CMS\Core\Utility\ArrayUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
@@ -35,22 +38,28 @@ use TYPO3\CMS\Core\Utility\MathUtility;
 /**
  * Handle FormEngine inline ajax calls
  */
+#[AsController]
 class FormInlineAjaxController extends AbstractFormEngineAjaxController
 {
+    public function __construct(
+        private readonly FormDataCompiler $formDataCompiler,
+        private readonly HashService $hashService,
+        private readonly NodeFactory $nodeFactory,
+    ) {}
+
     /**
      * Create a new inline child via AJAX.
-     *
-     * @param ServerRequestInterface $request
-     * @return ResponseInterface
      */
     public function createAction(ServerRequestInterface $request): ResponseInterface
     {
         $ajaxArguments = $request->getParsedBody()['ajax'] ?? $request->getQueryParams()['ajax'];
         $parentConfig = $this->extractSignedParentConfigFromRequest((string)$ajaxArguments['context']);
 
-        $domObjectId = $ajaxArguments[0];
+        $domObjectId = $ajaxArguments[0] ?? '';
         $inlineFirstPid = $this->getInlineFirstPidFromDomObjectId($domObjectId);
-        if (!MathUtility::canBeInterpretedAsInteger($inlineFirstPid) && strpos((string)$inlineFirstPid, 'NEW') !== 0) {
+        if (!MathUtility::canBeInterpretedAsInteger($inlineFirstPid)
+            && !str_starts_with((string)$inlineFirstPid, 'NEW')
+        ) {
             throw new \RuntimeException(
                 'inlineFirstPid should either be an integer or a "NEW..." string',
                 1521220491
@@ -62,10 +71,9 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         }
 
         // Parse the DOM identifier, add the levels to the structure stack
-        /** @var InlineStackProcessor $inlineStackProcessor */
         $inlineStackProcessor = GeneralUtility::makeInstance(InlineStackProcessor::class);
         $inlineStackProcessor->initializeByParsingDomObjectIdString($domObjectId);
-        $inlineStackProcessor->injectAjaxConfiguration($parentConfig);
+        $inlineStackProcessor->setAjaxConfiguration($parentConfig);
         $inlineTopMostParent = $inlineStackProcessor->getStructureLevel(0);
 
         // Parent, this table embeds the child table
@@ -73,7 +81,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
 
         // Child, a record from this table should be rendered
         $child = $inlineStackProcessor->getUnstableStructure();
-        if (MathUtility::canBeInterpretedAsInteger($child['uid'])) {
+        if (isset($child['uid']) && MathUtility::canBeInterpretedAsInteger($child['uid'])) {
             // If uid comes in, it is the id of the record neighbor record "create after"
             $childVanillaUid = -1 * abs((int)$child['uid']);
         } else {
@@ -83,11 +91,8 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
 
         $childTableName = $parentConfig['foreign_table'];
 
-        /** @var TcaDatabaseRecord $formDataGroup */
-        $formDataGroup = GeneralUtility::makeInstance(TcaDatabaseRecord::class);
-        /** @var FormDataCompiler $formDataCompiler */
-        $formDataCompiler = GeneralUtility::makeInstance(FormDataCompiler::class, $formDataGroup);
         $formDataCompilerInput = [
+            'request' => $request,
             'command' => 'new',
             'tableName' => $childTableName,
             'vanillaUid' => $childVanillaUid,
@@ -105,9 +110,9 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         if ($childChildUid) {
             $formDataCompilerInput['inlineChildChildUid'] = $childChildUid;
         }
-        $childData = $formDataCompiler->compile($formDataCompilerInput);
+        $childData = $this->formDataCompiler->compile($formDataCompilerInput, GeneralUtility::makeInstance(TcaDatabaseRecord::class));
 
-        if ($parentConfig['foreign_selector'] && $parentConfig['appearance']['useCombination']) {
+        if (($parentConfig['foreign_selector'] ?? false) && ($parentConfig['appearance']['useCombination'] ?? false)) {
             // We have a foreign_selector. So, we just created a new record on an intermediate table in $childData.
             // Now, if a valid id is given as second ajax parameter, the intermediate row should be connected to an
             // existing record of the child-child table specified by the given uid. If there is no such id, user
@@ -117,34 +122,30 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
                 $childData['databaseRow'][$parentConfig['foreign_selector']] = [
                     $childChildUid,
                 ];
-                $childData['combinationChild'] = $this->compileChildChild($childData, $parentConfig, $inlineStackProcessor->getStructure());
+                $childData['combinationChild'] = $this->compileChildChild($request, $childData, $parentConfig, $inlineStackProcessor->getStructure());
             } else {
-                /** @var TcaDatabaseRecord $formDataGroup */
-                $formDataGroup = GeneralUtility::makeInstance(TcaDatabaseRecord::class);
-                /** @var FormDataCompiler $formDataCompiler */
-                $formDataCompiler = GeneralUtility::makeInstance(FormDataCompiler::class, $formDataGroup);
                 $formDataCompilerInput = [
+                    'request' => $request,
                     'command' => 'new',
-                    'tableName' => $childData['processedTca']['columns'][$parentConfig['foreign_selector']]['config']['foreign_table'],
+                    'tableName' => $this->getChildChildTableName($parentConfig['foreign_selector'], $childData),
                     'vanillaUid' => $inlineFirstPid,
                     'isInlineChild' => true,
                     'isInlineAjaxOpeningContext' => true,
                     'inlineStructure' => $inlineStackProcessor->getStructure(),
                     'inlineFirstPid' => $inlineFirstPid,
                 ];
-                $childData['combinationChild'] = $formDataCompiler->compile($formDataCompilerInput);
+                $childData['combinationChild'] = $this->formDataCompiler->compile($formDataCompilerInput, GeneralUtility::makeInstance(TcaDatabaseRecord::class));
             }
         }
 
         $childData['inlineParentUid'] = $parent['uid'];
         $childData['renderType'] = 'inlineRecordContainer';
-        $nodeFactory = GeneralUtility::makeInstance(NodeFactory::class);
-        $childResult = $nodeFactory->create($childData)->render();
+        $childResult = $this->nodeFactory->create($childData)->render();
 
         $jsonArray = [
             'data' => '',
             'stylesheetFiles' => [],
-            'scriptCall' => [],
+            'scriptItems' => new JavaScriptItems(),
             'compilerInput' => [
                 'uid' => $childData['databaseRow']['uid'],
                 'childChildUid' => $childChildUid,
@@ -159,23 +160,19 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
 
     /**
      * Show the details of a child record.
-     *
-     * @param ServerRequestInterface $request
-     * @return ResponseInterface
      */
     public function detailsAction(ServerRequestInterface $request): ResponseInterface
     {
         $ajaxArguments = $request->getParsedBody()['ajax'] ?? $request->getQueryParams()['ajax'];
 
-        $domObjectId = $ajaxArguments[0] ?? null;
+        $domObjectId = $ajaxArguments[0] ?? '';
         $inlineFirstPid = $this->getInlineFirstPidFromDomObjectId($domObjectId);
         $parentConfig = $this->extractSignedParentConfigFromRequest((string)$ajaxArguments['context']);
 
         // Parse the DOM identifier, add the levels to the structure stack
-        /** @var InlineStackProcessor $inlineStackProcessor */
         $inlineStackProcessor = GeneralUtility::makeInstance(InlineStackProcessor::class);
         $inlineStackProcessor->initializeByParsingDomObjectIdString($domObjectId);
-        $inlineStackProcessor->injectAjaxConfiguration($parentConfig);
+        $inlineStackProcessor->setAjaxConfiguration($parentConfig);
 
         // Parent, this table embeds the child table
         $parent = $inlineStackProcessor->getStructureLevel(-1);
@@ -206,17 +203,16 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         // Child, a record from this table should be rendered
         $child = $inlineStackProcessor->getUnstableStructure();
 
-        $childData = $this->compileChild($parentData, $parentFieldName, (int)$child['uid'], $inlineStackProcessor->getStructure());
+        $childData = $this->compileChild($request, $parentData, $parentFieldName, (int)$child['uid'], $inlineStackProcessor->getStructure());
 
         $childData['inlineParentUid'] = (int)$parent['uid'];
         $childData['renderType'] = 'inlineRecordContainer';
-        $nodeFactory = GeneralUtility::makeInstance(NodeFactory::class);
-        $childResult = $nodeFactory->create($childData)->render();
+        $childResult = $this->nodeFactory->create($childData)->render();
 
         $jsonArray = [
             'data' => '',
             'stylesheetFiles' => [],
-            'scriptCall' => [],
+            'scriptItems' => new JavaScriptItems(),
         ];
 
         $jsonArray = $this->mergeChildResultIntoJsonResult($jsonArray, $childResult);
@@ -234,20 +230,20 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
     public function synchronizeLocalizeAction(ServerRequestInterface $request): ResponseInterface
     {
         $ajaxArguments = $request->getParsedBody()['ajax'] ?? $request->getQueryParams()['ajax'];
-        $domObjectId = $ajaxArguments[0] ?? null;
+        $domObjectId = $ajaxArguments[0] ?? '';
         $type = $ajaxArguments[1] ?? null;
         $parentConfig = $this->extractSignedParentConfigFromRequest((string)$ajaxArguments['context']);
 
-        /** @var InlineStackProcessor $inlineStackProcessor */
         $inlineStackProcessor = GeneralUtility::makeInstance(InlineStackProcessor::class);
         // Parse the DOM identifier (string), add the levels to the structure stack (array), load the TCA config:
         $inlineStackProcessor->initializeByParsingDomObjectIdString($domObjectId);
-        $inlineStackProcessor->injectAjaxConfiguration($parentConfig);
+        $inlineStackProcessor->setAjaxConfiguration($parentConfig);
         $inlineFirstPid = $this->getInlineFirstPidFromDomObjectId($domObjectId);
 
         $jsonArray = [
             'data' => '',
             'stylesheetFiles' => [],
+            'scriptItems' => new JavaScriptItems(),
             'compilerInput' => [
                 'localize' => [],
             ],
@@ -261,13 +257,14 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
             $processedTca['columns'][$parentFieldName]['config'] = $parentConfig;
 
             $formDataCompilerInputForParent = [
+                'request' => $request,
                 'vanillaUid' => (int)$parent['uid'],
                 'command' => 'edit',
                 'tableName' => $parent['table'],
                 'processedTca' => $processedTca,
                 'inlineFirstPid' => $inlineFirstPid,
                 'columnsToProcess' => [
-                    $parentFieldName
+                    $parentFieldName,
                 ],
                 // @todo: still needed? NO!
                 'inlineStructure' => $inlineStackProcessor->getStructure(),
@@ -275,11 +272,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
                 'inlineCompileExistingChildren' => false,
             ];
             // Full TcaDatabaseRecord is required here to have the list of connected uids $oldItemList
-            /** @var TcaDatabaseRecord $formDataGroup */
-            $formDataGroup = GeneralUtility::makeInstance(TcaDatabaseRecord::class);
-            /** @var FormDataCompiler $formDataCompiler */
-            $formDataCompiler = GeneralUtility::makeInstance(FormDataCompiler::class, $formDataGroup);
-            $parentData = $formDataCompiler->compile($formDataCompilerInputForParent);
+            $parentData = $this->formDataCompiler->compile($formDataCompilerInputForParent, GeneralUtility::makeInstance(TcaDatabaseRecord::class));
             $parentConfig = $parentData['processedTca']['columns'][$parentFieldName]['config'];
             $parentLanguageField = $parentData['processedTca']['ctrl']['languageField'];
             $parentLanguage = $parentData['databaseRow'][$parentLanguageField];
@@ -307,7 +300,6 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
                 ];
             }
 
-            /** @var DataHandler $tce */
             $tce = GeneralUtility::makeInstance(DataHandler::class);
             $tce->start([], $cmd);
             $tce->process_cmdmap();
@@ -326,9 +318,9 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
                     $jsonArray['messages'][] = [
                         'title'    => $message->getTitle(),
                         'message'  => $message->getMessage(),
-                        'severity' => $message->getSeverity()
+                        'severity' => $message->getSeverity(),
                     ];
-                    if ($message->getSeverity() === AbstractMessage::ERROR) {
+                    if ($message->getSeverity() === ContextualFeedbackSeverity::ERROR) {
                         $jsonArray['hasErrors'] = true;
                     }
                 }
@@ -340,18 +332,17 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
 
             $localizedItems = array_diff($newItems, $oldItems);
             foreach ($localizedItems as $i => $childUid) {
-                $childData = $this->compileChild($parentData, $parentFieldName, (int)$childUid, $inlineStackProcessor->getStructure());
+                $childData = $this->compileChild($request, $parentData, $parentFieldName, (int)$childUid, $inlineStackProcessor->getStructure());
 
                 $childData['inlineParentUid'] = (int)$parent['uid'];
                 $childData['renderType'] = 'inlineRecordContainer';
-                $nodeFactory = GeneralUtility::makeInstance(NodeFactory::class);
-                $childResult = $nodeFactory->create($childData)->render();
+                $childResult = $this->nodeFactory->create($childData)->render();
 
                 $jsonArray = $this->mergeChildResultIntoJsonResult($jsonArray, $childResult);
 
                 // Get the name of the field used as foreign selector (if any):
                 $foreignSelector = isset($parentConfig['foreign_selector']) && $parentConfig['foreign_selector'] ? $parentConfig['foreign_selector'] : false;
-                $selectedValue = $foreignSelector ? GeneralUtility::quoteJSvalue($childData['databaseRow'][$foreignSelector]) : null;
+                $selectedValue = $foreignSelector ? $childData['databaseRow'][$foreignSelector] : null;
                 if (is_array($selectedValue)) {
                     $selectedValue = $selectedValue[0];
                 }
@@ -369,11 +360,11 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
                         $transOrigPointerFieldValue = $transOrigPointerFieldValue[0];
                         if (is_array($transOrigPointerFieldValue) && ($transOrigPointerFieldValue['uid'] ?? false)) {
                             // With nested inline containers (eg. fal sys_file_reference), row[l10n_parent][0] is sometimes
-                            // a table / row combination again. See tx_styleguide_inline_fal inline_5. If this happens we
+                            // a table / row combination again. See tx_styleguide_file file_5. If this happens we
                             // pick the uid field from the array ... Basically, we need the uid of the 'default language' record,
                             // since this is used in JS to locate and remove the 'shadowed' container.
                             // @todo: Find out if this is really necessary that sometimes ['databaseRow']['l10n_parent'][0]
-                            // is resolved to a direct uid, and sometimes it's a array with items. Could this be harmonized?
+                            // is resolved to a direct uid, and sometimes it's an array with items. Could this be harmonized?
                             $transOrigPointerFieldValue = $transOrigPointerFieldValue['uid'];
                         }
                     }
@@ -395,7 +386,6 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         $ajaxArguments = $request->getParsedBody()['ajax'] ?? $request->getQueryParams()['ajax'];
         [$domObjectId, $expand, $collapse] = $ajaxArguments;
 
-        /** @var InlineStackProcessor $inlineStackProcessor */
         $inlineStackProcessor = GeneralUtility::makeInstance(InlineStackProcessor::class);
         // Parse the DOM identifier (string), add the levels to the structure stack (array), don't load TCA config
         $inlineStackProcessor->initializeByParsingDomObjectIdString($domObjectId);
@@ -428,7 +418,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
                 $backendUser->writeUC();
             }
         }
-        return (new JsonResponse())->setPayload([]);
+        return new JsonResponse([]);
     }
 
     /**
@@ -443,11 +433,10 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
      * @todo: This clones methods compileChild from TcaInline Provider. Find a better abstraction
      * @todo: to also encapsulate the more complex scenarios with combination child and friends.
      */
-    protected function compileChild(array $parentData, $parentFieldName, $childUid, array $inlineStructure)
+    protected function compileChild(ServerRequestInterface $request, array $parentData, $parentFieldName, $childUid, array $inlineStructure)
     {
         $parentConfig = $parentData['processedTca']['columns'][$parentFieldName]['config'];
 
-        /** @var InlineStackProcessor $inlineStackProcessor */
         $inlineStackProcessor = GeneralUtility::makeInstance(InlineStackProcessor::class);
         $inlineStackProcessor->initializeByGivenStructure($inlineStructure);
         $inlineTopMostParent = $inlineStackProcessor->getStructureLevel(0);
@@ -456,11 +445,8 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         $child = $inlineStackProcessor->getUnstableStructure();
         $childTableName = $child['table'];
 
-        /** @var TcaDatabaseRecord $formDataGroup */
-        $formDataGroup = GeneralUtility::makeInstance(TcaDatabaseRecord::class);
-        /** @var FormDataCompiler $formDataCompiler */
-        $formDataCompiler = GeneralUtility::makeInstance(FormDataCompiler::class, $formDataGroup);
         $formDataCompilerInput = [
+            'request' => $request,
             'command' => 'edit',
             'tableName' => $childTableName,
             'vanillaUid' => (int)$childUid,
@@ -485,10 +471,10 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         // For foreign_selector with useCombination $mainChild is the mm record
         // and $combinationChild is the child-child. For "normal" relations, $mainChild
         // is just the normal child record and $combinationChild is empty.
-        $mainChild = $formDataCompiler->compile($formDataCompilerInput);
-        if ($parentConfig['foreign_selector'] && $parentConfig['appearance']['useCombination']) {
+        $mainChild = $this->formDataCompiler->compile($formDataCompilerInput, GeneralUtility::makeInstance(TcaDatabaseRecord::class));
+        if (($parentConfig['foreign_selector'] ?? false) && ($parentConfig['appearance']['useCombination'] ?? false)) {
             // This kicks in if opening an existing mainChild that has a child-child set
-            $mainChild['combinationChild'] = $this->compileChildChild($mainChild, $parentConfig, $inlineStructure);
+            $mainChild['combinationChild'] = $this->compileChildChild($request, $mainChild, $parentConfig, $inlineStructure);
         }
         return $mainChild;
     }
@@ -502,19 +488,14 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
      * @param array $inlineStructure Current inline structure
      * @return array Full data array of child
      */
-    protected function compileChildChild(array $child, array $parentConfig, array $inlineStructure)
+    protected function compileChildChild(ServerRequestInterface $request, array $child, array $parentConfig, array $inlineStructure)
     {
         // foreign_selector on intermediate is probably type=select, so data provider of this table resolved that to the uid already
         $childChildUid = $child['databaseRow'][$parentConfig['foreign_selector']][0];
-        // child-child table name is set in child tca "the selector field" foreign_table
-        $childChildTableName = $child['processedTca']['columns'][$parentConfig['foreign_selector']]['config']['foreign_table'];
-        /** @var TcaDatabaseRecord $formDataGroup */
-        $formDataGroup = GeneralUtility::makeInstance(TcaDatabaseRecord::class);
-        /** @var FormDataCompiler $formDataCompiler */
-        $formDataCompiler = GeneralUtility::makeInstance(FormDataCompiler::class, $formDataGroup);
         $formDataCompilerInput = [
+            'request' => $request,
             'command' => 'edit',
-            'tableName' => $childChildTableName,
+            'tableName' => $this->getChildChildTableName($parentConfig['foreign_selector'] ?? '', $child),
             'vanillaUid' => (int)$childChildUid,
             'isInlineChild' => true,
             'isInlineAjaxOpeningContext' => true,
@@ -526,7 +507,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
             'inlineTopMostParentTableName' => $child['inlineTopMostParentTableName'],
             'inlineTopMostParentFieldName' => $child['inlineTopMostParentFieldName'],
         ];
-        return $formDataCompiler->compile($formDataCompilerInput);
+        return $this->formDataCompiler->compile($formDataCompilerInput, GeneralUtility::makeInstance(TcaDatabaseRecord::class));
     }
 
     /**
@@ -539,6 +520,9 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
      */
     protected function mergeChildResultIntoJsonResult(array $jsonResult, array $childResult)
     {
+        /** @var JavaScriptItems $scriptItems */
+        $scriptItems = $jsonResult['scriptItems'];
+
         $jsonResult['data'] .= $childResult['html'];
         $jsonResult['stylesheetFiles'] = [];
         foreach ($childResult['stylesheetFiles'] as $stylesheetFile) {
@@ -547,9 +531,6 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         if (!empty($childResult['inlineData'])) {
             $jsonResult['inlineData'] = $childResult['inlineData'];
         }
-        foreach ($childResult['additionalJavaScriptPost'] as $singleAdditionalJavaScriptPost) {
-            $jsonResult['scriptCall'][] = $singleAdditionalJavaScriptPost;
-        }
         if (!empty($childResult['additionalInlineLanguageLabelFiles'])) {
             $labels = [];
             foreach ($childResult['additionalInlineLanguageLabelFiles'] as $additionalInlineLanguageLabelFile) {
@@ -558,20 +539,9 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
                     $this->getLabelsFromLocalizationFile($additionalInlineLanguageLabelFile)
                 );
             }
-            $javaScriptCode = [];
-            $javaScriptCode[] = 'if (typeof TYPO3 === \'undefined\' || typeof TYPO3.lang === \'undefined\') {';
-            $javaScriptCode[] = '   TYPO3.lang = {}';
-            $javaScriptCode[] = '}';
-            $javaScriptCode[] = 'var additionalInlineLanguageLabels = ' . json_encode($labels) . ';';
-            $javaScriptCode[] = 'for (var attributeName in additionalInlineLanguageLabels) {';
-            $javaScriptCode[] = '   if (typeof TYPO3.lang[attributeName] === \'undefined\') {';
-            $javaScriptCode[] = '       TYPO3.lang[attributeName] = additionalInlineLanguageLabels[attributeName]';
-            $javaScriptCode[] = '   }';
-            $javaScriptCode[] = '}';
-
-            $jsonResult['scriptCall'][] = implode(LF, $javaScriptCode);
+            $scriptItems->addGlobalAssignment(['TYPO3' => ['lang' => $labels]]);
         }
-        $jsonResult['requireJsModules'] = $this->createExecutableStringRepresentationOfRegisteredRequireJsModules($childResult);
+        $this->addJavaScriptModulesToJavaScriptItems($childResult['javaScriptModules'] ?? [], $scriptItems);
 
         return $jsonResult;
     }
@@ -596,25 +566,6 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         return $itemArray;
     }
 
-    /**
-     * Return expand / collapse state array for a given table / uid combination
-     *
-     * @param string $table Handled table
-     * @param int $uid Handled uid
-     * @return array
-     */
-    protected function getInlineExpandCollapseStateArrayForTableUid($table, $uid)
-    {
-        $inlineView = $this->getInlineExpandCollapseStateArray();
-        $result = [];
-        if (MathUtility::canBeInterpretedAsInteger($uid)) {
-            if (!empty($inlineView[$table][$uid])) {
-                $result = $inlineView[$table][$uid];
-            }
-        }
-        return $result;
-    }
-
     /**
      * Get expand / collapse state of inline items
      *
@@ -639,7 +590,6 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
      * Method to check whether the backend user has the property inline view for the current IRRE item.
      * In existing or old IRRE items the attribute may not exist, then the json_decode will fail.
      *
-     * @param BackendUserAuthentication $backendUser
      * @return bool
      */
     protected function backendUserHasUcInlineView(BackendUserAuthentication $backendUser)
@@ -664,29 +614,13 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         return $haystack;
     }
 
-    /**
-     * Generates an error message that transferred as JSON for AJAX calls
-     *
-     * @param string $message The error message to be shown
-     * @return array The error message in a JSON array
-     */
-    protected function getErrorMessageForAJAX($message)
-    {
-        return [
-            'data' => $message,
-            'scriptCall' => [
-                'alert("' . $message . '");'
-            ],
-        ];
-    }
-
     /**
      * Get inlineFirstPid from a given objectId string
      *
      * @param string $domObjectId The id attribute of an element
      * @return int|string|null Pid or null
      */
-    protected function getInlineFirstPidFromDomObjectId($domObjectId)
+    protected function getInlineFirstPidFromDomObjectId(string $domObjectId)
     {
         // Substitute FlexForm addition and make parsing a bit easier
         $domObjectId = str_replace('---', ':', $domObjectId);
@@ -702,9 +636,7 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
      * Validates the config that is transferred over the wire to provide the
      * correct TCA config for the parent table
      *
-     * @param string $contextString
      * @throws \RuntimeException
-     * @return array
      */
     protected function extractSignedParentConfigFromRequest(string $contextString): array
     {
@@ -715,16 +647,29 @@ class FormInlineAjaxController extends AbstractFormEngineAjaxController
         if (empty($context['config'])) {
             throw new \RuntimeException('Empty context config section given', 1489751362);
         }
-        if (!hash_equals(GeneralUtility::hmac((string)$context['config'], 'InlineContext'), (string)$context['hmac'])) {
+        if (!hash_equals($this->hashService->hmac((string)$context['config'], 'InlineContext'), (string)$context['hmac'])) {
             throw new \RuntimeException('Hash does not validate', 1489751363);
         }
         return json_decode($context['config'], true);
     }
 
     /**
-     * @return BackendUserAuthentication
+     * The child-child table name is set in the child TCA "the selector field" and is depending on
+     * the TCA type (select or group) either the "foreign_table" or the (first) "allowed" table.
      */
-    protected function getBackendUserAuthentication()
+    protected function getChildChildTableName(string $foreignSelector, array $childConfiguration): string
+    {
+        $config = $childConfiguration['processedTca']['columns'][$foreignSelector]['config'] ?? [];
+        $type = $config['type'] ?? '';
+
+        return match ($type) {
+            'select' => $config['foreign_table'] ?? '',
+            'group' => GeneralUtility::trimExplode(',', $config['allowed'] ?? '', true)[0] ?? '',
+            default => '',
+        };
+    }
+
+    protected function getBackendUserAuthentication(): BackendUserAuthentication
     {
         return $GLOBALS['BE_USER'];
     }
