@@ -17,8 +17,11 @@ declare(strict_types=1);
 
 namespace TYPO3\CMS\Backend\Form\FormDataGroup;
 
+use Symfony\Component\DependencyInjection\Attribute\Autoconfigure;
+use Symfony\Component\DependencyInjection\Attribute\Autowire;
 use TYPO3\CMS\Backend\Form\FormDataGroupInterface;
 use TYPO3\CMS\Backend\Form\FormDataProviderInterface;
+use TYPO3\CMS\Core\Cache\Frontend\FrontendInterface;
 use TYPO3\CMS\Core\Service\DependencyOrderingService;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
@@ -28,44 +31,51 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
  * This group is used to call a list of providers in order by specified
  * dependencies before/depends.
  */
+#[Autoconfigure(public: true, shared: false)]
 class OrderedProviderList implements FormDataGroupInterface
 {
     /**
      * @var FormDataProviderInterface[]
      */
-    protected $providerList = [];
+    protected array $providerList = [];
+
+    public function __construct(
+        #[Autowire(service: 'cache.runtime')]
+        private readonly FrontendInterface $runtimeCache,
+        private readonly DependencyOrderingService $dependencyOrderingService,
+    ) {}
 
     /**
-     * Compile form data
-     *
      * @param array $result Initialized result array
      * @return array Result filled with data
-     * @throws \UnexpectedValueException
+     * @todo: compile() should receive $list from setProviderList() as argument to make this service
+     *        stateless and shared. It would be even better if the "static" lists from TYPO3_CONF_VARS
+     *        like 'tcaDatabaseRecord' could be provided as already ordered compile time service provider:
+     *        This would allow low lever caching and would avoid calls to DependencyOrderingService at runtime.
      */
     public function compile(array $result): array
     {
-        $orderingService = GeneralUtility::makeInstance(DependencyOrderingService::class);
-        $orderedDataProvider = $orderingService->orderByDependencies($this->providerList, 'before', 'depends');
-
-        foreach ($orderedDataProvider as $providerClassName => $providerConfig) {
+        $cacheIdentifier = 'FormEngine-OrderedProviderList-' . hash('xxh3', json_encode($this->providerList));
+        $orderedList = $this->runtimeCache->get($cacheIdentifier);
+        if (!is_array($orderedList)) {
+            $orderedList = $this->dependencyOrderingService->orderByDependencies($this->providerList, 'before', 'depends');
+            $this->runtimeCache->set($cacheIdentifier, $orderedList);
+        }
+        foreach ($orderedList as $providerClassName => $providerConfig) {
             if (isset($providerConfig['disabled']) && $providerConfig['disabled'] === true) {
                 // Skip this data provider if disabled by configuration
                 continue;
             }
-
+            if (!class_exists($providerClassName)) {
+                throw new \InvalidArgumentException('Implementation class for data provider ' . $providerClassName . ' does not exist', 1685542507);
+            }
             /** @var FormDataProviderInterface $provider */
             $provider = GeneralUtility::makeInstance($providerClassName);
-
             if (!$provider instanceof FormDataProviderInterface) {
-                throw new \UnexpectedValueException(
-                    'Data provider ' . $providerClassName . ' must implement FormDataProviderInterface',
-                    1485299408
-                );
+                throw new \UnexpectedValueException('Data provider ' . $providerClassName . ' must implement FormDataProviderInterface', 1485299408);
             }
-
             $result = $provider->addData($result);
         }
-
         return $result;
     }
 
@@ -80,9 +90,8 @@ class OrderedProviderList implements FormDataGroupInterface
      *   ]
      *
      * @param array $list Given list of Provider class names
-     * @see \TYPO3\CMS\Core\Service\DependencyOrderingService
      */
-    public function setProviderList(array $list)
+    public function setProviderList(array $list): void
     {
         $this->providerList = $list;
     }
