@@ -25,7 +25,6 @@ class TcaCheckboxItems extends AbstractItemProvider implements FormDataProviderI
     /**
      * Resolve checkbox items
      *
-     * @param array $result
      * @return array
      * @throws \UnexpectedValueException
      */
@@ -39,7 +38,7 @@ class TcaCheckboxItems extends AbstractItemProvider implements FormDataProviderI
                 continue;
             }
 
-            if (!is_array($fieldConfig['config']['items'])) {
+            if (!is_array($fieldConfig['config']['items'] ?? null)) {
                 $fieldConfig['config']['items'] = [];
             }
 
@@ -55,11 +54,11 @@ class TcaCheckboxItems extends AbstractItemProvider implements FormDataProviderI
 
             // Set label overrides from pageTsConfig if given
             if (isset($result['pageTsConfig']['TCEFORM.'][$table . '.'][$fieldName . '.']['altLabels.'])
-                && \is_array($result['pageTsConfig']['TCEFORM.'][$table . '.'][$fieldName . '.']['altLabels.'])
+                && is_array($result['pageTsConfig']['TCEFORM.'][$table . '.'][$fieldName . '.']['altLabels.'])
             ) {
                 foreach ($result['pageTsConfig']['TCEFORM.'][$table . '.'][$fieldName . '.']['altLabels.'] as $itemKey => $label) {
-                    if (isset($items[$itemKey][0])) {
-                        $items[$itemKey][0] = $languageService->sL($label);
+                    if (isset($items[$itemKey]['label'])) {
+                        $items[$itemKey]['label'] = $languageService->sL($label);
                     }
                 }
             }
@@ -71,9 +70,6 @@ class TcaCheckboxItems extends AbstractItemProvider implements FormDataProviderI
     }
 
     /**
-     * @param array $config
-     * @param string $fieldName
-     * @param string $tableName
      * @return array
      * @throws \UnexpectedValueException
      */
@@ -83,8 +79,7 @@ class TcaCheckboxItems extends AbstractItemProvider implements FormDataProviderI
         foreach ($config['items'] as $itemKey => $checkboxEntry) {
             $this->basicChecks($fieldName, $tableName, $checkboxEntry, $itemKey);
             $newItems[$itemKey] = [
-                $this->getLanguageService()->sL(trim($checkboxEntry[0])),
-                $checkboxEntry[1]
+                'label' => $this->getLanguageService()->sL(trim($checkboxEntry['label'])),
             ];
             if (isset($config['renderType']) && $config['renderType'] === 'checkboxToggle') {
                 $newItems = $this->sanitizeToggleCheckbox($checkboxEntry, $itemKey, $newItems);
@@ -98,21 +93,18 @@ class TcaCheckboxItems extends AbstractItemProvider implements FormDataProviderI
     }
 
     /**
-     * @param string $fieldName
-     * @param string $tableName
      * @param mixed $checkboxEntry
-     * @param int $checkboxKey
      * @throws \UnexpectedValueException
      */
     private function basicChecks(string $fieldName, string $tableName, $checkboxEntry, int $checkboxKey)
     {
-        if (!\is_array($checkboxEntry)) {
+        if (!is_array($checkboxEntry)) {
             throw new \UnexpectedValueException(
                 'Item ' . $checkboxKey . ' of field ' . $fieldName . ' of TCA table ' . $tableName . ' is not an array as expected',
                 1440499337
             );
         }
-        if (!\array_key_exists(0, $checkboxEntry)) {
+        if (!array_key_exists('label', $checkboxEntry)) {
             throw new \UnexpectedValueException(
                 'Item ' . $checkboxKey . ' of field ' . $fieldName . ' of TCA table ' . $tableName . ' has no label',
                 1440499338
@@ -121,9 +113,6 @@ class TcaCheckboxItems extends AbstractItemProvider implements FormDataProviderI
     }
 
     /**
-     * @param array $item
-     * @param int $itemKey
-     * @param array $newItems
      * @return array
      */
     private function sanitizeToggleCheckbox(array $item, int $itemKey, array $newItems)
@@ -137,9 +126,6 @@ class TcaCheckboxItems extends AbstractItemProvider implements FormDataProviderI
     }
 
     /**
-     * @param array $item
-     * @param int $itemKey
-     * @param array $newItems
      * @return array
      */
     private function sanitizeLabeledToggleCheckbox(array $item, int $itemKey, array $newItems)
@@ -159,9 +145,6 @@ class TcaCheckboxItems extends AbstractItemProvider implements FormDataProviderI
     }
 
     /**
-     * @param array $item
-     * @param int $itemKey
-     * @param array $newItems
      * @return array
      */
     private function sanitizeIconToggleCheckbox(array $item, int $itemKey, array $newItems)
