@@ -21,9 +21,9 @@ use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
 use Psr\Http\Server\MiddlewareInterface;
 use Psr\Http\Server\RequestHandlerInterface;
+use TYPO3\CMS\Backend\Authentication\BackendLocker;
 use TYPO3\CMS\Backend\Exception\BackendAccessDeniedException;
 use TYPO3\CMS\Backend\Exception\BackendLockedException;
-use TYPO3\CMS\Core\Core\Environment;
 use TYPO3\CMS\Core\Http\JsonResponse;
 use TYPO3\CMS\Core\Http\RedirectResponse;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
@@ -39,12 +39,13 @@ use TYPO3\CMS\Core\Utility\HttpUtility;
  */
 class LockedBackendGuard implements MiddlewareInterface
 {
+    public function __construct(
+        protected readonly BackendLocker $lockService
+    ) {}
+
     /**
-     * Checks the client's IP address and if typo3conf/LOCK_BACKEND is available
-     *
-     * @param ServerRequestInterface $request
-     * @param RequestHandlerInterface $handler
-     * @return ResponseInterface
+     * Checks the client's IP address and the availability of LOCK_BACKEND file,
+     * location may vary, @see BackendLocker->isLocked().
      */
     public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
     {
@@ -56,12 +57,12 @@ class LockedBackendGuard implements MiddlewareInterface
         } catch (BackendLockedException $e) {
             // Looks like an AJAX request that can handle JSON, (usually from the timeout functionality)
             // So, let's form a request that fits
-            if (strpos($request->getHeaderLine('Accept'), 'application/json') !== false) {
+            if (str_contains($request->getHeaderLine('Accept'), 'application/json')) {
                 $session = [
                     'timed_out' => false,
                     'will_time_out' => false,
                     'locked' => true,
-                    'message' => $e->getMessage()
+                    'message' => $e->getMessage(),
                 ];
                 return new JsonResponse(['login' => $session]);
             }
@@ -76,29 +77,29 @@ class LockedBackendGuard implements MiddlewareInterface
     }
 
     /**
-     * Check adminOnly configuration variable and redirects to an URL in file typo3conf/LOCK_BACKEND
+     * Check adminOnly configuration variable and redirects to an URL in file
+     * LOCK_BACKEND. Location may vary, @see BackendLocker->isLocked().
      *
-     * @return string|null
      * @throws BackendLockedException
      */
-    protected function checkLockedBackend()
+    protected function checkLockedBackend(): ?string
     {
         if ($GLOBALS['TYPO3_CONF_VARS']['BE']['adminOnly'] < 0) {
             throw new BackendLockedException(
                 HttpUtility::HTTP_STATUS_403,
-                'Backend and Install Tool are locked for maintenance. [BE][adminOnly] is set to "' . (int)$GLOBALS['TYPO3_CONF_VARS']['BE']['adminOnly'] . '".',
+                'Backend is locked for maintenance. [BE][adminOnly] is set to "' . (int)$GLOBALS['TYPO3_CONF_VARS']['BE']['adminOnly'] . '".',
                 'TYPO3 Backend locked',
                 1517949794
             );
         }
-        if (@is_file(Environment::getLegacyConfigPath() . '/LOCK_BACKEND')) {
-            $fileContent = file_get_contents(Environment::getLegacyConfigPath() . '/LOCK_BACKEND');
-            if ($fileContent) {
-                return $fileContent;
+        if ($this->lockService->isLocked()) {
+            $redirectUri = $this->lockService->getRedirectUriFromLockContents();
+            if ($redirectUri) {
+                return $redirectUri;
             }
             throw new BackendLockedException(
                 HttpUtility::HTTP_STATUS_403,
-                'Backend access by browser is locked for maintenance. Remove lock by removing the file "typo3conf/LOCK_BACKEND" or use CLI-scripts.',
+                'Backend access by browser is locked for maintenance. Remove lock by removing the file "LOCK_BACKEND" as configured in TYPO3_CONF_VARS[BE][lockBackendFile]. Or (better) use CLI-script "bin/typo3 backend:unlock".',
                 'TYPO3 Backend locked',
                 1517949793
             );
@@ -109,12 +110,8 @@ class LockedBackendGuard implements MiddlewareInterface
 
     /**
      * Compare client IP with IPmaskList and throw an exception
-     *
-     * @param string $ipAddress
-     * @param string $ipMaskList
-     * @throws \RuntimeException
      */
-    protected function validateVisitorsIpAgainstIpMaskList(string $ipAddress, string $ipMaskList = '')
+    protected function validateVisitorsIpAgainstIpMaskList(string $ipAddress, string $ipMaskList = ''): void
     {
         if ($ipMaskList !== '' && !GeneralUtility::cmpIP($ipAddress, $ipMaskList)) {
             throw new BackendAccessDeniedException(
