@@ -1,5 +1,6 @@
 <?php
-namespace TYPO3\CMS\Backend\Tree\View;
+
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -14,6 +15,8 @@ namespace TYPO3\CMS\Backend\Tree\View;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Backend\Tree\View;
+
 /**
  * Extension for the tree class that generates the tree of pages in the page-wizard mode
  *
@@ -22,6 +25,14 @@ namespace TYPO3\CMS\Backend\Tree\View;
  */
 class NewRecordPageTreeView extends PageTreeView
 {
+    protected int $currentPageId;
+
+    public function __construct(int $currentPageId)
+    {
+        $this->currentPageId = $currentPageId;
+        parent::__construct();
+    }
+
     /**
      * Determines whether to expand a branch or not.
      * Here the branch is expanded if the current id matches the global id for the listing/new
@@ -31,6 +42,6 @@ class NewRecordPageTreeView extends PageTreeView
      */
     public function expandNext($id)
     {
-        return $id == $GLOBALS['SOBE']->id;
+        return (int)$id === $this->currentPageId;
     }
 }
