@@ -1,7 +1,6 @@
 <?php
-declare(strict_types = 1);
 
-namespace TYPO3\CMS\Backend\View\BackendLayout\Grid;
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -16,6 +15,8 @@ namespace TYPO3\CMS\Backend\View\BackendLayout\Grid;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Backend\View\BackendLayout\Grid;
+
 /**
  * Grid Row
  *
@@ -23,17 +24,19 @@ namespace TYPO3\CMS\Backend\View\BackendLayout\Grid;
  * Is solely responsible for grouping GridColumns.
  *
  * Accessed in Fluid templates.
+ *
+ * @internal this is experimental and subject to change in TYPO3 v10 / v11
  */
 class GridRow extends AbstractGridObject
 {
     /**
      * @var GridColumn[]
      */
-    protected $columns = [];
+    protected array $columns = [];
 
     public function addColumn(GridColumn $column): void
     {
-        $this->columns[$column->getColumnNumber()] = $column;
+        $this->columns[$column->getColumnNumber() ?? ''] = $column;
     }
 
     /**
