@@ -25,18 +25,17 @@ class EnvVariableProcessor implements PlaceholderProcessorInterface
 {
     public function canProcess(string $placeholder, array $referenceArray): bool
     {
-        return is_string($placeholder) && (strpos($placeholder, '%env(') !== false);
+        return is_string($placeholder) && (str_contains($placeholder, '%env('));
     }
 
     /**
-     * @param string $value
      * @param array|null $referenceArray
      * @return mixed|string
      */
     public function process(string $value, array $referenceArray)
     {
         $envVar = getenv($value);
-        if (!$envVar) {
+        if ($envVar === false) {
             throw new \UnexpectedValueException('Value not found', 1581501124);
         }
         return $envVar;
