@@ -21,20 +21,24 @@ use Psr\Http\Message\RequestFactoryInterface;
 use Psr\Http\Message\RequestInterface;
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\UriInterface;
+use Symfony\Component\DependencyInjection\Attribute\AsAlias;
 use TYPO3\CMS\Core\Http\Client\GuzzleClientFactory;
 
 /**
- * Class RequestFactory to create Request objects
- * Returns PSR-7 Request objects
+ * Front-end for sending requests, using PSR-7.
  */
+#[AsAlias(RequestFactoryInterface::class, public: true)]
 class RequestFactory implements RequestFactoryInterface
 {
+    public function __construct(
+        private readonly GuzzleClientFactory $guzzleFactory,
+    ) {}
+
     /**
      * Create a new request.
      *
      * @param string $method The HTTP method associated with the request.
      * @param UriInterface|string $uri The URI associated with the request.
-     * @return RequestInterface
      */
     public function createRequest(string $method, $uri): RequestInterface
     {
@@ -42,16 +46,15 @@ class RequestFactory implements RequestFactoryInterface
     }
 
     /**
-     * Create a guzzle request object with our custom implementation
+     * Create a Guzzle request object with our custom implementation
      *
      * @param string $uri the URI to request
      * @param string $method the HTTP method (defaults to GET)
      * @param array $options custom options for this request
-     * @return ResponseInterface
+     * @param ?string $context request context
      */
-    public function request(string $uri, string $method = 'GET', array $options = []): ResponseInterface
+    public function request(string $uri, string $method = 'GET', array $options = [], ?string $context = null): ResponseInterface
     {
-        $client = GuzzleClientFactory::getClient();
-        return $client->request($method, $uri, $options);
+        return $this->guzzleFactory->getClient($context)->request($method, $uri, $options);
     }
 }
