@@ -15,23 +15,17 @@
 
 namespace TYPO3\CMS\Core\Imaging;
 
-use TYPO3\CMS\Core\Cache\CacheManager;
+use TYPO3\CMS\Core\Cache\Event\CacheWarmupEvent;
 use TYPO3\CMS\Core\Cache\Frontend\FrontendInterface;
-use TYPO3\CMS\Core\Cache\Frontend\VariableFrontend;
-use TYPO3\CMS\Core\Core\Environment;
 use TYPO3\CMS\Core\Exception;
 use TYPO3\CMS\Core\Imaging\IconProvider\BitmapIconProvider;
-use TYPO3\CMS\Core\Imaging\IconProvider\FontawesomeIconProvider;
 use TYPO3\CMS\Core\Imaging\IconProvider\SvgIconProvider;
 use TYPO3\CMS\Core\Imaging\IconProvider\SvgSpriteIconProvider;
-use TYPO3\CMS\Core\Information\Typo3Version;
 use TYPO3\CMS\Core\SingletonInterface;
-use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
-use TYPO3\CMS\Core\Utility\StringUtility;
 
 /**
- * Class IconRegistry, which makes it possible to register custom icons
+ * Class which makes it possible to register custom icons
  * from within an extension.
  */
 class IconRegistry implements SingletonInterface
@@ -51,11 +45,6 @@ class IconRegistry implements SingletonInterface
      */
     protected $flagsInitialized = false;
 
-    /**
-     * @var bool
-     */
-    protected $moduleIconsInitialized = false;
-
     /**
      * @var bool
      */
@@ -73,16 +62,6 @@ class IconRegistry implements SingletonInterface
      */
     protected $backendIconDeclaration = 'EXT:core/Resources/Public/Icons/T3Icons/icons.json';
 
-    /**
-     * List of allowed icon file extensions with their Provider class
-     *
-     * @var string[]
-     */
-    protected $backendIconAllowedExtensionsWithProvider = [
-        'png' => BitmapIconProvider::class,
-        'svg' => SvgIconProvider::class
-    ];
-
     /**
      * manually registered icons
      * hopefully obsolete one day
@@ -97,190 +76,6 @@ class IconRegistry implements SingletonInterface
          * Icons are maintained in an external repository, if new icons are needed
          * please request them at: https://github.com/typo3/typo3.icons/issues
          */
-
-        // Apps
-        'apps-pagetree-category-toggle-hide-checked' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'check-square'
-            ]
-        ],
-
-        // Status
-        'status-dialog-information' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'exclamation-circle'
-            ]
-        ],
-        'status-dialog-ok' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'check-circle',
-            ]
-        ],
-        'status-dialog-notification' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'exclamation-circle'
-            ]
-        ],
-        'status-dialog-warning' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'exclamation-triangle'
-            ]
-        ],
-        'status-dialog-error' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'exclamation-circle'
-            ]
-        ],
-        'status-status-checked' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'check',
-            ]
-        ],
-        'status-status-current' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'caret-right',
-            ]
-        ],
-        'status-status-sorting-asc' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'caret-up',
-            ]
-        ],
-        'status-status-sorting-desc' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'caret-down',
-            ]
-        ],
-        'status-status-sorting-light-asc' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'caret-up',
-            ]
-        ],
-        'status-status-sorting-light-desc' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'caret-down',
-            ]
-        ],
-        'status-status-permission-granted' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'check',
-            ]
-        ],
-        'status-status-permission-denied' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'times',
-            ]
-        ],
-
-        // Empty
-        'empty-empty' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'empty-empty',
-            ]
-        ],
-
-        // System Information
-        'information-php-version' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'code'
-            ]
-        ],
-        'information-database' =>  [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'database'
-            ]
-        ],
-        'information-application-context' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'tasks'
-            ]
-        ],
-        'information-composer-mode' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'music'
-            ]
-        ],
-        'information-git' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'git'
-            ]
-        ],
-        'information-webserver' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'server'
-            ]
-        ],
-        'information-os-linux' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'linux'
-            ]
-        ],
-        'information-os-apple' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'apple'
-            ]
-        ],
-        'information-os-windows' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'windows'
-            ]
-        ],
-
-        // Sysnote
-        'sysnote-type-0' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'sticky-note-o'
-            ]
-        ],
-        'sysnote-type-1' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'cog'
-            ]
-        ],
-        'sysnote-type-2' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'code'
-            ]
-        ],
-        'sysnote-type-3' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'thumb-tack'
-            ]
-        ],
-        'sysnote-type-4' => [
-            'provider' => FontawesomeIconProvider::class,
-            'options' => [
-                'name' => 'check-square'
-            ]
-        ]
     ];
 
     /**
@@ -390,59 +185,13 @@ class IconRegistry implements SingletonInterface
      *
      * Example:
      * [
-     *   'deprecated-icon-identifier' => 'new-icon-identifier',
-     *   'another-deprecated-identifier' => null,
+     *   'deprecated-icon-identifier' => ['since' => 'TYPO3 v12', 'until' => 'TYPO3 v13', 'replacement' => 'new-icon-identifier'],
+     *   'another-deprecated-identifier' => ['since' => 'TYPO3 v12', 'until' => 'TYPO3 v13', 'replacement' => null],
      * ]
      *
      * @var array
      */
-    protected $deprecatedIcons = [
-        'module-web' => 'modulegroup-web',
-        'module-site' => 'modulegroup-site',
-        'module-file' => 'modulegroup-file',
-        'module-tools' => 'modulegroup-tools',
-        'module-system' => 'modulegroup-system',
-        'module-help' => 'modulegroup-help',
-        'module-workspaces-action-preview-link' => 'actions-version-workspaces-preview-link',
-        'generate-ws-preview-link' => 'actions-version-workspaces-preview-link',
-        'extensions-workspaces-generatepreviewlink' => 'actions-version-workspaces-preview-link',
-        'extensions-extensionmanager-update-script' => 'actions-refresh',
-        'extensions-scheduler-run-task' => 'actions-play',
-        'extensions-scheduler-run-task-cron' => 'actions-clock',
-        'status-warning-lock' => 'warning-lock',
-        'status-warning-in-use' => 'warning-in-use',
-        'status-status-edit-read-only' => 'status-edit-read-only',
-        't3-form-icon-advanced-password' => 'form-advanced-password',
-        't3-form-icon-checkbox' => 'form-checkbox',
-        't3-form-icon-content-element' => 'form-content-element',
-        't3-form-icon-date-picker' => 'form-date-picker',
-        't3-form-icon-duplicate' => 'actions-duplicate',
-        't3-form-icon-email' => 'form-email',
-        't3-form-icon-fieldset' => 'form-fieldset',
-        't3-form-icon-file-upload' => 'form-file-upload',
-        't3-form-icon-finisher' => 'form-finisher',
-        't3-form-icon-form-element-selector' => 'actions-variable-select',
-        't3-form-icon-gridcontainer' => 'form-gridcontainer',
-        't3-form-icon-gridrow' => 'form-gridrow',
-        't3-form-icon-hidden' => 'form-hidden',
-        't3-form-icon-image-upload' => 'form-image-upload',
-        't3-form-icon-insert-after' => 'actions-form-insert-after',
-        't3-form-icon-insert-in' => 'actions-form-insert-in',
-        't3-form-icon-multi-checkbox' => 'form-multi-checkbox',
-        't3-form-icon-multi-select' => 'form-multi-select',
-        't3-form-icon-number' => 'form-number',
-        't3-form-icon-page' => 'form-page',
-        't3-form-icon-password' => 'form-password',
-        't3-form-icon-radio-button' => 'form-radio-button',
-        't3-form-icon-single-select' => 'form-single-select',
-        't3-form-icon-static-text' => 'form-static-text',
-        't3-form-icon-summary-page' => 'form-summary-page',
-        't3-form-icon-telephone' => 'form-telephone',
-        't3-form-icon-text' => 'form-text',
-        't3-form-icon-textarea' => 'form-textarea',
-        't3-form-icon-url' => 'form-url',
-        't3-form-icon-validator' => 'form-validator',
-    ];
+    protected $deprecatedIcons = [];
 
     /**
      * @var string
@@ -452,31 +201,17 @@ class IconRegistry implements SingletonInterface
     /**
      * @var FrontendInterface
      */
-    protected static $cache = null;
+    protected $cache;
 
-    /**
-     * The constructor
-     */
-    public function __construct()
+    private string $cacheIdentifier;
+
+    public function __construct(FrontendInterface $assetsCache, string $cacheIdentifier)
     {
-        if (ExtensionManagementUtility::isLoaded('impexp')) {
-            // ext:impexp is no required extension, the target items are located there.
-            // Register its deprecations only if impexp is loaded
-            $this->deprecatedIcons['status-status-reference-hard'] = 'status-reference-hard';
-            $this->deprecatedIcons['status-status-reference-soft'] = 'status-reference-soft';
-        }
+        $this->cache = $assetsCache;
+        $this->cacheIdentifier = $cacheIdentifier;
         $this->initialize();
     }
 
-    /**
-     * @param FrontendInterface $cache
-     * @internal
-     */
-    public static function setCache(FrontendInterface $cache)
-    {
-        static::$cache = $cache;
-    }
-
     /**
      * Initialize the registry
      * This method can be called multiple times, depending on initialization status.
@@ -490,29 +225,31 @@ class IconRegistry implements SingletonInterface
         if (!$this->tcaInitialized && !empty($GLOBALS['TCA'])) {
             $this->registerTCAIcons();
         }
-        if (!$this->moduleIconsInitialized && !empty($GLOBALS['TBE_MODULES'])) {
-            $this->registerModuleIcons();
-        }
         if (!$this->flagsInitialized) {
-            $this->registerFlags();
+            $this->getCachedFlagIcons();
         }
         if ($this->backendIconsInitialized
             && $this->tcaInitialized
-            && $this->moduleIconsInitialized
             && $this->flagsInitialized) {
             $this->fullInitialized = true;
         }
     }
 
+    /**
+     * @internal
+     */
+    public function getBackendIconsCacheIdentifier(): string
+    {
+        return $this->cacheIdentifier;
+    }
+
     /**
      * Retrieve the icons from cache render them when not cached yet
      */
     protected function getCachedBackendIcons()
     {
-        $cacheIdentifier = 'BackendIcons_' . sha1((string)(new Typo3Version()) . Environment::getProjectPath() . 'BackendIcons');
-        /** @var VariableFrontend $assetsCache */
-        $assetsCache = static::$cache ?? GeneralUtility::makeInstance(CacheManager::class)->getCache('assets');
-        $cacheEntry = $assetsCache->get($cacheIdentifier);
+        $cacheIdentifier = $this->getBackendIconsCacheIdentifier();
+        $cacheEntry = $this->cache->get($cacheIdentifier);
 
         if ($cacheEntry !== false) {
             $this->icons = $cacheEntry;
@@ -520,7 +257,7 @@ class IconRegistry implements SingletonInterface
             $this->registerBackendIcons();
             // all found icons should now be present, for historic reasons now merge w/ the statically declared icons
             $this->icons = array_merge($this->icons, $this->iconAliases, $this->staticIcons);
-            $assetsCache->set($cacheIdentifier, $this->icons);
+            $this->cache->set($cacheIdentifier, $this->icons);
         }
         // if there's now at least one icon registered, consider it successful
         if (is_array($this->icons) && (count($this->icons) >= count($this->staticIcons))) {
@@ -535,7 +272,7 @@ class IconRegistry implements SingletonInterface
     {
         $dir = dirname($this->backendIconDeclaration);
         $absoluteIconDeclarationPath = GeneralUtility::getFileAbsFileName($this->backendIconDeclaration);
-        $json = json_decode(file_get_contents($absoluteIconDeclarationPath), true);
+        $json = json_decode(file_get_contents($absoluteIconDeclarationPath) ?: '', true);
         foreach ($json['icons'] ?? [] as $declaration) {
             $iconOptions = [
                 'sprite' => $dir . '/' . $declaration['sprite'],
@@ -545,6 +282,9 @@ class IconRegistry implements SingletonInterface
             if ($declaration['category'] === 'spinner') {
                 $iconOptions['spinning'] = true;
             }
+            if (isset($declaration['bidi']) && $declaration['bidi'] === true) {
+                $iconOptions['bidi'] = true;
+            }
 
             $this->registerIcon(
                 $declaration['identifier'],
@@ -579,10 +319,7 @@ class IconRegistry implements SingletonInterface
         return isset($this->deprecatedIcons[$identifier]);
     }
 
-    /**
-     * @return string
-     */
-    public function getDefaultIconIdentifier()
+    public function getDefaultIconIdentifier(): string
     {
         return $this->defaultIconIdentifier;
     }
@@ -592,20 +329,23 @@ class IconRegistry implements SingletonInterface
      *
      * @param string $identifier
      * @param string $iconProviderClassName
-     * @param array $options
      *
      * @throws \InvalidArgumentException
      */
     public function registerIcon($identifier, $iconProviderClassName, array $options = [])
     {
-        if (!in_array(IconProviderInterface::class, class_implements($iconProviderClassName), true)) {
+        if (!in_array(IconProviderInterface::class, class_implements($iconProviderClassName) ?: [], true)) {
             throw new \InvalidArgumentException('An IconProvider must implement '
                 . IconProviderInterface::class, 1437425803);
         }
         $this->icons[$identifier] = [
             'provider' => $iconProviderClassName,
-            'options' => $options
+            'options' => $options,
         ];
+
+        if (isset($options['deprecated'])) {
+            $this->deprecatedIcons[$identifier] = $options['deprecated'];
+        }
     }
 
     /**
@@ -659,14 +399,21 @@ class IconRegistry implements SingletonInterface
             $this->initialize();
         }
         if ($this->isDeprecated($identifier)) {
-            $replacement = $this->deprecatedIcons[$identifier] ?? null;
-            if (!empty($replacement)) {
-                $message = 'The icon "%s" is deprecated since TYPO3 v9 and will be removed in TYPO3 v10.0. Please use "%s" instead.';
-                $arguments = [$identifier, $replacement];
-                $identifier = $replacement;
-            } else {
-                $message = 'The icon "%s" is deprecated since TYPO3 v9 and will be removed in TYPO3 v10.0.';
-                $arguments = [$identifier];
+            $deprecation = $this->deprecatedIcons[$identifier];
+            $since = $deprecation['since'] ?? null;
+            $until = $deprecation['until'] ?? null;
+            $replacement = $deprecation['replacement'] ?? null;
+
+            $message = 'The icon "%s" is deprecated%s%s.';
+            $arguments = [
+                $identifier,
+                $since !== null ? ' since ' . $since : '',
+                $until !== null ? ' and will be removed in ' . $until : '',
+            ];
+
+            if ($replacement) {
+                $message .= ' Please use "%s" instead.';
+                $arguments[] = $replacement;
             }
             trigger_error(vsprintf($message, $arguments), E_USER_DEPRECATED);
         }
@@ -687,9 +434,6 @@ class IconRegistry implements SingletonInterface
         return array_keys($this->icons);
     }
 
-    /**
-     * @return array
-     */
     public function getDeprecatedIcons(): array
     {
         return $this->deprecatedIcons;
@@ -722,21 +466,6 @@ class IconRegistry implements SingletonInterface
         return $this->mimeTypeMapping[$mimeType];
     }
 
-    /**
-     * Calculates the cache identifier based on the current registry
-     *
-     * @return string
-     * @internal
-     */
-    public function getCacheIdentifier(): string
-    {
-        if (!$this->fullInitialized) {
-            $this->initialize();
-        }
-
-        return sha1((string)json_encode($this->icons));
-    }
-
     /**
      * Load icons from TCA for each table and add them as "tcarecords-XX" to $this->icons
      */
@@ -763,92 +492,55 @@ class IconRegistry implements SingletonInterface
             $this->icons[$iconIdentifier] = [
                 'provider' => $iconProviderClass,
                 'options' => [
-                    'source' => $iconFilePath
-                ]
+                    'source' => $iconFilePath,
+                ],
             ];
         }
         $this->tcaInitialized = true;
     }
 
-    /**
-     * Register module icons
-     */
-    protected function registerModuleIcons()
+    protected function getCachedFlagIcons(): void
     {
-        $moduleConfiguration = $GLOBALS['TBE_MODULES']['_configuration'] ?? [];
-        foreach ($moduleConfiguration as $moduleKey => $singleModuleConfiguration) {
-            $iconIdentifier = !empty($singleModuleConfiguration['iconIdentifier'])
-                ? $singleModuleConfiguration['iconIdentifier']
-                : null;
+        $cacheIdentifier = $this->getBackendIconsCacheIdentifier() . '_flags';
+        $cacheEntry = $this->cache->get($cacheIdentifier);
 
-            if ($iconIdentifier !== null) {
-                // iconIdentifier found, icon is registered, continue
-                continue;
-            }
-
-            $iconPath = !empty($singleModuleConfiguration['icon'])
-                ? $singleModuleConfiguration['icon']
-                : null;
-            $iconProviderClass = $this->detectIconProvider($iconPath);
-            $iconIdentifier = 'module-icon-' . $moduleKey;
-
-            $this->icons[$iconIdentifier] = [
-                'provider' => $iconProviderClass,
-                'options' => [
-                    'source' => $iconPath
-                ]
-            ];
+        if ($cacheEntry === false) {
+            $cacheEntry = $this->registerFlags();
+            $this->cache->set($cacheIdentifier, $cacheEntry);
+        }
+        $this->icons = array_merge($this->icons, $cacheEntry);
+        // if there's now at least one icon registered, consider it successful
+        if (is_array($cacheEntry) && $cacheEntry !== []) {
+            $this->flagsInitialized = true;
         }
-        $this->moduleIconsInitialized = true;
     }
 
     /**
      * Register flags
      */
-    protected function registerFlags()
+    protected function registerFlags(): array
     {
         $iconFolder = 'EXT:core/Resources/Public/Icons/Flags/';
-        $files = [
-            'AD', 'AE', 'AF', 'AG', 'AI', 'AL', 'AM', 'AN', 'AO', 'AQ', 'AR', 'AS', 'AT', 'AU', 'AW', 'AX', 'AZ',
-            'BA', 'BB', 'BD', 'BE', 'BF', 'BG', 'BH', 'BI', 'BJ', 'BL', 'BM', 'BN', 'BO', 'BQ', 'BR', 'BS', 'BT', 'BV', 'BW', 'BY', 'BZ',
-            'CA', 'CC', 'CD', 'CF', 'CG', 'CH', 'CI', 'CK', 'CL', 'CM', 'CN', 'CO', 'CR', 'CS', 'CU', 'CV', 'CW', 'CX', 'CY', 'CZ',
-            'DE', 'DJ', 'DK', 'DM', 'DO', 'DZ',
-            'EC', 'EE', 'EG', 'EH', 'ER', 'ES', 'ET', 'EU',
-            'FI', 'FJ', 'FK', 'FM', 'FO', 'FR',
-            'GA', 'GB-ENG', 'GB-NIR', 'GB-SCT', 'GB-WLS', 'GB', 'GD', 'GE', 'GF', 'GG', 'GH', 'GI', 'GL', 'GM', 'GN', 'GP', 'GQ', 'GR', 'GS', 'GT', 'GU', 'GW', 'GY',
-            'HK', 'HM', 'HN', 'HR', 'HT', 'HU',
-            'ID', 'IE', 'IL', 'IM', 'IN', 'IO', 'IQ', 'IR', 'IS', 'IT',
-            'JE', 'JM', 'JO', 'JP',
-            'KE', 'KG', 'KH', 'KI', 'KL', 'KM', 'KN', 'KP', 'KR', 'KW', 'KY', 'KZ',
-            'LA', 'LB', 'LC', 'LI', 'LK', 'LR', 'LS', 'LT', 'LU', 'LV', 'LY',
-            'MA', 'MC', 'MD', 'ME', 'MF', 'MG', 'MH', 'MI', 'MK', 'ML', 'MM', 'MN', 'MO', 'MP', 'MQ', 'MR', 'MS', 'MT', 'MU', 'MV', 'MW', 'MX', 'MY', 'MZ',
-            'NA', 'NC', 'NE', 'NF', 'NG', 'NI', 'NL', 'NO', 'NP', 'NR', 'NU', 'NZ',
-            'OM',
-            'PA', 'PE', 'PF', 'PG', 'PH', 'PK', 'PL', 'PM', 'PN', 'PR', 'PS', 'PT', 'PW', 'PY',
-            'QA', 'QC',
-            'RE', 'RO', 'RS', 'RU', 'RW',
-            'SA', 'SB', 'SC', 'SD', 'SE', 'SG', 'SH', 'SI', 'SJ', 'SK', 'SL', 'SM', 'SN', 'SO', 'SR', 'SS', 'ST', 'SV', 'SX', 'SY', 'SZ',
-            'TC', 'TD', 'TF', 'TG', 'TH', 'TJ', 'TK', 'TL', 'TM', 'TN', 'TO', 'TR', 'TT', 'TV', 'TW', 'TZ',
-            'UA', 'UG', 'UM', 'US', 'UY', 'UZ',
-            'VA', 'VC', 'VE', 'VG', 'VI', 'VN', 'VU',
-            'WF', 'WS',
-            'YE', 'YT',
-            'ZA', 'ZM', 'ZW',
-            // Special Flags
-            'catalonia',
-            'multiple',
-            'en-us-gb'
-        ];
-        foreach ($files as $file) {
-            $identifier = strtolower($file);
-            $this->icons['flags-' . $identifier] = [
-                'provider' => BitmapIconProvider::class,
-                'options' => [
-                    'source' => $iconFolder . $file . '.png'
-                ]
-            ];
+        $folderPath = GeneralUtility::getFileAbsFileName($iconFolder);
+        $flagIcons = [];
+
+        if ($handle = opendir($folderPath)) {
+            while (($file = readdir($handle)) !== false) {
+                $fileInfo = pathinfo($folderPath . $file);
+                if ($fileInfo['extension'] !== 'webp') {
+                    continue;
+                }
+                $flagIcons['flags-' . strtolower($fileInfo['filename'])] = [
+                    'provider' => BitmapIconProvider::class,
+                    'options' => [
+                        'source' => $iconFolder . $file,
+                    ],
+                ];
+            }
+            closedir($handle);
         }
-        $this->flagsInitialized = true;
+
+        return $flagIcons;
     }
 
     /**
@@ -859,9 +551,27 @@ class IconRegistry implements SingletonInterface
      */
     public function detectIconProvider($iconReference)
     {
-        if (StringUtility::endsWith(strtolower($iconReference), 'svg')) {
+        if (str_ends_with(strtolower((string)$iconReference), 'svg')) {
             return SvgIconProvider::class;
         }
         return BitmapIconProvider::class;
     }
+
+    public function warmupCaches(CacheWarmupEvent $event): void
+    {
+        if ($event->hasGroup('system')) {
+            $backupIcons = $this->icons;
+            $backupAliases = $this->iconAliases;
+            $this->icons = [];
+            $this->iconAliases = [];
+
+            $this->registerBackendIcons();
+            // all found icons should now be present, for historic reasons now merge w/ the statically declared icons
+            $this->icons = array_merge($this->icons, $this->iconAliases, $this->staticIcons);
+            $this->cache->set($this->getBackendIconsCacheIdentifier(), $this->icons);
+
+            $this->icons = $backupIcons;
+            $this->iconAliases = $backupAliases;
+        }
+    }
 }
