@@ -15,7 +15,8 @@
 
 namespace TYPO3\CMS\Core\Log;
 
-use Psr\Log\AbstractLogger;
+use Psr\Log\LoggerInterface;
+use Psr\Log\LoggerTrait;
 use TYPO3\CMS\Core\Log\Processor\ProcessorInterface;
 use TYPO3\CMS\Core\Log\Writer\WriterInterface;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
@@ -23,45 +24,38 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
 /**
  * Logger to log events and data for different components.
  */
-class Logger extends AbstractLogger
+class Logger implements LoggerInterface
 {
+    use LoggerTrait;
+
     /**
      * Logger name or component for which this logger is meant to be used for.
+     *
      * This should be a dot-separated name and should normally be based on
      * the class name or the name of a subsystem, such as
      * core.t3lib.cache.manager, core.backend.workspaces or extension.news
-     *
-     * @var string
      */
-    protected $name = '';
+    protected string $name = '';
 
     /**
      * Unique ID of the request
-     *
-     * @var string
      */
-    protected $requestId = '';
+    protected string $requestId = '';
 
     /**
      * Minimum log level, anything below this level will be ignored.
-     *
-     * @var int
      */
-    protected $minimumLogLevel;
+    protected int $minimumLogLevel;
 
     /**
      * Writers used by this logger
-     *
-     * @var array
      */
-    protected $writers = [];
+    protected array $writers = [];
 
     /**
      * Processors used by this logger
-     *
-     * @var array
      */
-    protected $processors = [];
+    protected array $processors = [];
 
     /**
      * Constructor.
@@ -90,8 +84,6 @@ class Logger extends AbstractLogger
 
     /**
      * Remove everything except the name, to be able to restore it on wakeup
-     *
-     * @return array
      */
     public function __sleep(): array
     {
@@ -134,22 +126,17 @@ class Logger extends AbstractLogger
     /**
      * Adds a writer to this logger
      *
-     * @param string $minimumLevel
      * @param \TYPO3\CMS\Core\Log\Writer\WriterInterface $writer Writer object
      * @return \TYPO3\CMS\Core\Log\Logger $this
      */
     public function addWriter(string $minimumLevel, WriterInterface $writer)
     {
         $minLevelAsNumber = LogLevel::normalizeLevel($minimumLevel);
-        LogLevel::validateLevel($minLevelAsNumber);
         // Cycle through all the log levels which are as severe as or higher
         // than $minimumLevel and add $writer to each severity level
-        for ($logLevelWhichTriggersWriter = LogLevel::normalizeLevel(LogLevel::EMERGENCY); $logLevelWhichTriggersWriter <= $minLevelAsNumber; $logLevelWhichTriggersWriter++) {
-            $logLevelName = LogLevel::getInternalName($logLevelWhichTriggersWriter);
-            if (!isset($this->writers[$logLevelName])) {
-                $this->writers[$logLevelName] = [];
-            }
-            $this->writers[$logLevelName][] = $writer;
+        foreach (LogLevel::atLeast($minLevelAsNumber) as $levelName) {
+            $this->writers[$levelName] ??= [];
+            $this->writers[$levelName][] = $writer;
         }
         if ($minLevelAsNumber > $this->getMinimumLogLevel()) {
             $this->setMinimumLogLevel($minLevelAsNumber);
@@ -170,7 +157,6 @@ class Logger extends AbstractLogger
     /**
      * Adds a processor to the logger.
      *
-     * @param string $minimumLevel
      * @param \TYPO3\CMS\Core\Log\Processor\ProcessorInterface $processor The processor to add.
      */
     public function addProcessor(string $minimumLevel, ProcessorInterface $processor)
@@ -179,11 +165,9 @@ class Logger extends AbstractLogger
         LogLevel::validateLevel($minLevelAsNumber);
         // Cycle through all the log levels which are as severe as or higher
         // than $minimumLevel and add $processor to each severity level
-        for ($logLevelWhichTriggersProcessor = LogLevel::normalizeLevel(LogLevel::EMERGENCY); $logLevelWhichTriggersProcessor <= $minLevelAsNumber; $logLevelWhichTriggersProcessor++) {
+        for ($logLevelWhichTriggersProcessor = LogLevel::normalizeLevel(\Psr\Log\LogLevel::EMERGENCY); $logLevelWhichTriggersProcessor <= $minLevelAsNumber; $logLevelWhichTriggersProcessor++) {
             $logLevelName = LogLevel::getInternalName($logLevelWhichTriggersProcessor);
-            if (!isset($this->processors[$logLevelName])) {
-                $this->processors[$logLevelName] = [];
-            }
+            $this->processors[$logLevelName] ??= [];
             $this->processors[$logLevelName][] = $processor;
         }
         if ($minLevelAsNumber > $this->getMinimumLogLevel()) {
@@ -205,22 +189,19 @@ class Logger extends AbstractLogger
      * Adds a log record.
      *
      * @param int|string $level Log level. Value according to \TYPO3\CMS\Core\Log\LogLevel. Alternatively accepts a string.
-     * @param string $message Log message.
+     * @param string|\Stringable $message Log message.
      * @param array $data Additional data to log
-     * @return mixed
      */
-    public function log($level, $message, array $data = [])
+    public function log($level, string|\Stringable $message, array $data = []): void
     {
         $level = LogLevel::normalizeLevel($level);
         LogLevel::validateLevel($level);
         if ($level > $this->minimumLogLevel) {
-            return $this;
+            return;
         }
-        /** @var \TYPO3\CMS\Core\Log\LogRecord $record */
         $record = GeneralUtility::makeInstance(LogRecord::class, $this->name, LogLevel::getInternalName($level), $message, $data, $this->requestId);
         $record = $this->callProcessors($record);
         $this->writeLog($record);
-        return $this;
     }
 
     /**
@@ -232,31 +213,25 @@ class Logger extends AbstractLogger
      */
     protected function callProcessors(LogRecord $record)
     {
-        if (!empty($this->processors[$record->getLevel()])) {
-            /** @var ProcessorInterface $processor */
-            foreach ($this->processors[$record->getLevel()] as $processor) {
-                $processedRecord = $processor->processLogRecord($record);
-                if (!$processedRecord instanceof LogRecord) {
-                    throw new \RuntimeException('Processor ' . get_class($processor) . ' returned invalid data. Instance of TYPO3\\CMS\\Core\\Log\\LogRecord expected', 1343593398);
-                }
-                $record = $processedRecord;
+        /** @var ProcessorInterface $processor */
+        foreach ($this->processors[$record->getLevel()] ?? [] as $processor) {
+            $processedRecord = $processor->processLogRecord($record);
+            if (!$processedRecord instanceof LogRecord) {
+                throw new \RuntimeException('Processor ' . get_class($processor) . ' returned invalid data. Instance of TYPO3\\CMS\\Core\\Log\\LogRecord expected', 1343593398);
             }
+            $record = $processedRecord;
         }
         return $record;
     }
 
     /**
      * Passes the \TYPO3\CMS\Core\Log\LogRecord to all registered writers.
-     *
-     * @param \TYPO3\CMS\Core\Log\LogRecord $record
      */
     protected function writeLog(LogRecord $record)
     {
-        if (!empty($this->writers[$record->getLevel()])) {
-            /** @var WriterInterface $writer */
-            foreach ($this->writers[$record->getLevel()] as $writer) {
-                $writer->writeLog($record);
-            }
+        /** @var WriterInterface $writer */
+        foreach ($this->writers[$record->getLevel()] ?? [] as $writer) {
+            $writer->writeLog($record);
         }
     }
 }
