@@ -25,17 +25,9 @@ use TYPO3\CMS\Core\Resource\FileInterface;
  * Example: If an extension provides additional functionality (e.g. variants), this event allows listener to also clean
  * up their custom handling. This can also be used for versioning of files.
  */
-final class AfterFileDeletedEvent
+final readonly class AfterFileDeletedEvent
 {
-    /**
-     * @var FileInterface
-     */
-    private $file;
-
-    public function __construct(FileInterface $file)
-    {
-        $this->file = $file;
-    }
+    public function __construct(private FileInterface $file) {}
 
     public function getFile(): FileInterface
     {
