@@ -15,17 +15,20 @@
 
 namespace TYPO3\CMS\Core\Resource\Index;
 
-use Doctrine\DBAL\Platforms\SQLServerPlatform;
 use Psr\EventDispatcher\EventDispatcherInterface;
+use TYPO3\CMS\Core\Context\Context;
 use TYPO3\CMS\Core\Database\Connection;
 use TYPO3\CMS\Core\Database\ConnectionPool;
 use TYPO3\CMS\Core\Database\Query\Restriction\RootLevelRestriction;
+use TYPO3\CMS\Core\Database\Query\Restriction\WorkspaceRestriction;
+use TYPO3\CMS\Core\Database\Schema\Information\ColumnInfo;
 use TYPO3\CMS\Core\Resource\Event\AfterFileMetaDataCreatedEvent;
 use TYPO3\CMS\Core\Resource\Event\AfterFileMetaDataDeletedEvent;
 use TYPO3\CMS\Core\Resource\Event\AfterFileMetaDataUpdatedEvent;
 use TYPO3\CMS\Core\Resource\Event\EnrichFileMetaDataEvent;
 use TYPO3\CMS\Core\Resource\Exception\InvalidUidException;
 use TYPO3\CMS\Core\Resource\File;
+use TYPO3\CMS\Core\Resource\FileType;
 use TYPO3\CMS\Core\SingletonInterface;
 use TYPO3\CMS\Core\Type\File\ImageInfo;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
@@ -43,27 +46,13 @@ class MetaDataRepository implements SingletonInterface
      */
     protected $tableName = 'sys_file_metadata';
 
-    /**
-     * Internal storage for database table fields
-     *
-     * @var array
-     */
-    protected $tableFields = [];
-
-    /**
-     * @var EventDispatcherInterface
-     */
-    protected $eventDispatcher;
-
-    public function __construct(EventDispatcherInterface $eventDispatcher)
-    {
-        $this->eventDispatcher = $eventDispatcher;
-    }
+    public function __construct(
+        protected readonly EventDispatcherInterface $eventDispatcher,
+    ) {}
 
     /**
      * Returns array of meta-data properties
      *
-     * @param File $file
      * @return array
      */
     public function findByFile(File $file)
@@ -77,7 +66,7 @@ class MetaDataRepository implements SingletonInterface
         // This logic can be transferred into a custom PSR-14 event listener in the future by just using
         // the AfterMetaDataCreated event.
         if (!empty($record['crdate']) && (int)$record['crdate'] === $GLOBALS['EXEC_TIME']) {
-            if ($file->getType() === File::FILETYPE_IMAGE && $file->getStorage()->getDriverType() === 'Local') {
+            if ($file->isType(FileType::IMAGE) && $file->getStorage()->getDriverType() === 'Local') {
                 $fileNameAndPath = $file->getForLocalProcessing(false);
 
                 $imageInfo = GeneralUtility::makeInstance(ImageInfo::class, $fileNameAndPath);
@@ -87,7 +76,7 @@ class MetaDataRepository implements SingletonInterface
                     'height' => $imageInfo->getHeight(),
                 ];
 
-                $this->update($file->getUid(), $additionalMetaInformation);
+                $this->update($file->getUid(), $additionalMetaInformation, $record);
             }
             $record = $this->findByFileUid($file->getUid());
         }
@@ -99,7 +88,7 @@ class MetaDataRepository implements SingletonInterface
      * Retrieves metadata for file
      *
      * @param int $uid
-     * @return array
+     * @return array<string, string> $metaData
      * @throws InvalidUidException
      */
     public function findByFileUid($uid)
@@ -109,19 +98,24 @@ class MetaDataRepository implements SingletonInterface
             throw new InvalidUidException('Metadata can only be retrieved for indexed files. UID: "' . $uid . '"', 1381590731);
         }
 
+        $context = GeneralUtility::makeInstance(Context::class);
         $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable($this->tableName);
-
-        $queryBuilder->getRestrictions()->add(GeneralUtility::makeInstance(RootLevelRestriction::class));
+        $queryBuilder->getRestrictions()
+            ->add(GeneralUtility::makeInstance(RootLevelRestriction::class))
+            ->add(GeneralUtility::makeInstance(WorkspaceRestriction::class, $context->getAspect('workspace')->getId()));
 
         $record = $queryBuilder
             ->select('*')
             ->from($this->tableName)
             ->where(
-                $queryBuilder->expr()->eq('file', $queryBuilder->createNamedParameter($uid, \PDO::PARAM_INT)),
+                $queryBuilder->expr()->eq('file', $queryBuilder->createNamedParameter($uid, Connection::PARAM_INT)),
                 $queryBuilder->expr()->in('sys_language_uid', $queryBuilder->createNamedParameter([0, -1], Connection::PARAM_INT_ARRAY))
             )
-            ->execute()
-            ->fetch();
+            // assure deterministic sorting across all databases
+            ->orderBy('uid', 'ASC')
+            ->setMaxResults(1)
+            ->executeQuery()
+            ->fetchAssociative();
 
         if (empty($record)) {
             return [];
@@ -134,7 +128,6 @@ class MetaDataRepository implements SingletonInterface
      * Create empty
      *
      * @param int $fileUid
-     * @param array $additionalFields
      * @return array
      */
     public function createMetaDataRecord($fileUid, array $additionalFields = [])
@@ -144,8 +137,7 @@ class MetaDataRepository implements SingletonInterface
             'pid' => 0,
             'crdate' => $GLOBALS['EXEC_TIME'],
             'tstamp' => $GLOBALS['EXEC_TIME'],
-            'cruser_id' => isset($GLOBALS['BE_USER']->user['uid']) ? (int)$GLOBALS['BE_USER']->user['uid'] : 0,
-            'l10n_diffsource' => ''
+            'l10n_diffsource' => '',
         ];
         $additionalFields = array_intersect_key($additionalFields, $this->getTableFields());
         $emptyRecord = array_merge($emptyRecord, $additionalFields);
@@ -158,7 +150,7 @@ class MetaDataRepository implements SingletonInterface
         );
 
         $record = $emptyRecord;
-        $record['uid'] = $connection->lastInsertId($this->tableName);
+        $record['uid'] = $connection->lastInsertId();
 
         return $this->eventDispatcher->dispatch(new AfterFileMetaDataCreatedEvent($fileUid, (int)$record['uid'], $record))->getRecord();
     }
@@ -167,38 +159,43 @@ class MetaDataRepository implements SingletonInterface
      * Updates the metadata record in the database
      *
      * @param int $fileUid the file uid to update
-     * @param array $data Data to update
+     * @param array $updateData Data to update
+     * @param ?array $metaDataFromDatabase Current meta data from database
+     * @return array The updated database record - or just $metaDataFromDatabase if no update was done
      * @internal
      */
-    public function update($fileUid, array $data)
+    public function update($fileUid, array $updateData, ?array $metaDataFromDatabase = null): array
     {
-        $updateRow = array_intersect_key($data, $this->getTableFields());
+        // backwards compatibility layer
+        $metaDataFromDatabase ??= $this->findByFileUid($fileUid);
+
+        $updateRow = array_intersect_key($updateData, $this->getTableFields());
+        if ($updateRow === []) {
+            // No valid keys to update - return current database row
+            return $metaDataFromDatabase;
+        }
         if (array_key_exists('uid', $updateRow)) {
             unset($updateRow['uid']);
         }
-        $row = $this->findByFileUid($fileUid);
-        if (!empty($updateRow)) {
-            $updateRow['tstamp'] = time();
-            $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable($this->tableName);
-            $types = [];
-            if ($connection->getDatabasePlatform() instanceof SQLServerPlatform) {
-                // mssql needs to set proper PARAM_LOB and others to update fields
-                $tableDetails = $connection->getSchemaManager()->listTableDetails($this->tableName);
-                foreach ($updateRow as $columnName => $columnValue) {
-                    $types[$columnName] = $tableDetails->getColumn($columnName)->getType()->getBindingType();
-                }
-            }
-            $connection->update(
-                $this->tableName,
-                $updateRow,
-                [
-                    'uid' => (int)$row['uid']
-                ],
-                $types
-            );
-
-            $this->eventDispatcher->dispatch(new AfterFileMetaDataUpdatedEvent($fileUid, (int)$row['uid'], array_merge($row, $updateRow)));
+        $updateRow = array_diff_assoc($updateRow, $metaDataFromDatabase);
+        if ($updateRow === []) {
+            // Nothing to update - return current database row
+            return $metaDataFromDatabase;
         }
+
+        $updateRow['tstamp'] = time();
+        $connection = GeneralUtility::makeInstance(ConnectionPool::class)->getConnectionForTable($this->tableName);
+        $connection->update(
+            $this->tableName,
+            $updateRow,
+            [
+                'uid' => (int)$metaDataFromDatabase['uid'],
+            ]
+        );
+
+        return $this->eventDispatcher->dispatch(
+            new AfterFileMetaDataUpdatedEvent($fileUid, (int)$metaDataFromDatabase['uid'], array_merge($metaDataFromDatabase, $updateRow))
+        )->getRecord();
     }
 
     /**
@@ -213,7 +210,7 @@ class MetaDataRepository implements SingletonInterface
             ->delete(
                 $this->tableName,
                 [
-                    'file' => (int)$fileUid
+                    'file' => (int)$fileUid,
                 ]
             );
 
@@ -223,25 +220,13 @@ class MetaDataRepository implements SingletonInterface
     /**
      * Gets the fields that are available in the table
      *
-     * @return array
+     * @return array<string, ColumnInfo>
      */
     protected function getTableFields(): array
     {
-        if (empty($this->tableFields)) {
-            $this->tableFields = GeneralUtility::makeInstance(ConnectionPool::class)
-                ->getConnectionForTable($this->tableName)
-                ->getSchemaManager()
-                ->listTableColumns($this->tableName);
-        }
-
-        return $this->tableFields;
-    }
-
-    /**
-     * @return MetaDataRepository
-     */
-    public static function getInstance()
-    {
-        return GeneralUtility::makeInstance(self::class);
+        return GeneralUtility::makeInstance(ConnectionPool::class)
+            ->getConnectionForTable($this->tableName)
+            ->getSchemaInformation()
+            ->listTableColumnInfos($this->tableName);
     }
 }
