@@ -15,19 +15,20 @@
 
 namespace TYPO3\CMS\Core\Resource\Processing;
 
-use TYPO3\CMS\Core\Imaging\GraphicalFunctions;
+use Psr\Log\LoggerAwareInterface;
+use Psr\Log\LoggerAwareTrait;
+use TYPO3\CMS\Core\Type\File\ImageInfo;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 
 /**
  * Processes Local Images files
  */
-class LocalImageProcessor implements ProcessorInterface
+class LocalImageProcessor implements ProcessorInterface, LoggerAwareInterface
 {
+    use LoggerAwareTrait;
+
     /**
      * Returns TRUE if this processor can process the given task.
-     *
-     * @param TaskInterface $task
-     * @return bool
      */
     public function canProcessTask(TaskInterface $task): bool
     {
@@ -38,7 +39,6 @@ class LocalImageProcessor implements ProcessorInterface
     /**
      * Processes the given task.
      *
-     * @param TaskInterface $task
      * @throws \InvalidArgumentException
      */
     public function processTask(TaskInterface $task): void
@@ -52,8 +52,6 @@ class LocalImageProcessor implements ProcessorInterface
     /**
      * Processes an image described in a task, but optionally uses a given local image
      *
-     * @param TaskInterface $task
-     * @param string|null $localFile
      * @throws \InvalidArgumentException
      */
     public function processTaskWithLocalFile(TaskInterface $task, ?string $localFile): void
@@ -70,29 +68,40 @@ class LocalImageProcessor implements ProcessorInterface
                 $task->getTargetFile()->setUsesOriginalFile();
             } elseif (!empty($result['filePath']) && file_exists($result['filePath'])) {
                 $task->setExecuted(true);
-                $imageDimensions = $this->getGraphicalFunctionsObject()->getImageDimensions($result['filePath']);
-                $task->getTargetFile()->setName($task->getTargetFileName());
-                $task->getTargetFile()->updateProperties(
-                    ['width' => $imageDimensions[0], 'height' => $imageDimensions[1], 'size' => filesize($result['filePath']), 'checksum' => $task->getConfigurationChecksum()]
-                );
+                $imageInformation = GeneralUtility::makeInstance(ImageInfo::class, $result['filePath']);
+                if (($result['remapProcessedTargetFileExtension'] ?? null) !== null) {
+                    // Processing changed the target filename extension to something else.
+                    // We need to react on this, because otherwise the file contents will not
+                    // match the file extension.
+                    $task->getTargetFile()->setName($task->getTargetFileName() . '.' . $result['remapProcessedTargetFileExtension']);
+                } else {
+                    $task->getTargetFile()->setName($task->getTargetFileName());
+                }
+                $task->getTargetFile()->updateProperties([
+                    'width' => $imageInformation->getWidth(),
+                    'height' => $imageInformation->getHeight(),
+                    'size' => $imageInformation->getSize(),
+                    'checksum' => $task->getConfigurationChecksum(),
+                ]);
                 $task->getTargetFile()->updateWithLocalFile($result['filePath']);
             } else {
                 // Seems we have no valid processing result
                 $task->setExecuted(false);
             }
         } catch (\Exception $e) {
+            // @todo: Swallowing all exceptions including PHP warnings here is a bad idea.
+            // @todo: This should be restricted to more specific exceptions - if at all.
+            // @todo: For now, we at least log the situation.
+            $this->logger->error(sprintf('Processing task of image file'), ['exception' => $e]);
             $task->setExecuted(false);
         }
     }
 
     /**
-     * Check if the to be processed target file already exists
-     * if exist take info from that file and mark task as done
-     *
-     * @param TaskInterface $task
-     * @return bool
+     * Check if the target file that is to be processed already exists.
+     * If it exists, use the metadata from that file and mark task as done.
      */
-    protected function checkForExistingTargetFile(TaskInterface $task)
+    protected function checkForExistingTargetFile(TaskInterface $task): bool
     {
         // the storage of the processed file, not of the original file!
         $storage = $task->getTargetFile()->getStorage();
@@ -108,12 +117,12 @@ class LocalImageProcessor implements ProcessorInterface
             // have no API for fetching file metadata from a remote file.
             $localProcessedFile = $storage->getFileForLocalProcessing($task->getTargetFile(), false);
             $task->setExecuted(true);
-            $imageDimensions = $this->getGraphicalFunctionsObject()->getImageDimensions($localProcessedFile);
+            $imageInformation = GeneralUtility::makeInstance(ImageInfo::class, $localProcessedFile);
             $properties = [
-                'width' => $imageDimensions[0],
-                'height' => $imageDimensions[1],
-                'size' => filesize($localProcessedFile),
-                'checksum' => $task->getConfigurationChecksum()
+                'width' => $imageInformation->getWidth(),
+                'height' => $imageInformation->getHeight(),
+                'size' => $imageInformation->getSize(),
+                'checksum' => $task->getConfigurationChecksum(),
             ];
             $task->getTargetFile()->updateProperties($properties);
 
@@ -132,22 +141,14 @@ class LocalImageProcessor implements ProcessorInterface
         switch ($taskName) {
             case 'Preview':
                 $helper = GeneralUtility::makeInstance(LocalPreviewHelper::class);
-            break;
+                break;
             case 'CropScaleMask':
                 $helper = GeneralUtility::makeInstance(LocalCropScaleMaskHelper::class);
-            break;
+                break;
             default:
                 throw new \InvalidArgumentException('Cannot find helper for task name: "' . $taskName . '"', 1353401352);
         }
 
         return $helper;
     }
-
-    /**
-     * @return GraphicalFunctions
-     */
-    protected function getGraphicalFunctionsObject(): GraphicalFunctions
-    {
-        return GeneralUtility::makeInstance(GraphicalFunctions::class);
-    }
 }
