@@ -16,14 +16,16 @@
 namespace TYPO3\CMS\Core\Resource;
 
 use TYPO3\CMS\Core\Core\Environment;
+use TYPO3\CMS\Core\Log\LogManager;
+use TYPO3\CMS\Core\Page\PageRenderer;
+use TYPO3\CMS\Core\Type\DocType;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\MathUtility;
 use TYPO3\CMS\Core\Utility\PathUtility;
-use TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController;
 
 /**
- * Compressor
- * This merges and compresses CSS and JavaScript files of the TYPO3 Backend.
+ * This class merges and compresses CSS and JavaScript files of the TYPO3 Frontend.
+ * It should never be used for TYPO3 Backend.
  */
 class ResourceCompressor
 {
@@ -38,12 +40,14 @@ class ResourceCompressor
     protected $rootPath = '';
 
     /**
-     * gzipped versions are only created if $TYPO3_CONF_VARS[TYPO3_MODE]['compressionLevel'] is set
+     * gzipped versions are only created if $TYPO3_CONF_VARS['BE' or 'FE']['compressionLevel'] is set
      *
      * @var bool
      */
     protected $createGzipped = false;
 
+    protected string $gzipFileExtension = '.gz';
+
     /**
      * @var int
      */
@@ -52,7 +56,7 @@ class ResourceCompressor
     /**
      * @var string
      */
-    protected $htaccessTemplate = '<FilesMatch "\\.(js|css)(\\.gzip)?$">
+    protected $htaccessTemplate = '<FilesMatch "\\.(js|css)(\\.gz)?$">
 	<IfModule mod_expires.c>
 		ExpiresActive on
 		ExpiresDefault "access plus 7 days"
@@ -60,11 +64,13 @@ class ResourceCompressor
 	FileETag MTime Size
 </FilesMatch>';
 
-    /**
-     * Constructor
-     */
-    public function __construct()
+    protected bool $initialized = false;
+
+    protected function initialize(): void
     {
+        if ($this->initialized) {
+            return;
+        }
         // we check for existence of our targetDirectory
         if (!is_dir(Environment::getPublicPath() . '/' . $this->targetDirectory)) {
             GeneralUtility::mkdir_deep(Environment::getPublicPath() . '/' . $this->targetDirectory);
@@ -74,11 +80,11 @@ class ResourceCompressor
             // check whether .htaccess exists
             $htaccessPath = Environment::getPublicPath() . '/' . $this->targetDirectory . '.htaccess';
             if (!file_exists($htaccessPath)) {
-                GeneralUtility::writeFile($htaccessPath, $this->htaccessTemplate);
+                GeneralUtility::writeFile($htaccessPath, $this->htaccessTemplate, true);
             }
         }
         // decide whether we should create gzipped versions or not
-        $compressionLevel = $GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['compressionLevel'];
+        $compressionLevel = $GLOBALS['TYPO3_CONF_VARS']['FE']['compressionLevel'];
         // we need zlib for gzencode()
         if (extension_loaded('zlib') && $compressionLevel) {
             $this->createGzipped = true;
@@ -87,19 +93,8 @@ class ResourceCompressor
                 $this->gzipCompressionLevel = (int)$compressionLevel;
             }
         }
-        $this->setRootPath(TYPO3_MODE === 'BE' ? Environment::getBackendPath() . '/' : Environment::getPublicPath() . '/');
-    }
-
-    /**
-     * Sets absolute path to working directory
-     *
-     * @param string $rootPath Absolute path
-     */
-    public function setRootPath($rootPath)
-    {
-        if (is_string($rootPath)) {
-            $this->rootPath = $rootPath;
-        }
+        $this->rootPath = Environment::getPublicPath() . '/';
+        $this->initialized = true;
     }
 
     /**
@@ -110,6 +105,7 @@ class ResourceCompressor
      */
     public function concatenateCssFiles(array $cssFiles)
     {
+        $this->initialize();
         $filesToIncludeByType = ['all' => []];
         foreach ($cssFiles as $key => $fileOptions) {
             // no concatenation allowed for this file, so continue
@@ -141,7 +137,7 @@ class ResourceCompressor
                 'compress' => true,
                 'excludeFromConcatenation' => true,
                 'forceOnTop' => false,
-                'allWrap' => ''
+                'allWrap' => '',
             ];
             // place the merged stylesheet on top of the stylesheets
             $cssFiles = array_merge($cssFiles, [$targetFile => $concatenatedOptions]);
@@ -157,6 +153,7 @@ class ResourceCompressor
      */
     public function concatenateJsFiles(array $jsFiles)
     {
+        $this->initialize();
         $concatenatedJsFileIsAsync = false;
         $allFilesToConcatenateAreAsync = true;
         $filesToInclude = [];
@@ -174,7 +171,7 @@ class ResourceCompressor
             } else {
                 $filesToInclude[$fileOptions['section']][] = $filenameFromMainDir;
             }
-            if (!empty($fileOptions['async']) && (bool)$fileOptions['async']) {
+            if ($fileOptions['async'] ?? false) {
                 $concatenatedJsFileIsAsync = true;
             } else {
                 $allFilesToConcatenateAreAsync = false;
@@ -248,13 +245,13 @@ class ResourceCompressor
             if (GeneralUtility::isValidUrl($filename)) {
                 // check if it is possibly a local file with fully qualified URL
                 if (GeneralUtility::isOnCurrentHost($filename) &&
-                    GeneralUtility::isFirstPartOfStr(
+                    str_starts_with(
                         $filename,
-                        GeneralUtility::getIndpEnv('TYPO3_SITE_URL')
+                        $GLOBALS['TYPO3_REQUEST']->getAttribute('normalizedParams')->getSiteUrl()
                     )
                 ) {
                     // attempt to turn it into a local file path
-                    $localFilename = substr($filename, strlen(GeneralUtility::getIndpEnv('TYPO3_SITE_URL')));
+                    $localFilename = substr($filename, strlen($GLOBALS['TYPO3_REQUEST']->getAttribute('normalizedParams')->getSiteUrl()));
                     if (@is_file(GeneralUtility::resolveBackPath($this->rootPath . $localFilename))) {
                         $filesToInclude[$key] = $localFilename;
                     } else {
@@ -283,11 +280,11 @@ class ResourceCompressor
                 $filename = PathUtility::stripPathSitePrefix($filenameAbsolute);
                 $contents = (string)file_get_contents($filenameAbsolute);
                 // remove any UTF-8 byte order mark (BOM) from files
-                if (strpos($contents, "\xEF\xBB\xBF") === 0) {
+                if (str_starts_with($contents, "\xEF\xBB\xBF")) {
                     $contents = substr($contents, 3);
                 }
                 // only fix paths if files aren't already in typo3temp (already processed)
-                if ($type === 'css' && !GeneralUtility::isFirstPartOfStr($filename, $this->targetDirectory)) {
+                if ($type === 'css' && !str_starts_with($filename, $this->targetDirectory)) {
                     $contents = $this->cssFixRelativeUrlPaths($contents, $filename);
                 }
                 $concatenated .= LF . $contents;
@@ -296,7 +293,7 @@ class ResourceCompressor
             if ($type === 'css') {
                 $concatenated = $this->cssFixStatements($concatenated);
             }
-            GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $targetFile, $concatenated);
+            GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $targetFile, $concatenated, true);
         }
         return $targetFile;
     }
@@ -309,6 +306,7 @@ class ResourceCompressor
      */
     public function compressCssFiles(array $cssFiles)
     {
+        $this->initialize();
         $filesAfterCompression = [];
         foreach ($cssFiles as $key => $fileOptions) {
             // if compression is enabled
@@ -338,6 +336,7 @@ class ResourceCompressor
      */
     public function compressCssFile($filename)
     {
+        $this->initialize();
         // generate the unique name of the file
         $filenameAbsolute = GeneralUtility::resolveBackPath($this->rootPath . $this->getFilenameFromMainDir($filename));
         if (@file_exists($filenameAbsolute)) {
@@ -352,9 +351,9 @@ class ResourceCompressor
         $pathinfo = PathUtility::pathinfo($filenameAbsolute);
         $targetFile = $this->targetDirectory . $pathinfo['filename'] . '-' . md5($unique) . '.css';
         // only create it, if it doesn't exist, yet
-        if (!file_exists(Environment::getPublicPath() . '/' . $targetFile) || $this->createGzipped && !file_exists(Environment::getPublicPath() . '/' . $targetFile . '.gzip')) {
+        if (!file_exists(Environment::getPublicPath() . '/' . $targetFile) || $this->createGzipped && !file_exists(Environment::getPublicPath() . '/' . $targetFile . $this->gzipFileExtension)) {
             $contents = $this->compressCssString((string)file_get_contents($filenameAbsolute));
-            if (strpos($filename, $this->targetDirectory) === false) {
+            if (!str_contains($filename, $this->targetDirectory)) {
                 $contents = $this->cssFixRelativeUrlPaths($contents, $filename);
             }
             $this->writeFileAndCompressed($targetFile, $contents);
@@ -370,6 +369,7 @@ class ResourceCompressor
      */
     public function compressJsFiles(array $jsFiles)
     {
+        $this->initialize();
         $filesAfterCompression = [];
         foreach ($jsFiles as $fileName => $fileOptions) {
             // If compression is enabled
@@ -393,6 +393,7 @@ class ResourceCompressor
      */
     public function compressJsFile($filename)
     {
+        $this->initialize();
         // generate the unique name of the file
         $filenameAbsolute = GeneralUtility::resolveBackPath($this->rootPath . $this->getFilenameFromMainDir($filename));
         if (@file_exists($filenameAbsolute)) {
@@ -404,7 +405,7 @@ class ResourceCompressor
         $pathinfo = PathUtility::pathinfo($filename);
         $targetFile = $this->targetDirectory . $pathinfo['filename'] . '-' . md5($unique) . '.js';
         // only create it, if it doesn't exist, yet
-        if (!file_exists(Environment::getPublicPath() . '/' . $targetFile) || $this->createGzipped && !file_exists(Environment::getPublicPath() . '/' . $targetFile . '.gzip')) {
+        if (!file_exists(Environment::getPublicPath() . '/' . $targetFile) || $this->createGzipped && !file_exists(Environment::getPublicPath() . '/' . $targetFile . $this->gzipFileExtension)) {
             $contents = (string)file_get_contents($filenameAbsolute);
             $this->writeFileAndCompressed($targetFile, $contents);
         }
@@ -442,7 +443,8 @@ class ResourceCompressor
             $absolutePath = $filename;
         }
         if (@is_file($absolutePath)) {
-            if (strpos($absolutePath, $this->rootPath) === 0) {
+            $absolutePath = Environment::getPublicPath() . '/' . PathUtility::getAbsoluteWebPath($absolutePath, false);
+            if (str_starts_with($absolutePath, $this->rootPath)) {
                 // the path is within the current root path, simply strip rootPath off
                 return substr($absolutePath, strlen($this->rootPath));
             }
@@ -454,15 +456,9 @@ class ResourceCompressor
         if (is_file($this->rootPath . $fileNameWithoutSlash)) {
             return $fileNameWithoutSlash;
         }
-        // if the file is from a special TYPO3 internal directory, add the missing typo3/ prefix
-        if (is_file((string)realpath(Environment::getBackendPath() . '/' . $filename))) {
-            $filename = 'typo3/' . $filename;
-        }
         // build the file path relative to the public web path
-        if (strpos($filename, 'EXT:') === 0) {
-            $file = GeneralUtility::getFileAbsFileName($filename);
-        } elseif (strpos($filename, '../') === 0) {
-            $file = GeneralUtility::resolveBackPath(Environment::getBackendPath() . '/' . $filename);
+        if (PathUtility::isExtensionPath($filename)) {
+            $file = Environment::getPublicPath() . '/' . PathUtility::getPublicResourceWebPath($filename, false);
         } else {
             $file = Environment::getPublicPath() . '/' . $filename;
         }
@@ -486,18 +482,13 @@ class ResourceCompressor
     {
         foreach ($baseDirectories as $baseDirectory) {
             // check, if $filename starts with base directory
-            if (GeneralUtility::isFirstPartOfStr($filename, $baseDirectory)) {
+            if (str_starts_with($filename, $baseDirectory)) {
                 return true;
             }
         }
         return false;
     }
 
-    /**
-     * @param string $contents
-     * @param string $filename
-     * @return string
-     */
     protected function cssFixRelativeUrlPaths(string $contents, string $filename): string
     {
         $newDir = '../../../' . PathUtility::dirname($filename) . '/';
@@ -558,7 +549,7 @@ class ResourceCompressor
     }
 
     /**
-     * Writes $contents into file $filename together with a gzipped version into $filename.gz
+     * Writes $contents into file $filename together with a gzipped version into $filename.gz (gzipFileExtension)
      *
      * @param string $filename Target filename
      * @param string $contents File contents
@@ -566,10 +557,10 @@ class ResourceCompressor
     protected function writeFileAndCompressed($filename, $contents)
     {
         // write uncompressed file
-        GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $filename, $contents);
+        GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $filename, $contents, true);
         if ($this->createGzipped) {
             // create compressed version
-            GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $filename . '.gzip', (string)gzencode($contents, $this->gzipCompressionLevel));
+            GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $filename . $this->gzipFileExtension, (string)gzencode($contents, $this->gzipCompressionLevel), true);
         }
     }
 
@@ -578,13 +569,13 @@ class ResourceCompressor
      * based on HTTP_ACCEPT_ENCODING
      *
      * @param string $filename File name
-     * @return string $filename suffixed with '.gzip' or not - dependent on HTTP_ACCEPT_ENCODING
+     * @return string $filename suffixed with '.gz' or not - dependent on HTTP_ACCEPT_ENCODING
      */
     protected function returnFileReference($filename)
     {
         // if the client accepts gzip and we can create gzipped files, we give him compressed versions
-        if ($this->createGzipped && strpos(GeneralUtility::getIndpEnv('HTTP_ACCEPT_ENCODING'), 'gzip') !== false) {
-            $filename .= '.gzip';
+        if ($this->createGzipped && str_contains(GeneralUtility::getIndpEnv('HTTP_ACCEPT_ENCODING'), 'gzip')) {
+            $filename .= $this->gzipFileExtension;
         }
         return PathUtility::getRelativePath($this->rootPath, Environment::getPublicPath() . '/') . $filename;
     }
@@ -603,11 +594,30 @@ class ResourceCompressor
         if (!file_exists(Environment::getPublicPath() . '/' . $filename)
             || !hash_equals(md5((string)file_get_contents(Environment::getPublicPath() . '/' . $filename)), md5($externalContent))
         ) {
-            GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $filename, $externalContent);
+            GeneralUtility::writeFile(Environment::getPublicPath() . '/' . $filename, $externalContent, true);
         }
         return $filename;
     }
 
+    public function compressJavaScriptSource(string $javaScriptSourceCode): string
+    {
+        $this->initialize();
+        $fakeThis = null;
+        foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['t3lib/class.t3lib_div.php']['minifyJavaScript'] ?? [] as $hookMethod) {
+            try {
+                $parameters = ['script' => $javaScriptSourceCode];
+                $javaScriptSourceCode = GeneralUtility::callUserFunction($hookMethod, $parameters, $fakeThis);
+            } catch (\Exception $e) {
+                GeneralUtility::makeInstance(LogManager::class)->getLogger(__CLASS__)->warning('Error minifying Javascript: {file}, hook: {hook}', [
+                    'file' => $javaScriptSourceCode,
+                    'hook' => $hookMethod,
+                    'exception' => $e,
+                ]);
+            }
+        }
+        return $javaScriptSourceCode;
+    }
+
     /**
      * Compress a CSS string by removing comments and whitespace characters
      *
@@ -659,24 +669,12 @@ class ResourceCompressor
     }
 
     /**
-     * Determines the the JavaScript mime type
-     *
-     * The <script> tag only needs the type if the page is not rendered as HTML5.
-     * In TYPO3 Backend or when TSFE is not available we always use HTML5.
-     * For TYPO3 Frontend the configured config.doctype is evaluated.
-     *
-     * @return string
+     * Whenever HTML5 is used, do not use the "text/javascript" type attribute.
      */
     protected function getJavaScriptFileType(): string
     {
-        if (TYPO3_MODE === 'BE' || !isset($GLOBALS['TSFE']) || !($GLOBALS['TSFE'] instanceof TypoScriptFrontendController)) {
-            // Backend (or at least no TSFE), always HTML5
-            return '';
-        }
-        if (($GLOBALS['TSFE']->config['config']['doctype'] ?? 'html5') === 'html5') {
-            return '';
-        }
-        return 'text/javascript';
+        $docType = GeneralUtility::makeInstance(PageRenderer::class)->getDocType();
+        return $docType === DocType::html5 ? '' : 'text/javascript';
     }
 
     protected function getPathFixer(): RelativeCssPathFixer
