@@ -1,7 +1,6 @@
 <?php
-declare(strict_types = 1);
 
-namespace TYPO3\CMS\Core\Routing;
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -16,6 +15,8 @@ namespace TYPO3\CMS\Core\Routing;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Core\Routing;
+
 use TYPO3\CMS\Core\Utility\ArrayUtility;
 
 /**
@@ -23,53 +24,48 @@ use TYPO3\CMS\Core\Utility\ArrayUtility;
  */
 class PageArguments implements RouteResultInterface
 {
-    /**
-     * @var int
-     */
-    protected $pageId;
+    protected int $pageId;
+    protected string $pageType;
+    protected bool $dirty = false;
 
     /**
-     * @var string
+     * All (merged) arguments of this URI (routeArguments + dynamicArguments)
+     *
+     * @var array<string, string|array>
      */
-    protected $pageType;
+    protected array $arguments;
 
     /**
-     * @var array
+     * Route arguments mapped by static mappers
+     * "static" means the provided values in a URI maps to a finite number of values
+     * (routeArguments - "arguments mapped by non static mapper")
+     *
+     * @var array<string, string|array>
      */
-    protected $arguments;
+    protected array $staticArguments;
 
     /**
-     * @var array
+     * Route arguments, that have an infinite number of possible values
+     * AND query string arguments. These arguments require a cHash.
+     *
+     * @var array<string, string|array>
      */
-    protected $staticArguments;
+    protected array $dynamicArguments;
 
     /**
-     * @var array
+     * Arguments defined in and mapped by a route enhancer
+     *
+     * @var array<string, string|array>
      */
-    protected $dynamicArguments;
+    protected array $routeArguments;
 
     /**
-     * @var array
+     * Query arguments in the generated URI
+     *
+     * @var array<string, string|array>
      */
-    protected $routeArguments;
+    protected array $queryArguments = [];
 
-    /**
-     * @var array
-     */
-    protected $queryArguments = [];
-
-    /**
-     * @var bool
-     */
-    protected $dirty = false;
-
-    /**
-     * @param int $pageId
-     * @param string $pageType
-     * @param array $routeArguments
-     * @param array $staticArguments
-     * @param array $remainingArguments
-     */
     public function __construct(int $pageId, string $pageType, array $routeArguments, array $staticArguments = [], array $remainingArguments = [])
     {
         $this->pageId = $pageId;
@@ -83,49 +79,39 @@ class PageArguments implements RouteResultInterface
         }
     }
 
-    /**
-     * @return bool
-     */
     public function areDirty(): bool
     {
         return $this->dirty;
     }
 
     /**
-     * @return array
+     * @return array<string, string|array>
      */
     public function getRouteArguments(): array
     {
         return $this->routeArguments;
     }
 
-    /**
-     * @return int
-     */
     public function getPageId(): int
     {
         return $this->pageId;
     }
 
-    /**
-     * @return string
-     */
     public function getPageType(): string
     {
         return $this->pageType;
     }
 
     /**
-     * @param string $name
-     * @return mixed|null
+     * @return string|array<string, string|array>|null
      */
-    public function get(string $name)
+    public function get(string $name): mixed
     {
         return $this->arguments[$name] ?? null;
     }
 
     /**
-     * @return array
+     * @return array<string, string|array>
      */
     public function getArguments(): array
     {
@@ -133,7 +119,7 @@ class PageArguments implements RouteResultInterface
     }
 
     /**
-     * @return array
+     * @return array<string, string|array>
      */
     public function getStaticArguments(): array
     {
@@ -141,7 +127,7 @@ class PageArguments implements RouteResultInterface
     }
 
     /**
-     * @return array
+     * @return array<string, string|array>
      */
     public function getDynamicArguments(): array
     {
@@ -149,7 +135,7 @@ class PageArguments implements RouteResultInterface
     }
 
     /**
-     * @return array
+     * @return array<string, string|array>
      */
     public function getQueryArguments(): array
     {
@@ -157,7 +143,7 @@ class PageArguments implements RouteResultInterface
     }
 
     /**
-     * @param array $queryArguments
+     * @param array<string, string|array> $queryArguments
      */
     protected function updateQueryArguments(array $queryArguments)
     {
@@ -190,8 +176,7 @@ class PageArguments implements RouteResultInterface
     /**
      * Cleans empty array recursively.
      *
-     * @param array $array
-     * @return array
+     * @param array<string, string|array> $array
      */
     protected function clean(array $array): array
     {
@@ -212,8 +197,7 @@ class PageArguments implements RouteResultInterface
     /**
      * Sorts array keys recursively.
      *
-     * @param array $array
-     * @return array
+     * @param array<string, string|array> $array
      */
     protected function sort(array $array): array
     {
@@ -225,29 +209,23 @@ class PageArguments implements RouteResultInterface
     /**
      * Removes keys that are defined in $second from $first recursively.
      *
-     * @param array $first
-     * @param array $second
-     * @return array
+     * @param array<string, string|array> $first
+     * @param array<string, string|array> $second
      */
     protected function diff(array $first, array $second): array
     {
-        return ArrayUtility::arrayDiffAssocRecursive($first, $second);
+        return ArrayUtility::arrayDiffKeyRecursive($first, $second);
     }
 
-    /**
-     * @param mixed $offset
-     * @return bool
-     */
-    public function offsetExists($offset): bool
+    public function offsetExists(mixed $offset): bool
     {
         return $offset === 'pageId' || $offset === 'pageType' || isset($this->arguments[$offset]);
     }
 
     /**
-     * @param mixed $offset
-     * @return mixed
+     * @return string|array<string, string|array>|null
      */
-    public function offsetGet($offset)
+    public function offsetGet(mixed $offset): mixed
     {
         if ($offset === 'pageId') {
             return $this->getPageId();
@@ -258,19 +236,12 @@ class PageArguments implements RouteResultInterface
         return $this->arguments[$offset] ?? null;
     }
 
-    /**
-     * @param mixed $offset
-     * @param mixed $value
-     */
-    public function offsetSet($offset, $value)
+    public function offsetSet(mixed $offset, mixed $value): void
     {
         throw new \InvalidArgumentException('PageArguments cannot be modified.', 1538152266);
     }
 
-    /**
-     * @param mixed $offset
-     */
-    public function offsetUnset($offset)
+    public function offsetUnset(mixed $offset): void
     {
         throw new \InvalidArgumentException('PageArguments cannot be modified.', 1538152269);
     }
