@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 /*
  * This file is part of the TYPO3 CMS project.
  *
@@ -16,15 +18,17 @@
 namespace TYPO3\CMS\Core\Tree\TableConfiguration;
 
 use Psr\EventDispatcher\EventDispatcherInterface;
+use Symfony\Component\DependencyInjection\Attribute\Autoconfigure;
 use TYPO3\CMS\Backend\Tree\SortedTreeNodeCollection;
 use TYPO3\CMS\Backend\Tree\TreeNode;
 use TYPO3\CMS\Backend\Tree\TreeNodeCollection;
 use TYPO3\CMS\Backend\Utility\BackendUtility;
+use TYPO3\CMS\Core\Database\Connection;
 use TYPO3\CMS\Core\Database\ConnectionPool;
 use TYPO3\CMS\Core\Database\Query\Expression\ExpressionBuilder;
 use TYPO3\CMS\Core\Database\RelationHandler;
-use TYPO3\CMS\Core\Imaging\Icon;
 use TYPO3\CMS\Core\Imaging\IconFactory;
+use TYPO3\CMS\Core\Imaging\IconSize;
 use TYPO3\CMS\Core\Localization\LanguageService;
 use TYPO3\CMS\Core\Tree\Event\ModifyTreeDataEvent;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
@@ -32,140 +36,92 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
 /**
  * TCA tree data provider
  */
+#[Autoconfigure(public: true, shared: false)]
 class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvider
 {
-    /**
-     * @deprecated, will be removed in TYPO3 v11.0, use the EventDispatcher instead of Signal/Slot logic
-     */
-    const SIGNAL_PostProcessTreeData = 'PostProcessTreeData';
-    const MODE_CHILDREN = 1;
-    const MODE_PARENT = 2;
+    public const MODE_CHILDREN = 1;
+    public const MODE_PARENT = 2;
 
-    /**
-     * @var string
-     */
-    protected $tableName = '';
+    protected string $tableName = '';
 
     /**
      * @var string
      */
     protected $treeId = '';
 
-    /**
-     * @var string
-     */
-    protected $labelField = '';
+    protected string $labelField = '';
+
+    protected string $tableWhere = '';
 
     /**
-     * @var string
+     * @var self::MODE_*
      */
-    protected $tableWhere = '';
+    protected int $lookupMode = self::MODE_CHILDREN;
 
-    /**
-     * @var int
-     */
-    protected $lookupMode = self::MODE_CHILDREN;
+    protected string $lookupField = '';
 
-    /**
-     * @var string
-     */
-    protected $lookupField = '';
-
-    /**
-     * @var int
-     */
-    protected $rootUid = 0;
-
-    /**
-     * @var array
-     */
-    protected $idCache = [];
+    protected array $idCache = [];
 
     /**
      * Stores TCA-Configuration of the LookUpField in tableName
      *
-     * @var array
+     * @var array<string, mixed>
      */
-    protected $columnConfiguration;
+    protected array $columnConfiguration;
 
     /**
      * node sort values (the orderings from foreign_Table_where evaluation)
      *
-     * @var array
+     * @var array<string, mixed>
      */
-    protected $nodeSortValues = [];
+    protected array $nodeSortValues = [];
 
-    /**
-     * @var array TCEforms compiled TSConfig array
-     */
-    protected $generatedTSConfig = [];
-
-    /**
-     * @var EventDispatcherInterface
-     */
-    protected $eventDispatcher;
-
-    public function __construct(EventDispatcherInterface $eventDispatcher)
-    {
-        $this->eventDispatcher = $eventDispatcher;
-    }
+    public function __construct(protected EventDispatcherInterface $eventDispatcher) {}
 
     /**
      * Sets the label field
-     *
-     * @param string $labelField
      */
-    public function setLabelField($labelField)
+    public function setLabelField(string $labelField): void
     {
         $this->labelField = $labelField;
     }
 
     /**
      * Gets the label field
-     *
-     * @return string
      */
-    public function getLabelField()
+    public function getLabelField(): string
     {
         return $this->labelField;
     }
 
     /**
      * Sets the table name
-     *
-     * @param string $tableName
      */
-    public function setTableName($tableName)
+    public function setTableName(string $tableName): void
     {
         $this->tableName = $tableName;
     }
 
     /**
      * Gets the table name
-     *
-     * @return string
      */
-    public function getTableName()
+    public function getTableName(): string
     {
         return $this->tableName;
     }
 
     /**
      * Sets the lookup field
-     *
-     * @param string $lookupField
      */
-    public function setLookupField($lookupField)
+    public function setLookupField(string $lookupField): void
     {
         $this->lookupField = $lookupField;
     }
 
     /**
      * Gets the lookup field
-     *
-     * @return string
      */
-    public function getLookupField()
+    public function getLookupField(): string
     {
         return $this->lookupField;
     }
@@ -173,9 +129,9 @@ class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvide
     /**
      * Sets the lookup mode
      *
-     * @param int $lookupMode
+     * @param self::MODE_* $lookupMode
      */
-    public function setLookupMode($lookupMode)
+    public function setLookupMode(int $lookupMode): void
     {
         $this->lookupMode = $lookupMode;
     }
@@ -183,104 +139,70 @@ class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvide
     /**
      * Gets the lookup mode
      *
-     * @return int
+     * @return self::MODE_*
      */
-    public function getLookupMode()
+    public function getLookupMode(): int
     {
         return $this->lookupMode;
     }
 
     /**
      * Gets the nodes
-     *
-     * @param \TYPO3\CMS\Backend\Tree\TreeNode $node
      */
-    public function getNodes(TreeNode $node)
-    {
-    }
+    public function getNodes(TreeNode $node): void {}
 
     /**
      * Gets the root node
-     *
-     * @return \TYPO3\CMS\Core\Tree\TableConfiguration\DatabaseTreeNode
      */
-    public function getRoot()
+    public function getRoot(): DatabaseTreeNode
     {
         return $this->buildRepresentationForNode($this->treeData);
     }
 
-    /**
-     * Sets the root uid
-     *
-     * @param int $rootUid
-     */
-    public function setRootUid($rootUid)
-    {
-        $this->rootUid = $rootUid;
-    }
-
-    /**
-     * Gets the root uid
-     *
-     * @return int
-     */
-    public function getRootUid()
-    {
-        return $this->rootUid;
-    }
-
     /**
      * Sets the tableWhere clause
-     *
-     * @param string $tableWhere
      */
-    public function setTableWhere($tableWhere)
+    public function setTableWhere(string $tableWhere): void
     {
         $this->tableWhere = $tableWhere;
     }
 
     /**
      * Gets the tableWhere clause
-     *
-     * @return string
      */
-    public function getTableWhere()
+    public function getTableWhere(): string
     {
         return $this->tableWhere;
     }
 
     /**
-     * Builds a complete node including childs
-     *
-     * @param \TYPO3\CMS\Backend\Tree\TreeNode $basicNode
-     * @param \TYPO3\CMS\Core\Tree\TableConfiguration\DatabaseTreeNode|null $parent
-     * @param int $level
-     * @return \TYPO3\CMS\Core\Tree\TableConfiguration\DatabaseTreeNode Node object
+     * Builds a complete node including children
      */
-    protected function buildRepresentationForNode(TreeNode $basicNode, DatabaseTreeNode $parent = null, $level = 0)
+    protected function buildRepresentationForNode(TreeNode $basicNode, ?DatabaseTreeNode $parent = null, $level = 0): DatabaseTreeNode
     {
-        /** @var \TYPO3\CMS\Core\Tree\TableConfiguration\DatabaseTreeNode $node */
         $node = GeneralUtility::makeInstance(DatabaseTreeNode::class);
         $row = [];
         if ($basicNode->getId() == 0) {
             $node->setSelected(false);
-            $node->setExpanded(true);
-            $node->setLabel($this->getLanguageService()->sL($GLOBALS['TCA'][$this->tableName]['ctrl']['title']));
+            $node->setLabel($this->getLanguageService()?->sL($GLOBALS['TCA'][$this->tableName]['ctrl']['title']));
         } else {
-            $row = BackendUtility::getRecordWSOL($this->tableName, (int)$basicNode->getId(), '*', '', false);
+            if ($basicNode->getAdditionalData() === []) {
+                $row = BackendUtility::getRecordWSOL($this->tableName, (int)$basicNode->getId(), '*', '', false) ?? [];
+            } else {
+                // @todo: This is part of the category tree performance hack
+                $row = $basicNode->getAdditionalData();
+            }
             $node->setLabel(BackendUtility::getRecordTitle($this->tableName, $row) ?: $basicNode->getId());
             $node->setSelected(GeneralUtility::inList($this->getSelectedList(), $basicNode->getId()));
-            $node->setExpanded($this->isExpanded($basicNode));
         }
         $node->setId($basicNode->getId());
         $node->setSelectable(!GeneralUtility::inList($this->getNonSelectableLevelList(), (string)$level) && !in_array($basicNode->getId(), $this->getItemUnselectableList()));
-        $node->setSortValue($this->nodeSortValues[$basicNode->getId()]);
+        $node->setSortValue($this->nodeSortValues[$basicNode->getId()] ?? '');
         $iconFactory = GeneralUtility::makeInstance(IconFactory::class);
-        $node->setIcon($iconFactory->getIconForRecord($this->tableName, $row, Icon::SIZE_SMALL));
+        $node->setIcon($iconFactory->getIconForRecord($this->tableName, $row, IconSize::SMALL));
         $node->setParentNode($parent);
         if ($basicNode->hasChildNodes()) {
             $node->setHasChildren(true);
-            /** @var \TYPO3\CMS\Backend\Tree\SortedTreeNodeCollection $childNodes */
             $childNodes = GeneralUtility::makeInstance(SortedTreeNodeCollection::class);
             $tempNodes = [];
             foreach ($basicNode->getChildNodes() as $child) {
@@ -296,17 +218,15 @@ class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvide
     /**
      * Init the tree data
      */
-    public function initializeTreeData()
+    public function initializeTreeData(): void
     {
-        parent::initializeTreeData();
         $this->nodeSortValues = array_flip($this->itemWhiteList);
-        $this->columnConfiguration = $GLOBALS['TCA'][$this->getTableName()]['columns'][$this->getLookupField()]['config'];
-        if (isset($this->columnConfiguration['foreign_table']) && $this->columnConfiguration['foreign_table'] != $this->getTableName()) {
+        $this->columnConfiguration = $GLOBALS['TCA'][$this->getTableName()]['columns'][$this->lookupField]['config'] ?? [];
+        if (isset($this->columnConfiguration['foreign_table']) && $this->columnConfiguration['foreign_table'] !== $this->getTableName()) {
             throw new \InvalidArgumentException('TCA Tree configuration is invalid: tree for different node-Tables is not implemented yet', 1290944650);
         }
         $this->treeData = GeneralUtility::makeInstance(TreeNode::class);
         $this->loadTreeData();
-        /** @var ModifyTreeDataEvent $event */
         $event = $this->eventDispatcher->dispatch(new ModifyTreeDataEvent($this->treeData, $this));
         $this->treeData = $event->getTreeData();
     }
@@ -314,57 +234,75 @@ class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvide
     /**
      * Loads the tree data (all possible children)
      */
-    protected function loadTreeData()
+    protected function loadTreeData(): void
     {
-        $this->treeData->setId($this->getRootUid());
-        $this->treeData->setParentNode(null);
-        if ($this->levelMaximum >= 1) {
-            $childNodes = $this->getChildrenOf($this->treeData, 1);
-            if ($childNodes !== null) {
-                $this->treeData->setChildNodes($childNodes);
+        if ($this->getStartingPoints()) {
+            $startingPoints = $this->getStartingPoints();
+        } else {
+            $startingPoints = [0];
+        }
+
+        if (count($startingPoints) === 1) {
+            // Only one starting point is available, grab it and set it as root node
+            $startingPoint = current($startingPoints);
+            $this->treeData->setId((string)$startingPoint);
+            $this->treeData->setParentNode(null);
+
+            if ($this->levelMaximum >= 1) {
+                $childNodes = $this->getChildrenOf($this->treeData, 1);
+                if ($childNodes !== null) {
+                    $this->treeData->setChildNodes($childNodes);
+                }
             }
+        } else {
+            // The current tree implementation disallows multiple elements on root level, thus we have to work around
+            // this with a separate TreeNodeCollection that gets attached to the root node with uid 0. This has the
+            // nasty side effect we cannot avoid the root node being rendered.
+
+            $treeNodeCollection = GeneralUtility::makeInstance(TreeNodeCollection::class);
+            foreach ($startingPoints as $startingPoint) {
+                $treeData = GeneralUtility::makeInstance(TreeNode::class);
+                $treeData->setId((string)$startingPoint);
+
+                if ($this->levelMaximum >= 1) {
+                    $childNodes = $this->getChildrenOf($treeData, 1);
+                    if ($childNodes !== null) {
+                        $treeData->setChildNodes($childNodes);
+                    }
+                }
+                $treeNodeCollection->append($treeData);
+            }
+            $this->treeData->setId('0');
+            $this->treeData->setChildNodes($treeNodeCollection);
         }
     }
 
     /**
      * Gets node children
-     *
-     * @param \TYPO3\CMS\Backend\Tree\TreeNode $node
-     * @param int $level
-     * @return \TYPO3\CMS\Backend\Tree\TreeNodeCollection|null
      */
-    protected function getChildrenOf(TreeNode $node, $level)
+    protected function getChildrenOf(TreeNode $node, int $level): ?TreeNodeCollection
     {
         $nodeData = null;
-        if ($node->getId() !== 0) {
-            $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
-                ->getQueryBuilderForTable($this->getTableName());
-            $queryBuilder->getRestrictions()->removeAll();
-            $nodeData = $queryBuilder->select('*')
-                ->from($this->getTableName())
-                ->where(
-                    $queryBuilder->expr()->eq(
-                        'uid',
-                        $queryBuilder->createNamedParameter($node->getId(), \PDO::PARAM_INT)
-                    )
-                )
-                ->setMaxResults(1)
-                ->execute()
-                ->fetch();
+        if ($node->getId() !== 0 && $node->getId() !== '0') {
+            if (is_array($this->availableItems[(int)$node->getId()] ?? false)) {
+                // @todo: This is part of the category tree performance hack
+                $nodeData = $this->availableItems[(int)$node->getId()];
+            } else {
+                $nodeData = BackendUtility::getRecord($this->tableName, $node->getId(), '*', '', false);
+            }
         }
         if (empty($nodeData)) {
             $nodeData = [
                 'uid' => 0,
-                $this->getLookupField() => ''
+                $this->lookupField => '',
             ];
         }
         $storage = null;
         $children = $this->getRelatedRecords($nodeData);
         if (!empty($children)) {
-            /** @var \TYPO3\CMS\Backend\Tree\TreeNodeCollection $storage */
             $storage = GeneralUtility::makeInstance(TreeNodeCollection::class);
             foreach ($children as $child) {
-                $node = GeneralUtility::makeInstance(TreeNode::class);
+                $node = GeneralUtility::makeInstance(TreeNode::class, $this->availableItems[(int)$child] ?? []);
                 $node->setId($child);
                 if ($level < $this->levelMaximum) {
                     $children = $this->getChildrenOf($node, $level + 1);
@@ -380,20 +318,17 @@ class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvide
 
     /**
      * Gets related records depending on TCA configuration
-     *
-     * @param array $row
-     * @return array
      */
-    protected function getRelatedRecords(array $row)
+    protected function getRelatedRecords(array $row): array
     {
-        if ($this->getLookupMode() == self::MODE_PARENT) {
+        if ($this->getLookupMode() === self::MODE_PARENT) {
             $children = $this->getChildrenUidsFromParentRelation($row);
         } else {
             $children = $this->getChildrenUidsFromChildrenRelation($row);
         }
         $allowedArray = [];
         foreach ($children as $child) {
-            if (!in_array($child, $this->idCache) && in_array($child, $this->itemWhiteList)) {
+            if (!in_array($child, $this->idCache, true) && in_array($child, $this->itemWhiteList, true)) {
                 $allowedArray[] = $child;
             }
         }
@@ -403,52 +338,60 @@ class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvide
 
     /**
      * Gets related records depending on TCA configuration
-     *
-     * @param array $row
-     * @return array
      */
-    protected function getChildrenUidsFromParentRelation(array $row)
+    protected function getChildrenUidsFromParentRelation(array $row): array
     {
-        $uid = $row['uid'];
-        switch ((string)$this->columnConfiguration['type']) {
-            case 'inline':
-
-            case 'select':
-                if ($this->columnConfiguration['MM']) {
-                    /** @var \TYPO3\CMS\Core\Database\RelationHandler $dbGroup */
-                    $dbGroup = GeneralUtility::makeInstance(RelationHandler::class);
-                    // Dummy field for setting "look from other site"
-                    $this->columnConfiguration['MM_oppositeField'] = 'children';
-                    $dbGroup->start($row[$this->getLookupField()], $this->getTableName(), $this->columnConfiguration['MM'], $uid, $this->getTableName(), $this->columnConfiguration);
-                    $relatedUids = $dbGroup->tableArray[$this->getTableName()];
-                } elseif ($this->columnConfiguration['foreign_field']) {
-                    $relatedUids = $this->listFieldQuery($this->columnConfiguration['foreign_field'], $uid);
+        $uid = (int)$row['uid'];
+        if (in_array($this->columnConfiguration['type'] ?? '', ['select', 'category', 'inline', 'file'], true)) {
+            if ($this->columnConfiguration['MM'] ?? null) {
+                $dbGroup = GeneralUtility::makeInstance(RelationHandler::class);
+                // Dummy field for setting "look from other site"
+                $this->columnConfiguration['MM_opposite_field'] = 'children';
+                $dbGroup->start($row[$this->lookupField], $this->getTableName(), $this->columnConfiguration['MM'], $uid, $this->getTableName(), $this->columnConfiguration);
+                $relatedUids = $dbGroup->tableArray[$this->getTableName()];
+            } elseif ($this->columnConfiguration['foreign_field'] ?? null) {
+                $relatedUids = $this->listFieldQuery($this->columnConfiguration['foreign_field'], $uid);
+            } else {
+                // Check available items
+                if ($this->availableItems !== [] && $this->columnConfiguration['type'] === 'category') {
+                    // @todo: This is part of the category tree performance hack
+                    $relatedUids = [];
+                    foreach ($this->availableItems as $item) {
+                        if ($item[$this->lookupField] === $uid) {
+                            $relatedUids[$item['uid']] = $item['sorting'];
+                        }
+                    }
+                    if ($relatedUids !== []) {
+                        // Ensure sorting is kept
+                        asort($relatedUids);
+                        $relatedUids = array_keys($relatedUids);
+                    }
                 } else {
-                    $relatedUids = $this->listFieldQuery($this->getLookupField(), $uid);
+                    $relatedUids = $this->listFieldQuery($this->lookupField, $uid);
                 }
-                break;
-            default:
-                $relatedUids = $this->listFieldQuery($this->getLookupField(), $uid);
+            }
+        } else {
+            $relatedUids = $this->listFieldQuery($this->lookupField, $uid);
         }
+
         return $relatedUids;
     }
 
     /**
      * Gets related children records depending on TCA configuration
-     *
-     * @param array $row
-     * @return array
      */
-    protected function getChildrenUidsFromChildrenRelation(array $row)
+    protected function getChildrenUidsFromChildrenRelation(array $row): array
     {
         $relatedUids = [];
-        $uid = $row['uid'];
-        $value = $row[$this->getLookupField()];
+        $uid = (int)$row['uid'];
+        $value = (string)$row[$this->lookupField];
         switch ((string)$this->columnConfiguration['type']) {
             case 'inline':
+            case 'file':
                 // Intentional fall-through
             case 'select':
-                if ($this->columnConfiguration['MM']) {
+            case 'category':
+                if ($this->columnConfiguration['MM'] ?? false) {
                     $dbGroup = GeneralUtility::makeInstance(RelationHandler::class);
                     $dbGroup->start(
                         $value,
@@ -459,7 +402,7 @@ class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvide
                         $this->columnConfiguration
                     );
                     $relatedUids = $dbGroup->tableArray[$this->getTableName()];
-                } elseif ($this->columnConfiguration['foreign_field']) {
+                } elseif ($this->columnConfiguration['foreign_field'] ?? false) {
                     $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
                         ->getQueryBuilderForTable($this->getTableName());
                     $queryBuilder->getRestrictions()->removeAll();
@@ -468,11 +411,11 @@ class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvide
                         ->where(
                             $queryBuilder->expr()->eq(
                                 $this->columnConfiguration['foreign_field'],
-                                $queryBuilder->createNamedParameter($uid, \PDO::PARAM_INT)
+                                $queryBuilder->createNamedParameter($uid, Connection::PARAM_INT)
                             )
                         )
-                        ->execute()
-                        ->fetchAll();
+                        ->executeQuery()
+                        ->fetchAllAssociative();
 
                     if (!empty($records)) {
                         $relatedUids = array_column($records, 'uid');
@@ -494,16 +437,15 @@ class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvide
      * @param int $queryId the uid to search for
      * @return int[] all uids found
      */
-    protected function listFieldQuery($fieldName, $queryId)
+    protected function listFieldQuery(string $fieldName, int $queryId): array
     {
-        $queryId = (int)$queryId;
         $queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
             ->getQueryBuilderForTable($this->getTableName());
         $queryBuilder->getRestrictions()->removeAll();
 
         $queryBuilder->select('uid')
             ->from($this->getTableName())
-            ->where($queryBuilder->expr()->inSet($fieldName, $queryBuilder->quote($queryId)));
+            ->where($queryBuilder->expr()->inSet($fieldName, $queryBuilder->quote((string)$queryId)));
 
         if ($queryId === 0) {
             $queryBuilder->orWhere(
@@ -515,15 +457,10 @@ class DatabaseTreeDataProvider extends AbstractTableConfigurationTreeDataProvide
             );
         }
 
-        $records = $queryBuilder->execute()->fetchAll();
-        $uidArray = is_array($records) ? array_column($records, 'uid') : [];
-
-        return $uidArray;
+        $records = $queryBuilder->executeQuery()->fetchAllAssociative();
+        return array_column($records, 'uid');
     }
 
-    /**
-     * @return LanguageService|null
-     */
     protected function getLanguageService(): ?LanguageService
     {
         return $GLOBALS['LANG'] ?? null;
