@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 /*
  * This file is part of the TYPO3 CMS project.
  *
@@ -15,7 +17,11 @@
 
 namespace TYPO3\CMS\Core\Utility;
 
+use Psr\Log\LoggerInterface;
+use Symfony\Component\Process\Exception\RuntimeException;
+use Symfony\Component\Process\Process;
 use TYPO3\CMS\Core\Core\Environment;
+use TYPO3\CMS\Core\Log\LogManager;
 
 /**
  * Class to handle system commands.
@@ -25,7 +31,10 @@ use TYPO3\CMS\Core\Core\Environment;
  * or FALSE
  *
  * This class is meant to be used without instance:
+ *
+ * ```
  * $cmd = CommandUtility::getCommand ('awstats','perl');
+ * ```
  *
  * The data of this class is cached.
  * That means if a program is found once it don't have to be searched again.
@@ -37,7 +46,7 @@ use TYPO3\CMS\Core\Core\Environment;
  * checkCommand() returns TRUE if a command is available
  *
  * Search paths that are included:
- * $TYPO3_CONF_VARS['GFX']['processor_path_lzw'] or $TYPO3_CONF_VARS['GFX']['processor_path']
+ * $TYPO3_CONF_VARS['GFX']['processor_path']
  * $TYPO3_CONF_VARS['SYS']['binPath']
  * $GLOBALS['_SERVER']['PATH']
  * '/usr/bin/,/usr/local/bin/' on Unix
@@ -49,10 +58,8 @@ class CommandUtility
 {
     /**
      * Tells if object is already initialized
-     *
-     * @var bool
      */
-    protected static $initialized = false;
+    protected static bool $initialized = false;
 
     /**
      * Contains application list. This is an array with the following structure:
@@ -61,29 +68,55 @@ class CommandUtility
      * - valid => TRUE or FALSE
      * Array key is identical to 'app'.
      *
-     * @var array
+     * @var array<string, array{app: string, path: string, valid: bool}>
      */
-    protected static $applications = [];
+    protected static array $applications = [];
 
     /**
      * Paths where to search for applications
      *
-     * @var array
+     * The key is a path. The value is either the same path, or false if the path is not valid.
+     *
+     * @var array<string, string|false>|null
      */
-    protected static $paths;
+    protected static ?array $paths = null;
 
     /**
-     * Wrapper function for php exec function
-     * Needs to be central to have better control and possible fix for issues
+     * Execute a shell command.
      *
-     * @param string $command
-     * @param array|null $output
-     * @param int $returnValue
-     * @return string
+     * Needs to be central to have better control and possible fix for issues. Is a wrapper for Symfony's Process
+     * component.
+     *
+     * @see Process
      */
-    public static function exec($command, &$output = null, &$returnValue = 0)
+    public static function exec(string|array $command, ?array &$output = null, int &$returnValue = 0): string|false
     {
-        return exec($command, $output, $returnValue);
+        if (is_string($command)) {
+            $process = Process::fromShellCommandline($command);
+        } else {
+            $process = new Process($command);
+        }
+
+        try {
+            $returnValue = $process->run();
+        } catch (RuntimeException $runtimeException) {
+            self::getLogger()->warning('Executing command "{command}" failed.', [
+                'command' => $command,
+                'exception' => $runtimeException,
+            ]);
+
+            return false;
+        }
+
+        $processOutput = $process->getOutput();
+        if (str_ends_with($processOutput, PHP_EOL)) {
+            // Last \n is ignored by PHP exec(): https://github.com/php/php-src/blob/b675db4c56dd0de4ea1f5195d587ed90f0096ed8/ext/standard/exec.c#L148
+            $processOutput = substr($processOutput, 0, -1);
+        }
+        $output = explode(PHP_EOL, $processOutput);
+
+        return rtrim(strrchr($processOutput, PHP_EOL) ?: $processOutput);
+
     }
 
     /**
@@ -94,12 +127,12 @@ class CommandUtility
      * @param string $path Override the default path (e.g. used by the install tool)
      * @return string Compiled command that deals with ImageMagick & GraphicsMagick
      */
-    public static function imageMagickCommand($command, $parameters, $path = '')
+    public static function imageMagickCommand(string $command, string $parameters, string $path = ''): string
     {
         $gfxConf = $GLOBALS['TYPO3_CONF_VARS']['GFX'];
         $isExt = Environment::isWindows() ? '.exe' : '';
         if (!$path) {
-            $path = $gfxConf['processor_path'];
+            $path = (string)($gfxConf['processor_path'] ?? '');
         }
         $path = GeneralUtility::fixWindowsFilePath($path);
         // This is only used internally, has no effect outside
@@ -118,14 +151,18 @@ class CommandUtility
         }
         // strip profile information for thumbnails and reduce their size
         if ($parameters && $command !== 'identify') {
+            // Use legacy processor_stripColorProfileCommand setting if defined, otherwise
+            // use the preferred configuration option processor_stripColorProfileParameters
+            $stripColorProfileCommand = $gfxConf['processor_stripColorProfileCommand'] ??
+                implode(' ', array_map(CommandUtility::escapeShellArgument(...), $gfxConf['processor_stripColorProfileParameters'] ?? []));
             // Determine whether the strip profile action has be disabled by TypoScript:
             if ($gfxConf['processor_stripColorProfileByDefault']
-                && $gfxConf['processor_stripColorProfileCommand'] !== ''
-                && strpos($parameters, $gfxConf['processor_stripColorProfileCommand']) === false
+                && $stripColorProfileCommand !== ''
                 && $parameters !== '-version'
-                && strpos($parameters, '###SkipStripProfile###') === false
+                && !str_contains($parameters, $stripColorProfileCommand)
+                && !str_contains($parameters, '###SkipStripProfile###')
             ) {
-                $parameters = $gfxConf['processor_stripColorProfileCommand'] . ' ' . $parameters;
+                $parameters = $stripColorProfileCommand . ' ' . $parameters;
             } else {
                 $parameters = str_replace('###SkipStripProfile###', '', $parameters);
             }
@@ -136,7 +173,7 @@ class CommandUtility
         }
         // set interlace parameter for convert command
         if ($command !== 'identify' && $gfxConf['processor_interlace']) {
-            $parameters = '-interlace ' . $gfxConf['processor_interlace'] . ' ' . $parameters;
+            $parameters = '-interlace ' . CommandUtility::escapeShellArgument($gfxConf['processor_interlace']) . ' ' . $parameters;
         }
         $cmdLine = $path . ' ' . $parameters;
         // It is needed to change the parameters order when a mask image has been specified
@@ -157,16 +194,16 @@ class CommandUtility
      * Checks if a command is valid or not, updates global variables
      *
      * @param string $cmd The command that should be executed. eg: "convert"
-     * @param string $handler Executer for the command. eg: "perl"
-     * @return bool FALSE if cmd is not found, or -1 if the handler is not found
+     * @param string $handler Executor for the command. eg: "perl"
+     * @return bool|int True if the command is valid; False if cmd is not found; -1 if the handler is not found
      */
-    public static function checkCommand($cmd, $handler = '')
+    public static function checkCommand(string $cmd, string $handler = ''): bool|int
     {
         if (!self::init()) {
             return false;
         }
 
-        if ($handler && !self::checkCommand($handler)) {
+        if ($handler !== '' && !self::checkCommand($handler)) {
             return -1;
         }
         // Already checked and valid
@@ -212,8 +249,11 @@ class CommandUtility
         // Try to get the executable with the command 'which'.
         // It does the same like already done, but maybe on other paths
         if (!Environment::isWindows()) {
-            $cmd = @self::exec('which ' . self::escapeShellArgument($cmd));
-            if (@is_executable($cmd)) {
+            $output = null;
+            $returnValue = 0;
+            $cmd = @self::exec('which ' . self::escapeShellArgument($cmd), $output, $returnValue);
+
+            if ($returnValue === 0) {
                 self::$applications[$cmd]['app'] = $cmd;
                 self::$applications[$cmd]['path'] = PathUtility::dirname($cmd) . '/';
                 self::$applications[$cmd]['valid'] = true;
@@ -230,9 +270,9 @@ class CommandUtility
      * @param string $cmd The command that should be executed. eg: "convert"
      * @param string $handler Handler (executor) for the command. eg: "perl"
      * @param string $handlerOpt Options for the handler, like '-w' for "perl"
-     * @return mixed Returns command string, or FALSE if cmd is not found, or -1 if the handler is not found
+     * @return string|bool|int Returns command string, or FALSE if cmd is not found, or -1 if the handler is not found
      */
-    public static function getCommand($cmd, $handler = '', $handlerOpt = '')
+    public static function getCommand(string $cmd, string $handler = '', string $handlerOpt = ''): string|bool|int
     {
         if (!self::init()) {
             return false;
@@ -262,7 +302,7 @@ class CommandUtility
      *
      * @param string $paths Comma separated list of extra paths where a command should be searched. Relative paths (without leading "/") are prepend with public web path
      */
-    public static function addPaths($paths)
+    public static function addPaths(string $paths): void
     {
         self::initPaths($paths);
     }
@@ -271,32 +311,23 @@ class CommandUtility
      * Returns an array of search paths
      *
      * @param bool $addInvalid If set the array contains invalid path too. Then the key is the path and the value is empty
-     * @return array Array of search paths (empty if exec is disabled)
+     * @return array<string, string|false> Array of search paths (empty if exec is disabled)
      */
-    public static function getPaths($addInvalid = false)
+    public static function getPaths(bool $addInvalid = false): array
     {
         if (!self::init()) {
             return [];
         }
 
-        $paths = self::$paths;
-
-        if (!$addInvalid) {
-            foreach ($paths as $path => $validPath) {
-                if (!$validPath) {
-                    unset($paths[$path]);
-                }
-            }
-        }
-        return $paths;
+        return $addInvalid
+            ? self::$paths
+            : array_filter(self::$paths);
     }
 
     /**
      * Initializes this class
-     *
-     * @return bool
      */
-    protected static function init()
+    protected static function init(): bool
     {
         if ($GLOBALS['TYPO3_CONF_VARS']['BE']['disable_exec_function']) {
             return false;
@@ -314,7 +345,7 @@ class CommandUtility
      *
      * @param string $paths Comma separated list of extra paths where a command should be searched. Relative paths (without leading "/") are prepend with public web path
      */
-    protected static function initPaths($paths = '')
+    protected static function initPaths(string $paths = ''): void
     {
         $doCheck = false;
 
@@ -326,18 +357,16 @@ class CommandUtility
         // Merge the submitted paths array to the global
         if ($paths) {
             $paths = GeneralUtility::trimExplode(',', $paths, true);
-            if (is_array($paths)) {
-                foreach ($paths as $path) {
-                    // Make absolute path of relative
-                    if (!preg_match('#^/#', $path)) {
-                        $path = Environment::getPublicPath() . '/' . $path;
-                    }
-                    if (!isset(self::$paths[$path])) {
-                        if (@is_dir($path)) {
-                            self::$paths[$path] = $path;
-                        } else {
-                            self::$paths[$path] = false;
-                        }
+            foreach ($paths as $path) {
+                // Make absolute path of relative
+                if (!str_starts_with($path, '/')) {
+                    $path = Environment::getPublicPath() . '/' . $path;
+                }
+                if (!isset(self::$paths[$path])) {
+                    if (@is_dir($path)) {
+                        self::$paths[$path] = $path;
+                    } else {
+                        self::$paths[$path] = false;
                     }
                 }
             }
@@ -356,9 +385,9 @@ class CommandUtility
     /**
      * Processes and returns the paths from $GLOBALS['TYPO3_CONF_VARS']['SYS']['binSetup']
      *
-     * @return array Array of commands and path
+     * @return array<string, array{app: string, path: string, valid: bool}> Array of commands and path
      */
-    protected static function getConfiguredApps()
+    protected static function getConfiguredApps(): array
     {
         $cmdArr = [];
 
@@ -382,16 +411,15 @@ class CommandUtility
     /**
      * Sets the search paths from different sources, internal
      *
-     * @return array Array of absolute paths (keys and values are equal)
+     * @return array<string, string> Array of absolute paths (keys and values are equal)
      */
-    protected static function getPathsInternal()
+    protected static function getPathsInternal(): array
     {
         $pathsArr = [];
         $sysPathArr = [];
 
         // Image magick paths first
-        // processor_path_lzw take precedence over processor_path
-        if ($imPath = $GLOBALS['TYPO3_CONF_VARS']['GFX']['processor_path_lzw'] ?: $GLOBALS['TYPO3_CONF_VARS']['GFX']['processor_path']) {
+        if ($imPath = $GLOBALS['TYPO3_CONF_VARS']['GFX']['processor_path']) {
             $imPath = self::fixPath($imPath);
             $pathsArr[$imPath] = $imPath;
         }
@@ -424,9 +452,7 @@ class CommandUtility
             ]);
         }
 
-        $pathsArr = array_merge($pathsArr, $sysPathArr);
-
-        return $pathsArr;
+        return array_merge($pathsArr, $sysPathArr);
     }
 
     /**
@@ -435,7 +461,7 @@ class CommandUtility
      * @param string $path Input path
      * @return string Output path
      */
-    protected static function fixPath($path)
+    protected static function fixPath(string $path): string
     {
         return str_replace('//', '/', $path . '/');
     }
@@ -448,13 +474,15 @@ class CommandUtility
      * @param string[] $input Input arguments to be escaped
      * @return string[] Escaped shell arguments
      */
-    public static function escapeShellArguments(array $input)
+    public static function escapeShellArguments(array $input): array
     {
         $isUTF8Filesystem = !empty($GLOBALS['TYPO3_CONF_VARS']['SYS']['UTF8filesystem']);
         $currentLocale = false;
         if ($isUTF8Filesystem) {
-            $currentLocale = setlocale(LC_CTYPE, '0');
-            setlocale(LC_CTYPE, $GLOBALS['TYPO3_CONF_VARS']['SYS']['systemLocale']);
+            if ($GLOBALS['TYPO3_CONF_VARS']['SYS']['systemLocale'] ?? false) {
+                $currentLocale = setlocale(LC_CTYPE, '0');
+                setlocale(LC_CTYPE, $GLOBALS['TYPO3_CONF_VARS']['SYS']['systemLocale']);
+            }
         }
 
         $output = array_map('escapeshellarg', $input);
@@ -504,8 +532,13 @@ class CommandUtility
      * @param string $input Input-argument to be escaped
      * @return string Escaped shell argument
      */
-    public static function escapeShellArgument($input)
+    public static function escapeShellArgument(string $input): string
     {
         return self::escapeShellArguments([$input])[0];
     }
+
+    protected static function getLogger(): LoggerInterface
+    {
+        return GeneralUtility::makeInstance(LogManager::class)->getLogger(__CLASS__);
+    }
 }
