@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 /*
  * This file is part of the TYPO3 CMS project.
  *
@@ -23,13 +25,13 @@ class MathUtility
     /**
      * Forces the integer $theInt into the boundaries of $min and $max. If the $theInt is FALSE then the $defaultValue is applied.
      *
-     * @param int $theInt Input value
+     * @param mixed $theInt Input value - will be cast to int if non-integer value is passed.
      * @param int $min Lower limit
      * @param int $max Higher limit
      * @param int $defaultValue Default value if input is FALSE.
      * @return int The input value forced into the boundaries of $min and $max
      */
-    public static function forceIntegerInRange($theInt, $min, $max = 2000000000, $defaultValue = 0)
+    public static function forceIntegerInRange(mixed $theInt, int $min, int $max = 2000000000, int $defaultValue = 0): int
     {
         // Returns $theInt as an integer in the integerspace from $min to $max
         $theInt = (int)$theInt;
@@ -49,34 +51,51 @@ class MathUtility
 
     /**
      * Returns $theInt if it is greater than zero, otherwise returns zero.
-     *
-     * @param int $theInt Integer string to process
-     * @return int
+     * @deprecated will be removed in TYPO3 v14.0, use max() instead.
      */
-    public static function convertToPositiveInteger($theInt)
+    public static function convertToPositiveInteger(mixed $theInt): int
     {
-        $theInt = (int)$theInt;
-        if ($theInt < 0) {
-            $theInt = 0;
-        }
-        return $theInt;
+        trigger_error(
+            __METHOD__ . ' is deprecated and will be removed in TYPO3 v14. Use max() instead.',
+            E_USER_DEPRECATED
+        );
+        return self::forceIntegerInRange($theInt, 0, PHP_INT_MAX);
     }
 
     /**
      * Tests if the input can be interpreted as integer.
      *
+     * Note: "0" will return true while any other number with a leading 0 (including multiple zeroes) will be false.
+     *
      * Note: Integer casting from objects or arrays is considered undefined and thus will return false.
      *
      * @see https://php.net/manual/en/language.types.integer.php#language.types.integer.casting.from-other
      * @param mixed $var Any input variable to test
      * @return bool Returns TRUE if string is an integer
      */
-    public static function canBeInterpretedAsInteger($var)
+    public static function canBeInterpretedAsInteger(mixed $var): bool
     {
-        if ($var === '' || is_object($var) || is_array($var)) {
-            return false;
-        }
-        return (string)(int)$var === (string)$var;
+        return match (gettype($var)) {
+            'integer' => true,
+            // Due to historical reasons `TRUE` is correctly interpreted as integer
+            // but `FALSE` not even if a (int) cast would return `0` and keeping it
+            // we can simply return the boolean value to have the same behaviour and
+            // still avoiding type casting chain.
+            'boolean' => $var,
+            // We use a type casting chain here to ensure that value is the same after
+            // casting and eliminated invalid stuff from it. The `@` silence operator
+            // can look weird here but is required to avoid enforced casting issues
+            // with PHP 8.5.0 and newer.
+            'string' => (string)@(int)$var === $var,
+            // We use a type casting chain here to ensure that value is the same after
+            // casting and eliminated invalid stuff from it. The `@` silence operator
+            // can look weird here but is required to avoid enforced casting issues
+            // with PHP 8.5.0 and newer.
+            // gettype() returns `double` for `float values`
+            'double' => !is_nan($var) && (string)@(int)$var === (string)$var,
+            // non-scalar like array, object, resource, NULL or unknown_type
+            default => false,
+        };
     }
 
     /**
@@ -88,7 +107,7 @@ class MathUtility
      * @param mixed $var Any input variable to test
      * @return bool Returns TRUE if string is a float
      */
-    public static function canBeInterpretedAsFloat($var)
+    public static function canBeInterpretedAsFloat(mixed $var): bool
     {
         $pattern_lnum = '[0-9]+';
         $pattern_dnum = '([0-9]*[\.]' . $pattern_lnum . ')|(' . $pattern_lnum . '[\.][0-9]*)';
@@ -98,7 +117,7 @@ class MathUtility
             return false;
         }
 
-        $matches = preg_match('/^' . $pattern_exp_dnum . '$/', $var);
+        $matches = preg_match('/^' . $pattern_exp_dnum . '$/', (string)$var);
         return $matches === 1;
     }
 
@@ -106,10 +125,10 @@ class MathUtility
      * Calculates the input by +,-,*,/,%,^ with priority to + and -
      *
      * @param string $string Input string, eg "123 + 456 / 789 - 4
-     * @return int Calculated value. Or error string.
+     * @return float|string Calculated value. Or error string.
      * @see \TYPO3\CMS\Core\Utility\MathUtility::calculateWithParentheses()
      */
-    public static function calculateWithPriorityToAdditionAndSubtraction($string)
+    public static function calculateWithPriorityToAdditionAndSubtraction(string $string): float|string
     {
         // Removing all whitespace
         $string = preg_replace('/[[:space:]]*/', '', $string);
@@ -165,11 +184,11 @@ class MathUtility
      * Calculates the input with parenthesis levels
      *
      * @param string $string Input string, eg "(123 + 456) / 789 - 4
-     * @return int Calculated value. Or error string.
+     * @return string Calculated value. Or error string.
      * @see calculateWithPriorityToAdditionAndSubtraction()
      * @see \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer::stdWrap()
      */
-    public static function calculateWithParentheses($string)
+    public static function calculateWithParentheses(string $string): string
     {
         $securC = 100;
         do {
@@ -194,20 +213,18 @@ class MathUtility
     /**
      * Checks whether the given number $value is an integer in the range [$minimum;$maximum]
      *
-     * @param int $value Integer value to check
+     * @param mixed $value Integer value to check. If not an integer this method always returns false.
      * @param int $minimum Lower boundary of the range
      * @param int $maximum Upper boundary of the range
-     * @return bool
      */
-    public static function isIntegerInRange($value, $minimum, $maximum)
+    public static function isIntegerInRange(mixed $value, int $minimum, int $maximum): bool
     {
         $value = filter_var($value, FILTER_VALIDATE_INT, [
             'options' => [
                 'min_range' => $minimum,
-                'max_range' => $maximum
-            ]
+                'max_range' => $maximum,
+            ],
         ]);
-        $isInRange = is_int($value);
-        return $isInRange;
+        return is_int($value);
     }
 }
