@@ -17,195 +17,64 @@ declare(strict_types=1);
 
 namespace TYPO3\CMS\Extbase\Mvc\Web\Routing;
 
-use TYPO3\CMS\Backend\Routing\Exception\ResourceNotFoundException;
+use Symfony\Component\DependencyInjection\Attribute\Autoconfigure;
 use TYPO3\CMS\Backend\Routing\Exception\RouteNotFoundException;
+use TYPO3\CMS\Backend\Routing\Route;
+use TYPO3\CMS\Core\Http\ApplicationType;
 use TYPO3\CMS\Core\Utility\ArrayUtility;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Core\Utility\HttpUtility;
-use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
-use TYPO3\CMS\Extbase\DomainObject\AbstractDomainObject;
 use TYPO3\CMS\Extbase\DomainObject\AbstractValueObject;
+use TYPO3\CMS\Extbase\DomainObject\DomainObjectInterface;
 use TYPO3\CMS\Extbase\Mvc\Exception\InvalidArgumentValueException;
-use TYPO3\CMS\Extbase\Mvc\Request;
+use TYPO3\CMS\Extbase\Mvc\RequestInterface;
 use TYPO3\CMS\Extbase\Persistence\Generic\LazyLoadingProxy;
-use TYPO3\CMS\Extbase\Service\EnvironmentService;
 use TYPO3\CMS\Extbase\Service\ExtensionService;
 use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
 
 /**
- * An URI Builder
+ * URI Builder for extbase requests.
  */
+#[Autoconfigure(public: true, shared: false)]
 class UriBuilder
 {
-    /**
-     * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
-     */
-    protected $configurationManager;
+    protected RequestInterface $request;
 
-    /**
-     * @var \TYPO3\CMS\Extbase\Service\ExtensionService
-     */
-    protected $extensionService;
+    protected array $arguments = [];
+    protected array $lastArguments = [];
+    protected string $section = '';
+    protected bool $createAbsoluteUri = false;
+    protected ?string $absoluteUriScheme = null;
+    protected bool|string|int $addQueryString = false;
+    protected array $argumentsToBeExcludedFromQueryString = [];
+    protected bool $linkAccessRestrictedPages = false;
+    protected ?int $targetPageUid = null;
+    protected int $targetPageType = 0;
+    protected ?string $language = null;
+    protected bool $noCache = false;
+    protected string $format = '';
+    protected ?string $argumentPrefix = null;
 
-    /**
-     * An instance of \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer
-     *
-     * @var \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer
-     */
-    protected $contentObject;
-
-    /**
-     * @var Request|null
-     */
-    protected $request;
-
-    /**
-     * @var array
-     */
-    protected $arguments = [];
-
-    /**
-     * Arguments which have been used for building the last URI
-     *
-     * @var array
-     */
-    protected $lastArguments = [];
-
-    /**
-     * @var string
-     */
-    protected $section = '';
-
-    /**
-     * @var bool
-     */
-    protected $createAbsoluteUri = false;
-
-    /**
-     * @var string
-     */
-    protected $absoluteUriScheme;
-
-    /**
-     * @var bool
-     */
-    protected $addQueryString = false;
-
-    /**
-     * @var string
-     */
-    protected $addQueryStringMethod = '';
-
-    /**
-     * @var array
-     */
-    protected $argumentsToBeExcludedFromQueryString = [];
-
-    /**
-     * @var bool
-     */
-    protected $linkAccessRestrictedPages = false;
-
-    /**
-     * @var int|null
-     */
-    protected $targetPageUid;
-
-    /**
-     * @var int
-     */
-    protected $targetPageType = 0;
-
-    /**
-     * @var string
-     */
-    protected $language;
-
-    /**
-     * @var bool
-     */
-    protected $noCache = false;
-
-    /**
-     * @var string
-     */
-    protected $format = '';
-
-    /**
-     * @var string|null
-     */
-    protected $argumentPrefix;
-
-    /**
-     * @var \TYPO3\CMS\Extbase\Service\EnvironmentService
-     */
-    protected $environmentService;
-
-    /**
-     * @param \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface $configurationManager
-     * @internal only to be used within Extbase, not part of TYPO3 Core API.
-     */
-    public function injectConfigurationManager(ConfigurationManagerInterface $configurationManager): void
-    {
-        $this->configurationManager = $configurationManager;
-    }
-
-    /**
-     * @param \TYPO3\CMS\Extbase\Service\ExtensionService $extensionService
-     * @internal only to be used within Extbase, not part of TYPO3 Core API.
-     */
-    public function injectExtensionService(ExtensionService $extensionService): void
-    {
-        $this->extensionService = $extensionService;
-    }
-
-    /**
-     * @param \TYPO3\CMS\Extbase\Service\EnvironmentService $environmentService
-     * @internal only to be used within Extbase, not part of TYPO3 Core API.
-     */
-    public function injectEnvironmentService(EnvironmentService $environmentService): void
-    {
-        $this->environmentService = $environmentService;
-    }
-
-    /**
-     * Life-cycle method that is called by the DI container as soon as this object is completely built
-     * @internal only to be used within Extbase, not part of TYPO3 Core API.
-     */
-    public function initializeObject(): void
-    {
-        $this->contentObject = $this->configurationManager->getContentObject()
-            ?? GeneralUtility::makeInstance(ContentObjectRenderer::class);
-    }
+    public function __construct(
+        protected readonly ExtensionService $extensionService,
+    ) {}
 
     /**
      * Sets the current request
      *
-     * @param Request $request
      * @return static the current UriBuilder to allow method chaining
-     * @internal only to be used within Extbase, not part of TYPO3 Core API.
      */
-    public function setRequest(Request $request): UriBuilder
+    public function setRequest(RequestInterface $request): UriBuilder
     {
         $this->request = $request;
         return $this;
     }
 
-    /**
-     * @return Request|null
-     * @internal only to be used within Extbase, not part of TYPO3 Core API.
-     */
-    public function getRequest(): ?Request
-    {
-        return $this->request;
-    }
-
     /**
      * Additional query parameters.
      * If you want to "prefix" arguments, you can pass in multidimensional arrays:
      * array('prefix1' => array('foo' => 'bar')) gets "&prefix1[foo]=bar"
      *
-     * @param array $arguments
      * @return static the current UriBuilder to allow method chaining
      */
     public function setArguments(array $arguments): UriBuilder
@@ -215,7 +84,6 @@ class UriBuilder
     }
 
     /**
-     * @return array
      * @internal
      */
     public function getArguments(): array
@@ -226,7 +94,6 @@ class UriBuilder
     /**
      * If specified, adds a given HTML anchor to the URI (#...)
      *
-     * @param string $section
      * @return static the current UriBuilder to allow method chaining
      */
     public function setSection(string $section): UriBuilder
@@ -236,7 +103,6 @@ class UriBuilder
     }
 
     /**
-     * @return string
      * @internal
      */
     public function getSection(): string
@@ -247,7 +113,6 @@ class UriBuilder
     /**
      * Specifies the format of the target (e.g. "html" or "xml")
      *
-     * @param string $format
      * @return static the current UriBuilder to allow method chaining
      */
     public function setFormat(string $format): UriBuilder
@@ -257,7 +122,6 @@ class UriBuilder
     }
 
     /**
-     * @return string
      * @internal
      */
     public function getFormat(): string
@@ -268,7 +132,6 @@ class UriBuilder
     /**
      * If set, the URI is prepended with the current base URI. Defaults to FALSE.
      *
-     * @param bool $createAbsoluteUri
      * @return static the current UriBuilder to allow method chaining
      */
     public function setCreateAbsoluteUri(bool $createAbsoluteUri): UriBuilder
@@ -278,7 +141,6 @@ class UriBuilder
     }
 
     /**
-     * @return bool
      * @internal
      */
     public function getCreateAbsoluteUri(): bool
@@ -287,7 +149,6 @@ class UriBuilder
     }
 
     /**
-     * @return string|null
      * @internal only to be used within Extbase, not part of TYPO3 Core API.
      */
     public function getAbsoluteUriScheme(): ?string
@@ -309,8 +170,6 @@ class UriBuilder
 
     /**
      * Enforces a URI / link to a page to a specific language (or use "current")
-     * @param string|null $language
-     * @return UriBuilder
      */
     public function setLanguage(?string $language): UriBuilder
     {
@@ -318,76 +177,44 @@ class UriBuilder
         return $this;
     }
 
+    /**
+     * @internal only to be used within Extbase, not part of TYPO3 Core API.
+     */
     public function getLanguage(): ?string
     {
         return $this->language;
     }
 
     /**
-     * If set, the current query parameters will be merged with $this->arguments. Defaults to FALSE.
+     * If set, the current query parameters will be merged with $this->arguments in backend context.
+     * In frontend context, setting this property will only include mapped query arguments from the
+     * Page Routing. To include any - possible "unsafe" - GET parameters, the property has to be set
+     * to "untrusted". Defaults to FALSE.
      *
-     * @param bool $addQueryString
+     * @param bool|string|int $addQueryString is set to "1", "true", "0", "false" or "untrusted"
      * @return static the current UriBuilder to allow method chaining
-     * @see https://docs.typo3.org/m/typo3/reference-typoscript/master/en-us/Functions/Typolink.html#addquerystring
+     * @see https://docs.typo3.org/m/typo3/reference-typoscript/main/en-us/Functions/Typolink.html#addquerystring
      */
-    public function setAddQueryString(bool $addQueryString): UriBuilder
+    public function setAddQueryString(bool|string|int $addQueryString): UriBuilder
     {
         $this->addQueryString = $addQueryString;
         return $this;
     }
 
     /**
-     * @return bool
      * @internal
      */
-    public function getAddQueryString(): bool
+    public function getAddQueryString(): bool|string|int
     {
         return $this->addQueryString;
     }
 
-    /**
-     * Sets the method to get the addQueryString parameters. Defaults to an empty string
-     * which results in using GeneralUtility::_GET(). Possible values are
-     *
-     * + ''      -> uses GeneralUtility::_GET()
-     * + '0'     -> uses GeneralUtility::_GET()
-     * + 'GET'   -> uses GeneralUtility::_GET()
-     * + '<any>' -> uses parse_str(GeneralUtility::getIndpEnv('QUERY_STRING'))
-     *              (<any> refers to literally everything else than previously mentioned values)
-     *
-     * @param string $addQueryStringMethod
-     * @return static the current UriBuilder to allow method chaining
-     * @see https://docs.typo3.org/m/typo3/reference-typoscript/master/en-us/Functions/Typolink.html#addquerystring
-     */
-    public function setAddQueryStringMethod(string $addQueryStringMethod): UriBuilder
-    {
-        if ($addQueryStringMethod === 'POST') {
-            trigger_error('Assigning addQueryStringMethod = POST is not supported anymore since TYPO3 v10.0.', E_USER_WARNING);
-            $addQueryStringMethod = null;
-        } elseif ($addQueryStringMethod === 'GET,POST' || $addQueryStringMethod === 'POST,GET') {
-            trigger_error('Assigning addQueryStringMethod = GET,POST or POST,GET is not supported anymore since TYPO3 v10.0 - falling back to GET.', E_USER_WARNING);
-            $addQueryStringMethod = 'GET';
-        }
-        $this->addQueryStringMethod = $addQueryStringMethod;
-        return $this;
-    }
-
-    /**
-     * @return string
-     * @internal
-     */
-    public function getAddQueryStringMethod(): string
-    {
-        return $this->addQueryStringMethod;
-    }
-
     /**
      * A list of arguments to be excluded from the query parameters
      * Only active if addQueryString is set
      *
-     * @param array $argumentsToBeExcludedFromQueryString
      * @return static the current UriBuilder to allow method chaining
-     * @see https://docs.typo3.org/m/typo3/reference-typoscript/master/en-us/Functions/Typolink.html#addquerystring
+     * @see https://docs.typo3.org/m/typo3/reference-typoscript/main/en-us/Functions/Typolink.html#addquerystring
      * @see setAddQueryString()
      */
     public function setArgumentsToBeExcludedFromQueryString(array $argumentsToBeExcludedFromQueryString): UriBuilder
@@ -397,7 +224,6 @@ class UriBuilder
     }
 
     /**
-     * @return array
      * @internal
      */
     public function getArgumentsToBeExcludedFromQueryString(): array
@@ -408,7 +234,6 @@ class UriBuilder
     /**
      * Specifies the prefix to be used for all arguments.
      *
-     * @param string $argumentPrefix
      * @return static the current UriBuilder to allow method chaining
      */
     public function setArgumentPrefix(string $argumentPrefix): UriBuilder
@@ -418,7 +243,6 @@ class UriBuilder
     }
 
     /**
-     * @return string|null
      * @internal only to be used within Extbase, not part of TYPO3 Core API.
      */
     public function getArgumentPrefix(): ?string
@@ -429,7 +253,6 @@ class UriBuilder
     /**
      * If set, URIs for pages without access permissions will be created
      *
-     * @param bool $linkAccessRestrictedPages
      * @return static the current UriBuilder to allow method chaining
      */
     public function setLinkAccessRestrictedPages(bool $linkAccessRestrictedPages): UriBuilder
@@ -439,7 +262,6 @@ class UriBuilder
     }
 
     /**
-     * @return bool
      * @internal
      */
     public function getLinkAccessRestrictedPages(): bool
@@ -450,7 +272,6 @@ class UriBuilder
     /**
      * Uid of the target page
      *
-     * @param int $targetPageUid
      * @return static the current UriBuilder to allow method chaining
      */
     public function setTargetPageUid(int $targetPageUid): UriBuilder
@@ -460,10 +281,7 @@ class UriBuilder
     }
 
     /**
-     * returns $this->targetPageUid.
-     *
-     * @return int|null
-     * @internal
+     * @internal only to be used within Extbase, not part of TYPO3 Core API.
      */
     public function getTargetPageUid(): ?int
     {
@@ -473,7 +291,6 @@ class UriBuilder
     /**
      * Sets the page type of the target URI. Defaults to 0
      *
-     * @param int $targetPageType
      * @return static the current UriBuilder to allow method chaining
      */
     public function setTargetPageType(int $targetPageType): UriBuilder
@@ -483,7 +300,6 @@ class UriBuilder
     }
 
     /**
-     * @return int
      * @internal only to be used within Extbase, not part of TYPO3 Core API.
      */
     public function getTargetPageType(): int
@@ -494,7 +310,6 @@ class UriBuilder
     /**
      * by default FALSE; if TRUE, &no_cache=1 will be appended to the URI
      *
-     * @param bool $noCache
      * @return static the current UriBuilder to allow method chaining
      */
     public function setNoCache(bool $noCache): UriBuilder
@@ -504,7 +319,6 @@ class UriBuilder
     }
 
     /**
-     * @return bool
      * @internal
      */
     public function getNoCache(): bool
@@ -512,28 +326,6 @@ class UriBuilder
         return $this->noCache;
     }
 
-    /**
-     * by default TRUE; if FALSE, no cHash parameter will be appended to the URI
-     * If noCache is set, this setting will be ignored.
-     *
-     * @return static the current UriBuilder to allow method chaining
-     */
-    public function setUseCacheHash(): UriBuilder
-    {
-        trigger_error('Calling UriBuilder->setUseCacheHash() will be removed in TYPO3 v11.0. TYPO3 Core routing is taking care of handling this argument. Simply remove the line to avoid the notice.', E_USER_DEPRECATED);
-        return $this;
-    }
-
-    /**
-     * @return bool
-     * @internal
-     */
-    public function getUseCacheHash(): bool
-    {
-        trigger_error('Calling UriBuilder->getUseCacheHash() will be removed in TYPO3 v11.0. TYPO3 Core routing is taking care of handling this argument. Simply remove the line to avoid the notice.', E_USER_DEPRECATED);
-        return true;
-    }
-
     /**
      * Returns the arguments being used for the last URI being built.
      * This is only set after build() / uriFor() has been called.
@@ -559,7 +351,6 @@ class UriBuilder
         $this->language = null;
         $this->createAbsoluteUri = false;
         $this->addQueryString = false;
-        $this->addQueryStringMethod = '';
         $this->argumentsToBeExcludedFromQueryString = [];
         $this->linkAccessRestrictedPages = false;
         $this->targetPageUid = null;
@@ -567,16 +358,13 @@ class UriBuilder
         $this->noCache = false;
         $this->argumentPrefix = null;
         $this->absoluteUriScheme = null;
-        /*
-         * $this->request MUST NOT be reset here because the request is actually a hard dependency and not part
-         * of the internal state of this object.
-         * todo: consider making the request a constructor dependency or get rid of it's usage
-         */
+        // $this->request MUST NOT be reset here because the request is actually a hard dependency
+        // and not part of the internal state of this object.
         return $this;
     }
 
     /**
-     * Creates an URI used for linking to an Extbase action.
+     * Creates a URI used for linking to an Extbase action.
      * Works in Frontend and Backend mode of TYPO3.
      *
      * @param string|null $actionName Name of the action to be called
@@ -607,16 +395,14 @@ class UriBuilder
         if ($extensionName === null) {
             $extensionName = $this->request->getControllerExtensionName();
         }
-        if ($pluginName === null && $this->environmentService->isEnvironmentInFrontendMode()) {
+        $isFrontend = ApplicationType::fromRequest($this->request)->isFrontend();
+        if ($pluginName === null && $isFrontend) {
             $pluginName = $this->extensionService->getPluginNameByAction($extensionName, $controllerArguments['controller'], $controllerArguments['action'] ?? null);
         }
         if ($pluginName === null) {
             $pluginName = $this->request->getPluginName();
         }
-        if ($this->environmentService->isEnvironmentInFrontendMode() && $this->configurationManager->isFeatureEnabled('skipDefaultArguments')) {
-            $controllerArguments = $this->removeDefaultControllerAndAction($controllerArguments, $extensionName, $pluginName);
-        }
-        if ($this->targetPageUid === null && $this->environmentService->isEnvironmentInFrontendMode()) {
+        if ($this->targetPageUid === null && $isFrontend) {
             $this->targetPageUid = $this->extensionService->getTargetPidByPlugin($extensionName, $pluginName);
         }
         if ($this->format !== '') {
@@ -624,6 +410,12 @@ class UriBuilder
         }
         if ($this->argumentPrefix !== null) {
             $prefixedControllerArguments = [$this->argumentPrefix => $controllerArguments];
+        } elseif (!$isFrontend) {
+            $prefixedControllerArguments = $controllerArguments;
+            // Backend UriBuilder needs the route, which usually maps to the "route" parameter, which can be
+            // found in "Configuration/Backend/Modules.php" as the main key - that is the actual base route
+            // for the backend module, which in Extbase-speak is called a "pluginName"
+            $prefixedControllerArguments['route'] = $pluginName;
         } else {
             $pluginNamespace = $this->extensionService->getPluginNamespace($extensionName, $pluginName);
             $prefixedControllerArguments = [$pluginNamespace => $controllerArguments];
@@ -632,32 +424,6 @@ class UriBuilder
         return $this->build();
     }
 
-    /**
-     * This removes controller and/or action arguments from given controllerArguments
-     * if they are equal to the default controller/action of the target plugin.
-     * Note: This is only active in FE mode and if feature "skipDefaultArguments" is enabled
-     *
-     * @see \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::isFeatureEnabled()
-     * @param array $controllerArguments the current controller arguments to be modified
-     * @param string $extensionName target extension name
-     * @param string $pluginName target plugin name
-     * @return array
-     */
-    protected function removeDefaultControllerAndAction(array $controllerArguments, string $extensionName, string $pluginName): array
-    {
-        $defaultControllerName = $this->extensionService->getDefaultControllerNameByPlugin($extensionName, $pluginName);
-        if (isset($controllerArguments['action'])) {
-            $defaultActionName = $this->extensionService->getDefaultActionNameByPluginAndController($extensionName, $pluginName, $controllerArguments['controller']);
-            if ($controllerArguments['action'] === $defaultActionName) {
-                unset($controllerArguments['action']);
-            }
-        }
-        if ($controllerArguments['controller'] === $defaultControllerName) {
-            unset($controllerArguments['controller']);
-        }
-        return $controllerArguments;
-    }
-
     /**
      * Builds the URI
      * Depending on the current context this calls buildBackendUri() or buildFrontendUri()
@@ -668,7 +434,7 @@ class UriBuilder
      */
     public function build(): string
     {
-        if ($this->environmentService->isEnvironmentInBackendMode()) {
+        if (ApplicationType::fromRequest($this->request)->isBackend()) {
             return $this->buildBackendUri();
         }
         return $this->buildFrontendUri();
@@ -676,60 +442,60 @@ class UriBuilder
 
     /**
      * Builds the URI, backend flavour
-     * The resulting URI is relative and starts with "index.php".
      * The settings pageUid, pageType, noCache & linkAccessRestrictedPages
      * will be ignored in the backend.
      *
      * @return string The URI
      * @internal only to be used within Extbase, not part of TYPO3 Core API.
-     * @see \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer::getQueryArguments
      */
     public function buildBackendUri(): string
     {
         $arguments = [];
-        if ($this->addQueryString === true) {
-            if ($this->addQueryStringMethod === '' || $this->addQueryStringMethod === '0' || $this->addQueryStringMethod === 'GET') {
-                $arguments = GeneralUtility::_GET();
-            } else {
-                // Explode GET vars recursively
-                parse_str(GeneralUtility::getIndpEnv('QUERY_STRING'), $arguments);
-            }
+        if ($this->addQueryString && $this->addQueryString !== 'false') {
+            $arguments = $this->request->getQueryParams();
             foreach ($this->argumentsToBeExcludedFromQueryString as $argumentToBeExcluded) {
                 $argumentArrayToBeExcluded = [];
                 parse_str($argumentToBeExcluded, $argumentArrayToBeExcluded);
                 $arguments = ArrayUtility::arrayDiffKeyRecursive($arguments, $argumentArrayToBeExcluded);
             }
         } else {
-            $id = GeneralUtility::_GP('id');
-            $route = GeneralUtility::_GP('route');
+            $id = $this->request->getParsedBody()['id'] ?? $this->request->getQueryParams()['id'] ?? null;
             if ($id !== null) {
                 $arguments['id'] = $id;
             }
-            if ($route !== null) {
-                $arguments['route'] = $route;
-            }
         }
-        ArrayUtility::mergeRecursiveWithOverrule($arguments, $this->arguments);
+        if (($route = $this->request->getAttribute('route')) instanceof Route) {
+            /** @var Route $route */
+            $arguments['route'] = $route->getOption('_identifier');
+        }
+        $arguments = array_replace_recursive($arguments, $this->arguments);
         $arguments = $this->convertDomainObjectsToIdentityArrays($arguments);
         $this->lastArguments = $arguments;
-        $routeName = $arguments['route'] ?? null;
+        $routeIdentifier = $arguments['route'] ?? null;
         unset($arguments['route'], $arguments['token']);
-        $backendUriBuilder = GeneralUtility::makeInstance(\TYPO3\CMS\Backend\Routing\UriBuilder::class);
-        try {
-            if ($this->createAbsoluteUri) {
-                $uri = (string)$backendUriBuilder->buildUriFromRoutePath($routeName, $arguments, \TYPO3\CMS\Backend\Routing\UriBuilder::ABSOLUTE_URL);
-            } else {
-                $uri = (string)$backendUriBuilder->buildUriFromRoutePath($routeName, $arguments, \TYPO3\CMS\Backend\Routing\UriBuilder::ABSOLUTE_PATH);
-            }
-        } catch (ResourceNotFoundException $e) {
+
+        // In case the current route identifier is an identifier of a sub route, remove the sub route
+        // part to be able to add the actually requested sub route based on the current arguments.
+        if ($routeIdentifier && str_contains($routeIdentifier, '.')) {
+            [$routeIdentifier] = explode('.', $routeIdentifier);
+        }
+        // Build route identifier to the actually requested sub route (controller / action pair) - if any -
+        // and unset corresponding arguments.
+        if ($routeIdentifier && isset($arguments['controller'], $arguments['action'])) {
+            $routeIdentifier .= '.' . $arguments['controller'] . '_' . $arguments['action'];
+            unset($arguments['controller'], $arguments['action']);
+        }
+        $uri = '';
+        if ($routeIdentifier) {
+            $backendUriBuilder = GeneralUtility::makeInstance(\TYPO3\CMS\Backend\Routing\UriBuilder::class);
             try {
                 if ($this->createAbsoluteUri) {
-                    $uri = (string)$backendUriBuilder->buildUriFromRoute($routeName, $arguments, \TYPO3\CMS\Backend\Routing\UriBuilder::ABSOLUTE_URL);
+                    $uri = (string)$backendUriBuilder->buildUriFromRoute($routeIdentifier, $arguments, \TYPO3\CMS\Backend\Routing\UriBuilder::ABSOLUTE_URL);
                 } else {
-                    $uri = (string)$backendUriBuilder->buildUriFromRoute($routeName, $arguments, \TYPO3\CMS\Backend\Routing\UriBuilder::ABSOLUTE_PATH);
+                    $uri = (string)$backendUriBuilder->buildUriFromRoute($routeIdentifier, $arguments);
                 }
-            } catch (RouteNotFoundException $e) {
-                $uri = '';
+            } catch (RouteNotFoundException) {
+                // empty URL
             }
         }
         if ($this->section !== '') {
@@ -754,22 +520,21 @@ class UriBuilder
                 $typolinkConfiguration['forceAbsoluteUrl.']['scheme'] = $this->absoluteUriScheme;
             }
         }
-        // Other than stated in the doc block, typoLink_URL does not always return a string
-        // Thus, we explicitly cast to string here.
-        $uri = (string)$this->contentObject->typoLink_URL($typolinkConfiguration);
-        return $uri;
+        /** @var ?ContentObjectRenderer $currentContentObject */
+        $currentContentObject = $this->request->getAttribute('currentContentObject');
+        return $currentContentObject?->createUrl($typolinkConfiguration) ?? '';
     }
 
     /**
      * Builds a TypoLink configuration array from the current settings
      *
      * @return array typolink configuration array
-     * @see https://docs.typo3.org/m/typo3/reference-typoscript/master/en-us/Functions/Typolink.html
+     * @see https://docs.typo3.org/m/typo3/reference-typoscript/main/en-us/Functions/Typolink.html
      */
     protected function buildTypolinkConfiguration(): array
     {
         $typolinkConfiguration = [];
-        $typolinkConfiguration['parameter'] = $this->targetPageUid ?? $GLOBALS['TSFE']->id;
+        $typolinkConfiguration['parameter'] = $this->targetPageUid ?? $this->request->getAttribute('frontend.page.information')?->getId() ?? '';
         if ($this->targetPageType !== 0) {
             $typolinkConfiguration['parameter'] .= ',' . $this->targetPageType;
         } elseif ($this->format !== '') {
@@ -781,21 +546,17 @@ class UriBuilder
             $this->lastArguments = $arguments;
             $typolinkConfiguration['additionalParams'] = HttpUtility::buildQueryString($arguments, '&');
         }
-        if ($this->addQueryString === true) {
-            $typolinkConfiguration['addQueryString'] = 1;
+        if ($this->addQueryString && $this->addQueryString !== 'false') {
+            $typolinkConfiguration['addQueryString'] = $this->addQueryString;
             if (!empty($this->argumentsToBeExcludedFromQueryString)) {
                 $typolinkConfiguration['addQueryString.'] = [
-                    'exclude' => implode(',', $this->argumentsToBeExcludedFromQueryString)
+                    'exclude' => implode(',', $this->argumentsToBeExcludedFromQueryString),
                 ];
             }
-            if ($this->addQueryStringMethod !== '') {
-                $typolinkConfiguration['addQueryString.']['method'] = $this->addQueryStringMethod;
-            }
         }
         if ($this->language !== null) {
             $typolinkConfiguration['language'] = $this->language;
         }
-
         if ($this->noCache === true) {
             $typolinkConfiguration['no_cache'] = 1;
         }
@@ -813,7 +574,7 @@ class UriBuilder
      * into an arrays containing the uid of the domain object.
      *
      * @param array $arguments The arguments to be iterated
-     * @throws \TYPO3\CMS\Extbase\Mvc\Exception\InvalidArgumentValueException
+     * @throws InvalidArgumentValueException
      * @return array The modified arguments array
      */
     protected function convertDomainObjectsToIdentityArrays(array $arguments): array
@@ -829,7 +590,7 @@ class UriBuilder
             if ($argumentValue instanceof \Iterator) {
                 $argumentValue = $this->convertIteratorToArray($argumentValue);
             }
-            if ($argumentValue instanceof AbstractDomainObject) {
+            if ($argumentValue instanceof DomainObjectInterface) {
                 if ($argumentValue->getUid() !== null) {
                     $arguments[$argumentKey] = $argumentValue->getUid();
                 } elseif ($argumentValue instanceof AbstractValueObject) {
@@ -839,15 +600,15 @@ class UriBuilder
                 }
             } elseif (is_array($argumentValue)) {
                 $arguments[$argumentKey] = $this->convertDomainObjectsToIdentityArrays($argumentValue);
+            } elseif ($argumentValue instanceof \UnitEnum) {
+                $arguments[$argumentKey] = $argumentValue->value ?? $argumentValue->name;
+            } elseif ($argumentValue instanceof \Stringable) {
+                $arguments[$argumentKey] = (string)$argumentValue;
             }
         }
         return $arguments;
     }
 
-    /**
-     * @param \Iterator $iterator
-     * @return array
-     */
     protected function convertIteratorToArray(\Iterator $iterator): array
     {
         if (method_exists($iterator, 'toArray')) {
@@ -861,19 +622,16 @@ class UriBuilder
     /**
      * Converts a given object recursively into an array.
      *
-     * @param \TYPO3\CMS\Extbase\DomainObject\AbstractDomainObject $object
-     * @return array
      * @todo Refactor this into convertDomainObjectsToIdentityArrays()
-     * @internal only to be used within Extbase, not part of TYPO3 Core API.
      */
-    public function convertTransientObjectToArray(AbstractDomainObject $object): array
+    protected function convertTransientObjectToArray(DomainObjectInterface $object): array
     {
         $result = [];
         foreach ($object->_getProperties() as $propertyName => $propertyValue) {
             if ($propertyValue instanceof \Iterator) {
                 $propertyValue = $this->convertIteratorToArray($propertyValue);
             }
-            if ($propertyValue instanceof AbstractDomainObject) {
+            if ($propertyValue instanceof DomainObjectInterface) {
                 if ($propertyValue->getUid() !== null) {
                     $result[$propertyName] = $propertyValue->getUid();
                 } else {
