@@ -1,7 +1,6 @@
 <?php
-declare(strict_types = 1);
 
-namespace TYPO3\CMS\Extbase\Persistence\Generic\Exception;
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -16,11 +15,11 @@ namespace TYPO3\CMS\Extbase\Persistence\Generic\Exception;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Extbase\Persistence\Generic\Exception;
+
 use TYPO3\CMS\Extbase\Persistence\Generic\Exception;
 
 /**
  * An "Unsupported Order" exception: The order you specified in the query is not supported by now.
  */
-class UnsupportedOrderException extends Exception
-{
-}
+class UnsupportedOrderException extends Exception {}
