@@ -20,7 +20,6 @@ namespace TYPO3\CMS\Extbase\Reflection\ClassSchema;
 use TYPO3\CMS\Extbase\Reflection\ClassSchema\Exception\NoSuchMethodParameterException;
 
 /**
- * Class TYPO3\CMS\Extbase\Reflection\ClassSchema\Property
  * @internal only to be used within Extbase, not part of TYPO3 Core API.
  */
 class Method
@@ -45,11 +44,6 @@ class Method
      */
     private $parameters = [];
 
-    /**
-     * @param string $name
-     * @param array $definition
-     * @param string $className
-     */
     public function __construct(string $name, array $definition, string $className)
     {
         $this->name = $name;
@@ -60,8 +54,6 @@ class Method
             'public' => false,
             'protected' => false,
             'private' => false,
-            'injectMethod' => false,
-            'static' => false,
         ];
 
         foreach ($defaults as $key => $defaultValue) {
@@ -77,9 +69,6 @@ class Method
         }
     }
 
-    /**
-     * @return string
-     */
     public function getName(): string
     {
         return $this->name;
@@ -95,35 +84,6 @@ class Method
 
     /**
      * @throws NoSuchMethodParameterException
-     *
-     * @return MethodParameter
-     */
-    public function getFirstParameter(): MethodParameter
-    {
-        $position = 0;
-
-        $parameters = array_filter($this->getParameters(), function (MethodParameter $parameter) use ($position) {
-            return $parameter->getPosition() === $position;
-        });
-
-        $parameter = reset($parameters);
-
-        if (!$parameter instanceof MethodParameter) {
-            throw NoSuchMethodParameterException::createForParameterPosition(
-                $this->className,
-                $this->name,
-                $position
-            );
-        }
-
-        return $parameter;
-    }
-
-    /**
-     * @throws NoSuchMethodParameterException
-     *
-     * @param string $parameterName
-     * @return MethodParameter
      */
     public function getParameter(string $parameterName): MethodParameter
     {
@@ -138,43 +98,18 @@ class Method
         return $this->parameters[$parameterName];
     }
 
-    /**
-     * @return bool
-     */
     public function isPublic(): bool
     {
         return $this->definition['public'];
     }
 
-    /**
-     * @return bool
-     */
     public function isProtected(): bool
     {
         return $this->definition['protected'];
     }
 
-    /**
-     * @return bool
-     */
     public function isPrivate(): bool
     {
         return $this->definition['private'];
     }
-
-    /**
-     * @return bool
-     */
-    public function isInjectMethod(): bool
-    {
-        return $this->definition['injectMethod'];
-    }
-
-    /**
-     * @return bool
-     */
-    public function isStatic(): bool
-    {
-        return $this->definition['static'];
-    }
 }
