@@ -18,7 +18,6 @@ declare(strict_types=1);
 namespace TYPO3\CMS\Extbase\Reflection\ClassSchema;
 
 /**
- * Class TYPO3\CMS\Extbase\Reflection\ClassSchema\Property
  * @internal only to be used within Extbase, not part of TYPO3 Core API.
  */
 class MethodParameter
@@ -33,10 +32,6 @@ class MethodParameter
      */
     private $definition;
 
-    /**
-     * @param string $name
-     * @param array $definition
-     */
     public function __construct(string $name, array $definition)
     {
         $this->name = $name;
@@ -47,10 +42,8 @@ class MethodParameter
             'optional' => false,
             'hasDefaultValue' => false,
             'defaultValue' => null,
-            'dependency' => null,
             'ignoreValidation' => false,
             'validators' => [],
-            'position' => -1,
         ];
 
         foreach ($defaults as $key => $defaultValue) {
@@ -62,33 +55,21 @@ class MethodParameter
         $this->definition = $definition;
     }
 
-    /**
-     * @return string
-     */
     public function getName(): string
     {
         return $this->name;
     }
 
-    /**
-     * @return string|null
-     */
     public function getType(): ?string
     {
         return $this->definition['type'];
     }
 
-    /**
-     * @return bool
-     */
     public function isArray(): bool
     {
         return $this->definition['array'];
     }
 
-    /**
-     * @return bool
-     */
     public function hasDefaultValue(): bool
     {
         return $this->definition['hasDefaultValue'];
@@ -102,43 +83,18 @@ class MethodParameter
         return $this->definition['defaultValue'];
     }
 
-    /**
-     * @return array
-     */
     public function getValidators(): array
     {
         return $this->definition['validators'];
     }
 
-    /**
-     * @return bool
-     */
     public function ignoreValidation(): bool
     {
         return $this->definition['ignoreValidation'];
     }
 
-    /**
-     * @return bool
-     */
     public function isOptional(): bool
     {
         return $this->definition['optional'];
     }
-
-    /**
-     * @return string|null
-     */
-    public function getDependency(): ?string
-    {
-        return $this->definition['dependency'];
-    }
-
-    /**
-     * @return int
-     */
-    public function getPosition(): int
-    {
-        return $this->definition['position'];
-    }
 }
