@@ -17,11 +17,12 @@ declare(strict_types=1);
 
 namespace TYPO3\CMS\Extbase\Reflection;
 
+use Symfony\Component\PropertyAccess\Exception\NoSuchIndexException;
 use Symfony\Component\PropertyAccess\PropertyAccess;
 use Symfony\Component\PropertyAccess\PropertyAccessor;
+use Symfony\Component\PropertyAccess\PropertyAccessorInterface;
 use Symfony\Component\PropertyAccess\PropertyPath;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
-use TYPO3\CMS\Core\Utility\StringUtility;
 use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
 use TYPO3\CMS\Extbase\Reflection\Exception\PropertyNotAccessibleException;
 
@@ -36,40 +37,32 @@ use TYPO3\CMS\Extbase\Reflection\Exception\PropertyNotAccessibleException;
  */
 class ObjectAccess
 {
-    /**
-     * @var PropertyAccessor
-     */
-    private static $propertyAccessor;
+    private static ?PropertyAccessorInterface $propertyAccessor = null;
 
     /**
      * Get a property of a given object.
      * Tries to get the property the following ways:
      * - if the target is an array, and has this property, we call it.
-     * - if super cow powers should be used, fetch value through reflection
      * - if public getter method exists, call it.
      * - if the target object is an instance of ArrayAccess, it gets the property
      * on it if it exists.
      * - if public property exists, return the value of it.
      * - else, throw exception
      *
-     * @param mixed $subject Object or array to get the property from
+     * @param object|array $subject Object or array to get the property from
      * @param string $propertyName name of the property to retrieve
-     * @param bool $forceDirectAccess directly access property using reflection(!)
      *
      * @throws \InvalidArgumentException in case $subject was not an object or $propertyName was not a string
      * @throws Exception\PropertyNotAccessibleException
      * @return mixed Value of the property
      */
-    public static function getProperty($subject, string $propertyName, bool $forceDirectAccess = false)
+    public static function getProperty(object|array $subject, string $propertyName): mixed
     {
-        if (!is_object($subject) && !is_array($subject)) {
-            throw new \InvalidArgumentException(
-                '$subject must be an object or array, ' . gettype($subject) . ' given.',
-                1237301367
-            );
+        try {
+            return self::getPropertyInternal($subject, $propertyName);
+        } catch (NoSuchIndexException) {
+            return null;
         }
-
-        return self::getPropertyInternal($subject, $propertyName, $forceDirectAccess);
     }
 
     /**
@@ -79,21 +72,16 @@ class ObjectAccess
      *
      * @see getProperty()
      *
-     * @param mixed $subject Object or array to get the property from
+     * @param object|array $subject Object or array to get the property from
      * @param string $propertyName name of the property to retrieve
-     * @param bool $forceDirectAccess directly access property using reflection(!)
      *
      * @throws Exception\PropertyNotAccessibleException
      * @return mixed Value of the property
      * @internal
      */
-    public static function getPropertyInternal($subject, string $propertyName, bool $forceDirectAccess = false)
+    public static function getPropertyInternal(object|array $subject, string $propertyName): mixed
     {
-        if ($forceDirectAccess === true) {
-            trigger_error('Argument $forceDirectAccess will be removed in TYPO3 11.0', E_USER_DEPRECATED);
-        }
-
-        if (!$forceDirectAccess && ($subject instanceof \SplObjectStorage || $subject instanceof ObjectStorage)) {
+        if ($subject instanceof \SplObjectStorage || $subject instanceof ObjectStorage) {
             $subject = iterator_to_array(clone $subject, false);
         }
 
@@ -114,14 +102,14 @@ class ObjectAccess
         }
 
         if (is_object($subject)) {
-            return self::getObjectPropertyValue($subject, $propertyPath, $forceDirectAccess);
+            return self::getObjectPropertyValue($subject, $propertyPath);
         }
 
-        if (is_array($subject)) {
+        try {
             return self::getArrayIndexValue($subject, self::convertToArrayPropertyPath($propertyPath));
+        } catch (NoSuchIndexException) {
+            return null;
         }
-
-        return null;
     }
 
     /**
@@ -132,18 +120,18 @@ class ObjectAccess
      *
      * For arrays the keys are checked likewise.
      *
-     * @param mixed $subject Object or array to get the property path from
+     * @param object|array $subject Object or array to get the property path from
      * @param string $propertyPath
      *
      * @return mixed Value of the property
      */
-    public static function getPropertyPath($subject, string $propertyPath)
+    public static function getPropertyPath(object|array $subject, string $propertyPath): mixed
     {
         try {
             foreach (new PropertyPath($propertyPath) as $pathSegment) {
                 $subject = self::getPropertyInternal($subject, $pathSegment);
             }
-        } catch (PropertyNotAccessibleException $error) {
+        } catch (\TypeError|PropertyNotAccessibleException) {
             return null;
         }
         return $subject;
@@ -160,46 +148,25 @@ class ObjectAccess
      * on it without checking if it existed.
      * - else, return FALSE
      *
-     * @param mixed $subject The target object or array
+     * @param object|array $subject The target object or array
      * @param string $propertyName Name of the property to set
      * @param mixed $propertyValue Value of the property
-     * @param bool $forceDirectAccess directly access property using reflection(!)
      *
      * @throws \InvalidArgumentException in case $object was not an object or $propertyName was not a string
      * @return bool TRUE if the property could be set, FALSE otherwise
      */
-    public static function setProperty(&$subject, string $propertyName, $propertyValue, bool $forceDirectAccess = false): bool
+    public static function setProperty(object|array &$subject, string $propertyName, mixed $propertyValue): bool
     {
-        if ($forceDirectAccess === true) {
-            trigger_error('Argument $forceDirectAccess will be removed in TYPO3 11.0', E_USER_DEPRECATED);
-        }
-
-        if (is_array($subject) || ($subject instanceof \ArrayAccess && !$forceDirectAccess)) {
+        if (is_array($subject) || $subject instanceof \ArrayAccess) {
             $subject[$propertyName] = $propertyValue;
             return true;
         }
-        if (!is_object($subject)) {
-            throw new \InvalidArgumentException('subject must be an object or array, ' . gettype($subject) . ' given.', 1237301368);
-        }
 
         $accessor = self::createAccessor();
         if ($accessor->isWritable($subject, $propertyName)) {
             $accessor->setValue($subject, $propertyName, $propertyValue);
             return true;
         }
-
-        if ($forceDirectAccess) {
-            if (property_exists($subject, $propertyName)) {
-                $propertyReflection = new \ReflectionProperty($subject, $propertyName);
-                $propertyReflection->setAccessible(true);
-                $propertyReflection->setValue($subject, $propertyValue);
-            } else {
-                $subject->{$propertyName} = $propertyValue;
-            }
-
-            return true;
-        }
-
         return false;
     }
 
@@ -212,7 +179,7 @@ class ObjectAccess
      *
      * @param object $object Object to receive property names for
      *
-     * @return array Array of all gettable property names
+     * @return list<string> Array of all gettable property names
      * @throws Exception\UnknownClassException
      */
     public static function getGettablePropertyNames(object $object): array
@@ -226,13 +193,61 @@ class ObjectAccess
         $classSchema = GeneralUtility::makeInstance(ReflectionService::class)
             ->getClassSchema($object);
 
-        $accessor = self::createAccessor();
-        $propertyNames = array_keys($classSchema->getProperties());
-        $accessiblePropertyNames = array_filter($propertyNames, function ($propertyName) use ($accessor, $object) {
-            return $accessor->isReadable($object, $propertyName);
-        });
+        $accessiblePropertyNames = [];
+        foreach ($classSchema->getProperties() as $propertyName => $propertyDefinition) {
+            if ($propertyDefinition->isPublic()) {
+                $accessiblePropertyNames[] = $propertyName;
+                continue;
+            }
 
+            $accessors = [
+                'get' . ucfirst($propertyName),
+                'has' . ucfirst($propertyName),
+                'is' . ucfirst($propertyName),
+            ];
+
+            foreach ($accessors as $accessor) {
+                if (!$classSchema->hasMethod($accessor)) {
+                    continue;
+                }
+
+                if (!$classSchema->getMethod($accessor)->isPublic()) {
+                    continue;
+                }
+
+                foreach ($classSchema->getMethod($accessor)->getParameters() as $methodParam) {
+                    if (!$methodParam->isOptional()) {
+                        continue 2;
+                    }
+                }
+
+                if (!is_callable([$object, $accessor])) {
+                    continue;
+                }
+
+                $accessiblePropertyNames[] = $propertyName;
+            }
+        }
+
+        // Fallback mechanism to not break former behaviour
+        //
+        // todo: Checking accessor methods of virtual(non-existing) properties should be removed (breaking) in
+        //       upcoming versions. It was an unintentionally added "feature" in the past. It contradicts the method
+        //       name "getGettablePropertyNames".
         foreach ($classSchema->getMethods() as $methodName => $methodDefinition) {
+            $propertyName = null;
+            if (str_starts_with($methodName, 'get') || str_starts_with($methodName, 'has')) {
+                $propertyName = lcfirst(substr($methodName, 3));
+            }
+
+            if (str_starts_with($methodName, 'is')) {
+                $propertyName = lcfirst(substr($methodName, 2));
+            }
+
+            if ($propertyName === null) {
+                continue;
+            }
+
             if (!$methodDefinition->isPublic()) {
                 continue;
             }
@@ -243,19 +258,7 @@ class ObjectAccess
                 }
             }
 
-            if (StringUtility::beginsWith($methodName, 'get')) {
-                $accessiblePropertyNames[] = lcfirst(substr($methodName, 3));
-                continue;
-            }
-
-            if (StringUtility::beginsWith($methodName, 'has')) {
-                $accessiblePropertyNames[] = lcfirst(substr($methodName, 3));
-                continue;
-            }
-
-            if (StringUtility::beginsWith($methodName, 'is')) {
-                $accessiblePropertyNames[] = lcfirst(substr($methodName, 2));
-            }
+            $accessiblePropertyNames[] = $propertyName;
         }
 
         $accessiblePropertyNames = array_unique($accessiblePropertyNames);
@@ -273,7 +276,7 @@ class ObjectAccess
      * @param object $object Object to receive property names for
      *
      * @throws \InvalidArgumentException
-     * @return array Array of all settable property names
+     * @return list<string> Array of all settable property names
      */
     public static function getSettablePropertyNames(object $object): array
     {
@@ -284,13 +287,15 @@ class ObjectAccess
         } else {
             $classSchema = GeneralUtility::makeInstance(ReflectionService::class)->getClassSchema($object);
 
-            $propertyNames = array_filter(array_keys($classSchema->getProperties()), function ($methodName) use ($accessor, $object) {
-                return $accessor->isWritable($object, $methodName);
-            });
+            $propertyNames = array_filter(
+                array_keys($classSchema->getProperties()),
+                static fn(string $propertyName): bool => $accessor->isWritable($object, $propertyName)
+            );
 
-            $setters = array_filter(array_keys($classSchema->getMethods()), function ($methodName) use ($object) {
-                return StringUtility::beginsWith($methodName, 'set') && is_callable([$object, $methodName]);
-            });
+            $setters = array_filter(
+                array_keys($classSchema->getMethods()),
+                static fn(string $methodName): bool => str_starts_with($methodName, 'set') && is_callable([$object, $methodName])
+            );
 
             foreach ($setters as $setter) {
                 $propertyNames[] = lcfirst(substr($setter, 3));
@@ -307,11 +312,8 @@ class ObjectAccess
      *
      * @param object $object Object containing the property
      * @param string $propertyName Name of the property to check
-     *
-     * @throws \InvalidArgumentException
-     * @return bool
      */
-    public static function isPropertySettable(object $object, $propertyName): bool
+    public static function isPropertySettable(object $object, string $propertyName): bool
     {
         if ($object instanceof \stdClass && array_key_exists($propertyName, get_object_vars($object))) {
             return true;
@@ -325,19 +327,14 @@ class ObjectAccess
     /**
      * Tells if the value of the specified property can be retrieved by this Object Accessor.
      *
-     * @param object $object Object containing the property
+     * @param object|array $object Object containing the property
      * @param string $propertyName Name of the property to check
      *
      * @throws \InvalidArgumentException
-     * @return bool
      */
-    public static function isPropertyGettable($object, $propertyName): bool
+    public static function isPropertyGettable(object|array $object, string $propertyName): bool
     {
-        if (($object instanceof \ArrayAccess) && !$object->offsetExists($propertyName)) {
-            return false;
-        }
-
-        if (is_array($object) || $object instanceof \ArrayAccess) {
+        if (is_array($object) || ($object instanceof \ArrayAccess && $object->offsetExists($propertyName))) {
             $propertyName = self::wrap($propertyName);
         }
 
@@ -351,7 +348,7 @@ class ObjectAccess
      * @param object $object Object to get all properties from.
      *
      * @throws \InvalidArgumentException
-     * @return array Associative array of all properties.
+     * @return array<string, mixed> Associative array of all properties.
      * @todo What to do with ArrayAccess
      */
     public static function getGettableProperties(object $object): array
@@ -363,44 +360,21 @@ class ObjectAccess
         return $properties;
     }
 
-    /**
-     * Build the setter method name for a given property by capitalizing the
-     * first letter of the property, and prepending it with "set".
-     *
-     * @param string $propertyName Name of the property
-     *
-     * @return string Name of the setter method name
-     * @deprecated
-     */
-    public static function buildSetterMethodName($propertyName): string
-    {
-        trigger_error(__METHOD__ . ' will be removed in TYPO3 11.0', E_USER_DEPRECATED);
-
-        return 'set' . ucfirst($propertyName);
-    }
-
-    /**
-     * @return PropertyAccessor
-     */
     private static function createAccessor(): PropertyAccessor
     {
-        if (static::$propertyAccessor === null) {
-            static::$propertyAccessor = PropertyAccess::createPropertyAccessorBuilder()
+        if (self::$propertyAccessor === null) {
+            self::$propertyAccessor = PropertyAccess::createPropertyAccessorBuilder()
+                ->enableExceptionOnInvalidIndex()
                 ->getPropertyAccessor();
         }
 
-        return static::$propertyAccessor;
+        return self::$propertyAccessor;
     }
 
     /**
-     * @param object $subject
-     * @param PropertyPath $propertyPath
-     * @param bool $forceDirectAccess
-     * @return mixed
      * @throws Exception\PropertyNotAccessibleException
-     * @throws \ReflectionException
      */
-    private static function getObjectPropertyValue(object $subject, PropertyPath $propertyPath, bool $forceDirectAccess)
+    private static function getObjectPropertyValue(object $subject, PropertyPath $propertyPath): mixed
     {
         $accessor = self::createAccessor();
 
@@ -408,48 +382,21 @@ class ObjectAccess
             return $accessor->getValue($subject, $propertyPath);
         }
 
-        $propertyName = (string)$propertyPath;
-
-        if (!$forceDirectAccess) {
-            throw new PropertyNotAccessibleException('The property "' . $propertyName . '" on the subject does not exist.', 1476109666);
-        }
-
-        if (!property_exists($subject, $propertyName)) {
-            throw new PropertyNotAccessibleException('The property "' . $propertyName . '" on the subject does not exist.', 1302855001);
-        }
-
-        $propertyReflection = new \ReflectionProperty($subject, $propertyName);
-        $propertyReflection->setAccessible(true);
-        return $propertyReflection->getValue($subject);
+        throw new PropertyNotAccessibleException('The property "' . (string)$propertyPath . '" on the subject does not exist.', 1476109666);
     }
 
-    /**
-     * @param array $subject
-     * @param PropertyPath $propertyPath
-     * @return mixed
-     */
-    private static function getArrayIndexValue(array $subject, PropertyPath $propertyPath)
+    private static function getArrayIndexValue(array $subject, PropertyPath $propertyPath): mixed
     {
         return self::createAccessor()->getValue($subject, $propertyPath);
     }
 
-    /**
-     * @param PropertyPath $propertyPath
-     * @return PropertyPath
-     */
     private static function convertToArrayPropertyPath(PropertyPath $propertyPath): PropertyPath
     {
-        $segments = array_map(function ($segment) {
-            return static::wrap($segment);
-        }, $propertyPath->getElements());
+        $segments = array_map(static fn(string $segment): string => static::wrap($segment), $propertyPath->getElements());
 
         return new PropertyPath(implode('.', $segments));
     }
 
-    /**
-     * @param string $segment
-     * @return string
-     */
     private static function wrap(string $segment): string
     {
         return '[' . $segment . ']';
