@@ -25,11 +25,8 @@ use TYPO3\CMS\Form\Domain\Configuration\FrameworkConfiguration\Extractors\Abstra
  */
 class MultiValuePropertiesExtractor extends AbstractExtractor
 {
-
     /**
-     * @param string $_
      * @param mixed $value
-     * @param array $matches
      */
     public function __invoke(string $_, $value, array $matches)
     {
@@ -38,6 +35,7 @@ class MultiValuePropertiesExtractor extends AbstractExtractor
         if (
             $value !== 'Inspector-PropertyGridEditor'
             && $value !== 'Inspector-MultiSelectEditor'
+            && $value !== 'Inspector-CountrySelectEditor'
             && $value !== 'Inspector-ValidationErrorMessageEditor'
         ) {
             return;
