@@ -15,9 +15,9 @@
 
 namespace TYPO3\CMS\Frontend\ContentObject;
 
+use Psr\Log\LogLevel;
 use TYPO3\CMS\Core\TimeTracker\TimeTracker;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
-use TYPO3\CMS\Frontend\Controller\TypoScriptFrontendController;
 
 /**
  * Contains COA_INT class object.
@@ -33,15 +33,17 @@ class ContentObjectArrayInternalContentObject extends AbstractContentObject
     public function render($conf = [])
     {
         if (!is_array($conf)) {
-            $this->getTimeTracker()->setTSlogMessage('No elements in this content object array (COA_INT).', 2);
+            $this->getTimeTracker()->setTSlogMessage('No elements in this content object array (COA_INT).', LogLevel::WARNING);
             return '';
         }
-        $substKey = 'INT_SCRIPT.' . $this->getTypoScriptFrontendController()->uniqueHash();
+
+        $frontendController = $this->getTypoScriptFrontendController();
+        $substKey = 'INT_SCRIPT.' . $frontendController->uniqueHash();
         $content = '<!--' . $substKey . '-->';
-        $this->getTypoScriptFrontendController()->config['INTincScript'][$substKey] = [
+        $frontendController->config['INTincScript'][$substKey] = [
             'conf' => $conf,
             'cObj' => serialize($this->cObj),
-            'type' => 'COA'
+            'type' => 'COA',
         ];
         return $content;
     }
@@ -53,12 +55,4 @@ class ContentObjectArrayInternalContentObject extends AbstractContentObject
     {
         return GeneralUtility::makeInstance(TimeTracker::class);
     }
-
-    /**
-     * @return TypoScriptFrontendController
-     */
-    protected function getTypoScriptFrontendController()
-    {
-        return $GLOBALS['TSFE'];
-    }
 }
