@@ -18,7 +18,6 @@ declare(strict_types=1);
 namespace TYPO3\CMS\Frontend\Typolink;
 
 use TYPO3\CMS\Core\LinkHandling\LinkService;
-use TYPO3\CMS\Frontend\Http\UrlProcessorInterface;
 
 /**
  * Builds a TypoLink to a file (relative to fileadmin/ or something)
@@ -26,29 +25,26 @@ use TYPO3\CMS\Frontend\Http\UrlProcessorInterface;
  */
 class LegacyLinkBuilder extends AbstractTypolinkBuilder
 {
-    /**
-     * @inheritdoc
-     */
-    public function build(array &$linkDetails, string $linkText, string $target, array $conf): array
+    public function build(array &$linkDetails, string $linkText, string $target, array $conf): LinkResultInterface
     {
         $tsfe = $this->getTypoScriptFrontendController();
-        if ($linkDetails['file']) {
+        if ($linkDetails['file'] ?? false) {
             $linkDetails['type'] = LinkService::TYPE_FILE;
             $linkLocation = $linkDetails['file'];
             // Setting title if blank value to link
             $linkText = $this->encodeFallbackLinkTextIfLinkTextIsEmpty($linkText, rawurldecode($linkLocation));
-            $linkLocation = (strpos($linkLocation, '/') !== 0 ? $tsfe->absRefPrefix : '') . $linkLocation;
-            $url = $this->processUrl(UrlProcessorInterface::CONTEXT_FILE, $linkLocation, $conf) ?? '';
+            $linkLocation = (!str_starts_with($linkLocation, '/') ? $tsfe->absRefPrefix : '') . $linkLocation;
+            $url = $linkLocation;
             $url = $this->forceAbsoluteUrl($url, $conf);
-            $target = $target ?: $this->resolveTargetAttribute($conf, 'fileTarget', false, $tsfe->fileTarget);
-        } elseif ($linkDetails['url']) {
+            $target = $target ?: $this->resolveTargetAttribute($conf, 'fileTarget');
+        } elseif ($linkDetails['url'] ?? false) {
             $linkDetails['type'] = LinkService::TYPE_URL;
-            $target = $target ?: $this->resolveTargetAttribute($conf, 'extTarget', true, $tsfe->extTarget);
+            $target = $target ?: $this->resolveTargetAttribute($conf, 'extTarget');
             $linkText = $this->encodeFallbackLinkTextIfLinkTextIsEmpty($linkText, $linkDetails['url']);
-            $url = $this->processUrl(UrlProcessorInterface::CONTEXT_EXTERNAL, $linkDetails['url'], $conf);
+            $url = $linkDetails['url'];
         } else {
             throw new UnableToLinkException('Unknown link detected, so ' . $linkText . ' was not linked.', 1490990031, null, $linkText);
         }
-        return [$url, $linkText, $target];
+        return (new LinkResult((string)$linkDetails['type'], (string)$url))->withTarget($target)->withLinkConfiguration($conf)->withLinkText($linkText);
     }
 }
