@@ -1,6 +1,6 @@
 <?php
-declare(strict_types = 1);
-namespace TYPO3\CMS\Install\Controller;
+
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -15,13 +15,14 @@ namespace TYPO3\CMS\Install\Controller;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Install\Controller;
+
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
 use TYPO3\CMS\Core\Http\HtmlResponse;
 use TYPO3\CMS\Core\Imaging\IconFactory;
-use TYPO3\CMS\Core\Imaging\IconRegistry;
-use TYPO3\CMS\Core\Type\Icon\IconState;
-use TYPO3\CMS\Core\Utility\GeneralUtility;
+use TYPO3\CMS\Core\Imaging\IconSize;
+use TYPO3\CMS\Core\Imaging\IconState;
 
 /**
  * Controller for icon handling
@@ -29,37 +30,11 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
  */
 class IconController extends AbstractController
 {
-    /**
-     * @var IconRegistry
-     */
-    protected $iconRegistry;
+    public function __construct(
+        protected readonly IconFactory $iconFactory
+    ) {}
 
     /**
-     * @var IconFactory
-     */
-    protected $iconFactory;
-
-    /**
-     * Set up dependencies
-     */
-    public function __construct()
-    {
-        $this->iconRegistry = GeneralUtility::makeInstance(IconRegistry::class);
-        $this->iconFactory = GeneralUtility::makeInstance(IconFactory::class);
-    }
-
-    /**
-     * @return ResponseInterface
-     * @internal
-     */
-    public function getCacheIdentifierAction(): ResponseInterface
-    {
-        return new HtmlResponse($this->iconRegistry->getCacheIdentifier());
-    }
-
-    /**
-     * @param ServerRequestInterface $request
-     * @return ResponseInterface
      * @internal
      */
     public function getIconAction(ServerRequestInterface $request): ResponseInterface
@@ -74,7 +49,8 @@ class IconController extends AbstractController
             $overlayIdentifier = null;
         }
 
-        $iconState = IconState::cast($iconState);
+        $iconState = IconState::tryFrom($iconState);
+        $size = IconSize::tryFrom($size);
         $icon = $this->iconFactory->getIcon($identifier, $size, $overlayIdentifier, $iconState);
 
         return new HtmlResponse($icon->render($alternativeMarkupIdentifier));
