@@ -1,7 +1,6 @@
 <?php
-declare(strict_types = 1);
 
-namespace TYPO3\CMS\Install\SystemEnvironment;
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -16,31 +15,26 @@ namespace TYPO3\CMS\Install\SystemEnvironment;
  * The TYPO3 project - inspiring people to share!
  */
 
-use Doctrine\DBAL\Driver;
-use Doctrine\DBAL\Driver\DrizzlePDOMySql\Driver as DoctrineDrizzlePDOMySQLDriver;
-use Doctrine\DBAL\Driver\IBMDB2\DB2Driver;
+namespace TYPO3\CMS\Install\SystemEnvironment;
+
+use Doctrine\DBAL\Driver\IBMDB2\Driver as DB2Driver;
 use Doctrine\DBAL\Driver\Mysqli\Driver as DoctrineMysqliDriver;
 use Doctrine\DBAL\Driver\OCI8\Driver as DoctrineOCI8Driver;
-use Doctrine\DBAL\Driver\PDOOracle\Driver as DoctrinePDOOCIDriver;
-use Doctrine\DBAL\Driver\SQLAnywhere\Driver as DoctrineSQLAnywhereDriver;
-use Doctrine\DBAL\Driver\SQLSrv\Driver as DoctrineSQLSrvDriver;
-use TYPO3\CMS\Core\Database\Driver\PDOMySql\Driver as TYPO3PDOMySqlDriver;
-use TYPO3\CMS\Core\Database\Driver\PDOPgSql\Driver as TYPO3PDOPgSqlDriver;
-use TYPO3\CMS\Core\Database\Driver\PDOSqlite\Driver as TYPO3PDOSqliteDriver;
-use TYPO3\CMS\Core\Database\Driver\PDOSqlsrv\Driver as TYPO3PDOSqlSrvDriver;
+use Doctrine\DBAL\Driver\PDO\MySQL\Driver as DoctrinePDOMySqlDriver;
+use Doctrine\DBAL\Driver\PDO\OCI\Driver as DoctrinePDOOCIDriver;
+use Doctrine\DBAL\Driver\PDO\PgSQL\Driver as DoctrinePDOPgSqlDriver;
+use Doctrine\DBAL\Driver\PDO\SQLite\Driver as DoctrinePDOSqliteDriver;
 use TYPO3\CMS\Core\Messaging\FlashMessage;
 use TYPO3\CMS\Core\Messaging\FlashMessageQueue;
+use TYPO3\CMS\Core\Type\ContextualFeedbackSeverity;
 use TYPO3\CMS\Install\Exception;
 use TYPO3\CMS\Install\SystemEnvironment\DatabaseCheck\Driver\Mysqli as DatabaseCheckDriverMysqli;
 use TYPO3\CMS\Install\SystemEnvironment\DatabaseCheck\Driver\PdoMysql as DatabaseCheckDriverPdoMysql;
 use TYPO3\CMS\Install\SystemEnvironment\DatabaseCheck\Driver\PDOPgSql as DatabaseCheckDriverPDOPgSql;
 use TYPO3\CMS\Install\SystemEnvironment\DatabaseCheck\Driver\PDOSqlite as DatabaseCheckDriverPDOSqlite;
-use TYPO3\CMS\Install\SystemEnvironment\DatabaseCheck\Driver\PDOSqlsrv as DatabaseCheckDriverPDOSqlsrv;
-use TYPO3\CMS\Install\SystemEnvironment\DatabaseCheck\Driver\SQLSrv as DatabaseCheckDriverSQLSrv;
 use TYPO3\CMS\Install\SystemEnvironment\DatabaseCheck\Platform\MySql as DatabaseCheckPlatformMysql;
 use TYPO3\CMS\Install\SystemEnvironment\DatabaseCheck\Platform\PostgreSql as DatabaseCheckPlatformPostgreSql;
 use TYPO3\CMS\Install\SystemEnvironment\DatabaseCheck\Platform\Sqlite as DatabaseCheckPlatformSqlite;
-use TYPO3\CMS\Install\SystemEnvironment\DatabaseCheck\Platform\SqlSrv as DatabaseCheckPlatformSqlSrv;
 
 /**
  * Check database configuration status
@@ -84,31 +78,27 @@ class DatabaseCheck implements CheckInterface
      * List of database platforms to check
      *
      * @var string[]
+     * @todo Check if this property can be removed after DatabaseCheck::retrieveDatabasePlatformByDriverName() could be
+     *       removed.
      */
     private static $databaseDriverToPlatformMapping = [
         DoctrineMysqliDriver::class => DatabaseCheckPlatformMysql::class,
-        TYPO3PDOMySqlDriver::class => DatabaseCheckPlatformMysql::class,
-        TYPO3PDOPgSqlDriver::class => DatabaseCheckPlatformPostgreSql::class,
-        DoctrineSQLSrvDriver::class => DatabaseCheckPlatformSqlSrv::class,
-        TYPO3PDOSqlSrvDriver::class => DatabaseCheckPlatformSqlSrv::class,
-        TYPO3PDOSqliteDriver::class => DatabaseCheckPlatformSqlite::class,
+        DoctrinePDOMySqlDriver::class => DatabaseCheckPlatformMysql::class,
+        DoctrinePDOPgSqlDriver::class => DatabaseCheckPlatformPostgreSql::class,
+        DoctrinePDOSqliteDriver::class => DatabaseCheckPlatformSqlite::class,
     ];
 
     /**
      * @var string[]
      */
     private static $driverMap = [
-        'pdo_mysql' => TYPO3PDOMySqlDriver::class,
-        'pdo_sqlite' => TYPO3PDOSqliteDriver::class,
-        'pdo_pgsql' => TYPO3PDOPgSqlDriver::class,
+        'pdo_mysql' => DoctrinePDOMySqlDriver::class,
+        'pdo_sqlite' => DoctrinePDOSqliteDriver::class,
+        'pdo_pgsql' => DoctrinePDOPgSqlDriver::class,
         'pdo_oci' => DoctrinePDOOCIDriver::class,
         'oci8' => DoctrineOCI8Driver::class,
         'ibm_db2' => DB2Driver::class,
-        'pdo_sqlsrv' => TYPO3PDOSqlSrvDriver::class,
         'mysqli' => DoctrineMysqliDriver::class,
-        'drizzle_pdo_mysql' => DoctrineDrizzlePDOMySQLDriver::class,
-        'sqlanywhere' => DoctrineSQLAnywhereDriver::class,
-        'sqlsrv' => DoctrineSQLSrvDriver::class,
     ];
 
     /**
@@ -118,11 +108,9 @@ class DatabaseCheck implements CheckInterface
      */
     private $databaseDriverCheckMap = [
         DoctrineMysqliDriver::class => DatabaseCheckDriverMysqli::class,
-        TYPO3PDOMySqlDriver::class => DatabaseCheckDriverPdoMysql::class,
-        TYPO3PDOPgSqlDriver::class => DatabaseCheckDriverPDOPgSql::class,
-        DoctrineSQLSrvDriver::class => DatabaseCheckDriverSQLSrv::class,
-        TYPO3PDOSqlSrvDriver::class => DatabaseCheckDriverPDOSqlsrv::class,
-        TYPO3PDOSqliteDriver::class => DatabaseCheckDriverPDOSqlite::class,
+        DoctrinePDOMySqlDriver::class => DatabaseCheckDriverPdoMysql::class,
+        DoctrinePDOPgSqlDriver::class => DatabaseCheckDriverPDOPgSql::class,
+        DoctrinePDOSqliteDriver::class => DatabaseCheckDriverPDOSqlite::class,
     ];
 
     public function __construct()
@@ -132,8 +120,6 @@ class DatabaseCheck implements CheckInterface
 
     /**
      * Get status of each database platform identified to be installed on the system
-     *
-     * @return FlashMessageQueue
      */
     public function getStatus(): FlashMessageQueue
     {
@@ -148,7 +134,7 @@ class DatabaseCheck implements CheckInterface
                     new FlashMessage(
                         '',
                         $exception->getMessage(),
-                        FlashMessage::INFO
+                        ContextualFeedbackSeverity::INFO
                     )
                 );
             }
@@ -163,7 +149,7 @@ class DatabaseCheck implements CheckInterface
                     new FlashMessage(
                         '',
                         $exception->getMessage(),
-                        FlashMessage::INFO
+                        ContextualFeedbackSeverity::INFO
                     )
                 );
             }
@@ -176,7 +162,7 @@ class DatabaseCheck implements CheckInterface
     {
         if (!empty($this->databaseDriverCheckMap[$databaseDriver])) {
             /** @var CheckInterface $databaseDriverCheck */
-            $databaseDriverCheck = new $this->databaseDriverCheckMap[$databaseDriver];
+            $databaseDriverCheck = new $this->databaseDriverCheckMap[$databaseDriver]();
             foreach ($databaseDriverCheck->getStatus() as $message) {
                 $this->messageQueue->addMessage($message);
             }
@@ -196,23 +182,20 @@ class DatabaseCheck implements CheckInterface
     /**
      * Get the status of a specific database platform
      *
-     * @param string $databaseDriver
-     * @return FlashMessageQueue
-     *
      * @throws Exception
      */
     public function checkDatabasePlatformRequirements(string $databaseDriver): FlashMessageQueue
     {
         static $checkedPlatform = [];
-        $databasePlatformClass = static::$databaseDriverToPlatformMapping[$databaseDriver];
+        $databasePlatformClass = self::$databaseDriverToPlatformMapping[$databaseDriver];
 
         // execute platform checks only once
         if (in_array($databasePlatformClass, $checkedPlatform, true)) {
             return $this->messageQueue;
         }
 
-        if (!empty(static::$databaseDriverToPlatformMapping[$databaseDriver])) {
-            $platformMessageQueue = (new $databasePlatformClass)->getStatus();
+        if (!empty(self::$databaseDriverToPlatformMapping[$databaseDriver])) {
+            $platformMessageQueue = (new $databasePlatformClass())->getStatus();
             foreach ($platformMessageQueue as $message) {
                 $this->messageQueue->enqueue($message);
             }
@@ -239,39 +222,29 @@ class DatabaseCheck implements CheckInterface
         }
 
         if (static::isPdoMysql()) {
-            $installedDrivers[] = TYPO3PDOMySqlDriver::class;
+            $installedDrivers[] = DoctrinePDOMySqlDriver::class;
         }
 
         if (static::isPdoPgsql()) {
-            $installedDrivers[] = TYPO3PDOPgSqlDriver::class;
+            $installedDrivers[] = DoctrinePDOPgSqlDriver::class;
         }
 
         if (static::isPdoSqlite()) {
-            $installedDrivers[] = TYPO3PDOSqliteDriver::class;
-        }
-
-        if (static::isPdoSqlSrv()) {
-            $installedDrivers[] = TYPO3PDOSqlSrvDriver::class;
-        }
-
-        if (static::isSqlSrv()) {
-            $installedDrivers[] = Driver\SQLSrv\Driver::class;
+            $installedDrivers[] = DoctrinePDOSqliteDriver::class;
         }
 
         return $installedDrivers;
     }
 
     /**
-     * @param string $databaseDriverName
-     * @return string
-     *
      * @throws Exception
+     * @todo This method seems to be unused. Check if it can be removed or if it needs to be deprecated.
      */
     public static function retrieveDatabasePlatformByDriverName(string $databaseDriverName): string
     {
         $databaseDriverClassName = static::retrieveDatabaseDriverClassByDriverName($databaseDriverName);
-        if (!empty(static::$databaseDriverToPlatformMapping[$databaseDriverClassName])) {
-            return static::$databaseDriverToPlatformMapping[$databaseDriverClassName];
+        if (!empty(self::$databaseDriverToPlatformMapping[$databaseDriverClassName])) {
+            return self::$databaseDriverToPlatformMapping[$databaseDriverClassName];
         }
 
         throw new Exception(
@@ -281,14 +254,12 @@ class DatabaseCheck implements CheckInterface
     }
 
     /**
-     * @param string $driverName
-     * @return Driver
      * @throws Exception
      */
     public static function retrieveDatabaseDriverClassByDriverName(string $driverName): string
     {
-        if (!empty(static::$driverMap[$driverName])) {
-            return static::$driverMap[$driverName];
+        if (!empty(self::$driverMap[$driverName])) {
+            return self::$driverMap[$driverName];
         }
 
         throw new Exception(
@@ -321,14 +292,4 @@ class DatabaseCheck implements CheckInterface
     {
         return extension_loaded('pdo_sqlite');
     }
-
-    public static function isPdoSqlSrv(): bool
-    {
-        return extension_loaded('pdo_sqlsrv');
-    }
-
-    public static function isSqlSrv(): bool
-    {
-        return extension_loaded('sqlsrv');
-    }
 }
